/***************************************************************************
                          audio.cpp  -  description
                             -------------------
    begin                : Mon Jan 7 2002
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "audio.h"
#include <arts/soundserver.h>
#include <arts/dispatcher.h>
#include <arts/flowsystem.h>
#include <arts/connect.h>
#include <ksimpleconfig.h>
#include <kio/netaccess.h>
#include <qptrlist.h>
#include <qfile.h>
#include <qmap.h>

class audioPrivate
{
public:
    bool                          enabled;
    bool                          firstTime;
    QString                       theme;
    Arts::Dispatcher              dispatcher;
    Arts::SoundServerV2           server;
    Arts::PlayObjectFactory       playObjectFactory;
    QMap<int, Arts::PlayObject>   audioMap;
    QPtrList<KTempFile>           fileRef;
};

audio::audio()
{
    a = new audioPrivate;
    a->server = Arts::Reference("global:Arts_SoundServerV2");
    a->fileRef.setAutoDelete( true );
    if( a->server.isNull() )
    {
        a->enabled = false;
        kdWarning() << "audio::audio: Can not create Arts::SoundServerV2" << endl;
    }
    else
    {
        a->enabled = true;
        a->playObjectFactory = Arts::Reference("global:Arts_PlayObjectFactory");
    }
    a->firstTime = true;
}
audio::~audio()
{
    a->audioMap.clear();
    a->fileRef.clear();
    delete a;
}
void audio::play( const int snd )
{
    if( a->firstTime || !enabled || !a->enabled )
        return;

    QMap<int, Arts::PlayObject>::Iterator IT = a->audioMap.find( snd );

    if( IT == a->audioMap.end() )
    {
        return;
    }

    Arts::PlayObject* current = &IT.data();

    if( volume )
    {
        Arts::Synth_BUS_UPLINK uplink = Arts::DynamicCast( current->_getChild( "uplink" ) );
        uplink.stop();
        current->_node()->stop();
        Arts::disconnect( *current, "left", uplink, "left" );
        Arts::disconnect( *current, "right", uplink, "right" );
        Arts::StereoVolumeControl volumeControl = Arts::DynamicCast( a->server.createObject( "Arts::StereoVolumeControl" ) );
        current->_addChild( volumeControl, "volume" );
        uplink.start();
        volumeControl.start();
        current->_node()->start();
        Arts::connect( *current, "left", volumeControl, "inleft" );
        Arts::connect( *current, "right", volumeControl, "inright" );
        Arts::connect( volumeControl, "outleft", uplink, "left" );
        Arts::connect( volumeControl, "outright", uplink, "right" );
        volumeControl.scaleFactor( ( 100 - volume ) / 100.0 );
    }

    Arts::poTime time = current->currentTime();
    time.seconds = 0;
    time.ms = 0;
    current->seek( time );
    current->play();
}

void audio::setTheme( const QString &newTheme )
{
    QString configFile;
    KSimpleConfig *themeConfig;

    if( !enabled || !a->enabled )
        return;

    a->audioMap.clear();
    a->fileRef.clear();

    if( !KIO::NetAccess::download( newTheme + "theme.conf" , configFile, NULL ) )
    {
        kdWarning() << "audio::setTheme: Can not access theme.conf from " << newTheme << endl;
        return;
    }
    themeConfig = new KSimpleConfig( configFile, true );

    /* Read the sounds */
    themeConfig->setGroup( "Audio" );
    a->theme = newTheme;
    prepFile( a->theme + themeConfig->readEntry("Select"), Sound::Select );
    prepFile( a->theme + themeConfig->readEntry("Move"), Sound::Move );
    prepFile( a->theme + themeConfig->readEntry("Check"), Sound::Check );
    prepFile( a->theme + themeConfig->readEntry("MatchOver"), Sound::MatchOver );
    prepFile( a->theme + themeConfig->readEntry("Challenge"), Sound::Challenge );
    prepFile( a->theme + themeConfig->readEntry("Tell"), Sound::Tell );
    prepFile( a->theme + themeConfig->readEntry("Notification"), Sound::Notification );
    prepFile( a->theme + themeConfig->readEntry("DrawOffer"), Sound::DrawOffer );
    prepFile( a->theme + themeConfig->readEntry("Say"), Sound::Say );
    prepFile( a->theme + themeConfig->readEntry("Promote"), Sound::Promote );
    prepFile( a->theme + themeConfig->readEntry("Flag"), Sound::Flag );
    a->firstTime = false;

    delete themeConfig;
    KIO::NetAccess::removeTempFile( configFile );
}
void audio::prepFile( const QString &source, const int &ref )
{
    Arts::PlayObject player;
    KTempFile *dest;
    QString filename;
    if( source.isEmpty() )
        return;

    dest = new KTempFile( QString::null, source.right(4), 700 );
    dest->setAutoDelete( true );
    filename = dest->name();

    if( !KIO::NetAccess::download( source, filename, NULL ) )
    {
        kdWarning() << "audio::prepFile: Can not download " << source << endl;
        delete dest;
        return;
    }
    player = a->playObjectFactory.createPlayObject( QFile::encodeName( dest->name() ).data() );
    if( player.isNull() )
    {
        delete dest;
        return;
    }
    a->fileRef.append( dest );
    a->audioMap[ ref ] = player;
    return;
}
