/***************************************************************************
                          board_2d.h  -  description
                             -------------------
    begin                : Fri Feb 28 2003
    copyright            : (C) 2003 by The Knights Project
    email                : knights-general@lists.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BOARD_2D_H
#define BOARD_2D_H

#include <board_base.h>
#include <qpixmap.h>
#include <qptrlist.h>
#include <qlabel.h>
#include <kcombobox.h>

/**
  *@author The Knights Team
  */

class KnightsPixCache;

class board_2d : public board_base
{
    Q_OBJECT
public:
    board_2d(QWidget *parent=0, const char *name=0, resource *Rsrc=0, logic *Lgc=0);
    ~board_2d();
    virtual void        drawMove( const ChessMove &chessMove, const bool &reverse=FALSE );
    virtual void        setPremovePositions( const int&, const int& );
    virtual void        redrawLights( void );
    virtual void        setClocks( const QString&, const QString& );
    virtual void        setNotation( QStringList* list, int index=-1 );
    void                prevNotation( void );
    void                nextNotation( void );

    QSize               sizeHint( void ) const;
    void                paintEvent( QPaintEvent *event );
    void                resizeEvent( QResizeEvent* );
    void                mouseReleaseEvent( QMouseEvent *event );
    void                mousePressEvent( QMouseEvent *event );
    void                mouseMoveEvent( QMouseEvent *event );
    void                wheelEvent( QWheelEvent* );

public slots:
    virtual void        redrawAll( void );
    virtual void        drawPosition( const int& );

protected:

    /** Use sprites to track objects in motion */
    typedef struct Sprite
    {
        bool Restore;
        int Steps;
        int POSITION_Origin;
        int POSITION_Destination;
        int POSITION_TargetTaken;
        QPoint POINT_Origin;
        QPoint POINT_Current;
        QPoint POINT_LastUpdate;
        QPoint POINT_Destination;
        QPixmap PIXMAP_Sprite;
        QPixmap PIXMAP_FlipFrame;
    };

    /* These define various visual effects */
    enum EffectType
    {
        HighlightFadeUp,
        HighlightFadeDown,
        ChessmanFadeUp,
        ChessmanFadeDown
    };

    /* Use this to track visual effects in progress */
    typedef struct Effect
    {
        int Position;
        EffectType Type;
        float Fade_Start;
        float Fade_Stop;
        float Fade_Step;
        float Fade_Current;
    };

    typedef QPtrList<Sprite> SpriteList;
    typedef QPtrList<Effect> EffectList;

    void                drawPosition( const int &pos, const float &highlightAlpha );
    void                drawSprites( void );
    void                drawVisuals( const int &pos, const EffectType &et );
    void                drawCoords( QPixmap *pic, const int &pos );
    void                drawCount( QPixmap *pic, const int &num );
    QPoint              coords( const int&, const int&);
    int                 position( const QPoint& );
    QPixmap             getChessman( const int &pos );
    bool                isSprite( const int &pos );
    void                myBlit( const QPoint &dp, const QPaintDevice *src, const QRect &sr );

protected slots:
    void                updateVisuals( void );
    void                themeChange( void );

private:
    SpriteList          sprites; // List of all sprites
    EffectList          effects; // List of all effects
    Sprite              *DragSprite; // Pointer to sprite being dragged by cursor
    KnightsPixCache     *cache; // Pointer to app-wide cache
    QPoint              pressPoint;

    QPixmap             drawBuffer; // Draw Everything to this
    QRect               rectBoard; // rectangle containing the Board
    QRect               rectWSide; // rectangle containing the Left Sideboard
    QRect               rectBSide; // rectangle containing the Right Sideboard

    int                 squareSize;
    int                 sideSquareSize;
    int                 boardSize;
    bool                orientation;
    bool                lastMoveWasDrag;
    bool                usingSideboard;
    bool                init;

    QLabel              *whiteTimeLabel;
    QLabel              *blackTimeLabel;
    QLabel              *messageBar;
    KComboBox           *notationBar;

    /* Used to define updated portion of the board */
    QRegion             updateRegion;

    /* Premove Positions */
    int                 premoveFrom;
    int                 premoveTo;
};

#endif
