/***************************************************************************
                          board_base.h  -  description
                             -------------------
    begin                : Fri Feb 28 2003
    copyright            : (C) 2003 by The Knights Project
    email                : knights-general@lists.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BOARD_BASE_H
#define BOARD_BASE_H

#include <stdlib.h>
#include <qwidget.h>
#include <qstringlist.h>
#include "definitions.h"
#include "command.h"

/**
  *@author The Knights Project
  */

class resource;
class logic;
class KPopupMenu;

class board_base : public QWidget
{
    Q_OBJECT

public:
    board_base(QWidget *parent=0, const char *name=0, resource *Rsrc=0, logic *Lgc=0);
    virtual             ~board_base();
    virtual void        drawMove( const ChessMove &chessMove, const bool &reverse=FALSE ) = 0;
    virtual void        setPremovePositions( const int&, const int& );
    virtual void        redrawLights( void );
    virtual void        setClocks( const QString&, const QString& ) = 0;
    virtual void        setNotation( QStringList* list, int index=-1 );
    void                setMenu( KPopupMenu* );

public slots:
    virtual void        recvCMD( const Command& );
    virtual void        redrawAll( void ) = 0;
    virtual void        setPaused( const bool& );
    virtual void        setLocalArmy( const bool& );
    virtual void        drawPosition( const int& );
    void                setStatusMessage( const int &ID, const QString& MSG=QString::null );
    void                requestWidgetFeatureList( void ); // For the Tab System

signals: // These SIGNALS are meant for 'match'
    void  leftClick( int );
    void  rightClick( int );
    void  statusbarText( const QString& );
    void  requestReview( int );
    void  internalReqWFL( void );

signals: // These SIGNALS are meant for the Tab system
    void  updateWidgetFeatureList( const WFL& );
    void  requestClose( bool& );
    void  sendCMD( const Command& );
    void  fetchMenu( KPopupMenu*, QString& );

protected:
    resource            *myResource;
    logic               *myLogic;
    KPopupMenu          *myMenu;
    bool                paused;
    bool                localArmy;
    bool                flip;

protected:
    inline bool         color( const int &rank, const int &file )
    {
        return abs( ( rank % 2 ) - ( file % 2 ) );
    }
};

#endif
