/***************************************************************************
                          console.cpp  -  description
                             -------------------
    begin                : Sun May 27 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "console.moc"
#include "knightstextview.h"
#include "accel.h"
#include "tabmanager.h"
#include "dlg_settings.h"
#include "knightspixcache.h"
#include "core.h"

/* KDE */
#include <klineedit.h>
#include <kiconloader.h>
#include <kpopupmenu.h>
#include <qregexp.h>
#include <qstyle.h>

Console::Console(QWidget *parent,
                 const char *name,
                 resource *Rsrc ) : QVBox(parent,name)
{
    myResource = Rsrc;
    SelfMoving = false;

    margin = QApplication::style().pixelMetric( QStyle::PM_DefaultFrameWidth );
    myTextView = new KnightsTextView( this, myResource );
    myLineEdit = new KLineEdit( this, "console_edit" );
    myName = name;

    /* Configure misc */
    setBG();
    myTextView->setStaticBackground( false );

    /* Setup Keyboard Shortcuts */
    connect( myLineEdit, SIGNAL( returnPressed( const QString& ) ), this, SLOT( getText( const QString& ) ) );
    connect( myResource->myAccel, SIGNAL( history_prev() ), this, SLOT( historyBackward() ) );
    connect( myResource->myAccel, SIGNAL( history_next() ), this, SLOT( historyForward() ) );
    connect( myResource->myAccel, SIGNAL( page_up() ), this, SLOT( pageUp() ) );
    connect( myResource->myAccel, SIGNAL( page_down() ), this, SLOT( pageDown() ) );
    connect( myResource->myAccel, SIGNAL( kibitz() ), this, SLOT( kibitz() ) );
    connect( myResource->myAccel, SIGNAL( whisper() ), this, SLOT( whisper() ) );
    connect( myResource->myAccel, SIGNAL( reply_tell() ), this, SLOT( replyPrivate() ) );
    connect( myResource->myAccel, SIGNAL( reply_channel() ), this, SLOT( replyChannel() ) );
    connect( myResource->myAccel, SIGNAL( focus() ), this, SLOT( getFocus() ) );
    connect( myResource->myAccel, SIGNAL( focus( const QChar& ) ), this, SLOT( getFocus( const QChar& ) ) );

    /* Populate Widget Feature List */
    myFeatureList.EnableSave = false;
    myFeatureList.EnableSaveAs = false;
    myFeatureList.EnablePrint = false;
    myFeatureList.ContextMenu = true;
    myFeatureList.ReqPermissionForClose = false;

    /*
      Configure functions in the View right click menu
    */
    myMenu = new KPopupMenu( this );
    myMenu->insertItem( QIconSet( Rsrc->LoadIcon( QString("editcopy"), KIcon::Small ) ),
                        i18n( "&Copy" ), myTextView, SLOT( menuFunct(int) ), CTRL+Key_C, MENU_COPY );
    myMenu->insertItem( QIconSet( Rsrc->LoadIcon( QString("editpaste"), KIcon::Small ) ),
                        i18n( "Paste" ), this, SLOT( menuFunct(int) ), CTRL+Key_V, MENU_PASTE );
    myMenu->insertItem( i18n("Select &All"), myTextView, SLOT( menuFunct(int) ), CTRL+Key_A, MENU_SELECT_ALL );
    myMenu->insertSeparator();
    myMenu->insertItem( QIconSet( Rsrc->LoadIcon( QString("fileprint"), KIcon::Small ) ),
                        i18n( "&Print" ), myTextView, SLOT( menuFunct(int) ), CTRL+Key_P, MENU_PRINT );
    //  myMenu->insertItem( QIconSet( Rsrc->LoadIcon( QString("find"), KIcon::Small ) ),
    //                        i18n( "&Find" ), this, SLOT( menuFunct(int) ), CTRL+Key_F, MENU_FIND );
    //  myMenu->insertItem( QIconSet( Rsrc->LoadIcon( QString("next"), KIcon::Small ) ),
    //                        i18n( "Find &Next" ), this, SLOT( menuFunct(int) ), Key_F3, MENU_FIND_NEXT );
    //  myMenu->insertSeparator();
    myMenu->insertSeparator();
    myMenu->insertItem( QIconSet( Rsrc->LoadIcon( QString("viewmag+"), KIcon::Small ) ),
                        i18n( "Zoom &In" ), myTextView, SLOT( menuFunct(int) ), 0, MENU_ZOOM_IN );
    myMenu->insertItem( QIconSet( Rsrc->LoadIcon( QString("viewmag-"), KIcon::Small ) ),
                        i18n( "Zoom &Out" ), myTextView, SLOT( menuFunct(int) ), 0, MENU_ZOOM_OUT );
    myMenu->insertSeparator();
    myMenu->insertItem( QIconSet( Rsrc->LoadIcon( QString("configure"), KIcon::Small ) ),
                        i18n( "Configure Console..." ), this, SLOT( menuFunct(int) ), 0, MENU_SETTINGS_DIALOG );

    myTextView->show();
    myLineEdit->show();
    show();
}
Console::~Console()
{
    delete myLineEdit;
    delete myTextView;
    delete myMenu;
}
///////////////////////////////////////
//
//      Console::pageDown
//
///////////////////////////////////////
void Console::pageDown( void )
{
    myTextView->pageMove( Qt::Key_PageDown );
}
///////////////////////////////////////
//
//      Console::pageUp
//
///////////////////////////////////////
void Console::pageUp( void )
{
    myTextView->pageMove();
}
///////////////////////////////////////
//
//      Console::recvCMD
//
///////////////////////////////////////
void Console::recvCMD( const Command& command )
{
    switch(command.getCommand())
    {
    case CMD_Append_To_Console:
        appendText( command.getData() );
        break;
    case CMD_Set_Input:
        myLineEdit ->setText( command.getData() );
        if( !myLineEdit->hasFocus() )
            getFocus();
        break;
    case CMD_Set_Src_Tell:
        lastPrivateSource = command.getData();
        break;
    case CMD_Set_Src_Channel:
        lastChannelSource = command.getData();
        break;
    default:
        break;
    }
}
///////////////////////////////////////
//
//      Console::getText
//
///////////////////////////////////////
void Console::getText( const QString &text )
{
    /* Append this text into our history */
    history.append( text );
    if( history.count() > 100 )
        history.remove( history.begin() );
    historyIT = history.end();
    emit sendCMD( Command( 0, CMD_Send_To_ICS, text ) );
    myLineEdit->clear();
}
///////////////////////////////////////
//
//      Console::historyBack
//
///////////////////////////////////////
void Console::historyBackward( void )
{
    if( history.count() && ( historyIT != history.begin() ) )
    {
        historyIT--;
        myLineEdit->setText( (*historyIT) );
    }
    if( !myLineEdit->hasFocus() )
        getFocus();
}
///////////////////////////////////////
//
//      Console::historyForward
//
///////////////////////////////////////
void Console::historyForward( void )
{
    if( history.count() && ( historyIT != history.end() ) )
    {
        historyIT++;
        myLineEdit->setText( (*historyIT) );
    }
    if( !myLineEdit->hasFocus() )
        getFocus();
}
///////////////////////////////////////
//
//      Console::replyPrivate
//
///////////////////////////////////////
void Console::replyPrivate( void )
{
    if(NULL != lastPrivateSource && lastPrivateSource != "")
    {
        myLineEdit->setText( QString( "tell %1 " ).arg( lastPrivateSource ) );
    }
    if(!myLineEdit->hasFocus())
    {
        getFocus();
    }
}
///////////////////////////////////////
//
//      Console::replyChannel
//
///////////////////////////////////////
void Console::replyChannel( void )
{
    myLineEdit->setText( QString( "tell %1 " ).arg( lastChannelSource ) );
    if( !myLineEdit->hasFocus() )
        getFocus();
}
///////////////////////////////////////
//
//      Console::kibitz
//
///////////////////////////////////////
void Console::kibitz( void )
{
    myLineEdit->setText( QString( "kibitz " ) );
    if( !myLineEdit->hasFocus() )
        getFocus();
}
///////////////////////////////////////
//
//      Console::whisper
//
///////////////////////////////////////
void Console::whisper( void )
{
    myLineEdit->setText( QString( "whisper " ) );
    if( !myLineEdit->hasFocus() )
        getFocus();
}
///////////////////////////////////////
//
//      Console::getFocus
//
///////////////////////////////////////
void Console::getFocus( void )
{
    if( myLineEdit->hasFocus() )
    {
        getText( myLineEdit->text() );
    }
    setActiveWindow();
    myResource->tabManager->showTab( this );
    myLineEdit->setFocus();
}
void Console::getFocus( const QChar &c )
{
    myLineEdit->setText( myLineEdit->text() + QString( c ) );
    if( !myLineEdit->hasFocus() )
        getFocus();
}
///////////////////////////////////////
//
//      Console::setBG
//
///////////////////////////////////////
void Console::setBG( void )
{
    myTextView->setPaper( QBrush( myResource->COLOR_Background ) );
}

///////////////////////////////////////
//
//      Console::appendText( const QString& )
//
///////////////////////////////////////
void Console::appendText( const QString &data )
{
  QString myText = insertTags( data );

  /* Apply Filters */
  if( myText.startsWith( "<K_STD" ) && ( myFilterRules.at(0) == '0' ) )
    return;
  if( myText.startsWith( "<K_PVT" ) && ( myFilterRules.at(1) == '0' ) )
    return;
  if( myText.startsWith( "<K_SHT" ) && ( myFilterRules.at(2) == '0' ) )
    return;
  if( myText.startsWith( "<K_WSP" ) && ( myFilterRules.at(3) == '0' ) )
    return;
  if( myText.startsWith( "<K_NOT" ) && ( myFilterRules.at(4) == '0' ) )
    return;
  if( myText.startsWith( "<K_CH" ) && ( myFilterRules.at( 4 + lastChannelSource.toInt() ) == '0' ) )
    return;

  /* To scroll or not to scroll, that is the question */
  if( myTextView->contentsY() >= ( myTextView->contentsHeight() - myTextView->visibleHeight() - 12 ) )
  {
    myTextView->append( myText );
    myTextView->scrollToBottom();
  }
  else
  {
    myTextView->append( myText );
  }
}

///////////////////////////////////////
//
//      Console::insertTags(QString data)
//
///////////////////////////////////////
QString Console::insertTags(QString data)
{
    QRegExp hyperLinks("(http://(?:.)+(?:\\.(?:[^\\s\\r\\t<])+)+)");
    QRegExp mailLinks("\\b((?:\\w|\\d)+\\@(?:\\w|\\d)+(?:\\.(?:\\d|\\w)+)+)\\b");
    QRegExp ftpLinks("(ftp://(?:.)+(?:\\.(?:[^\\s\\r\\t<])+)+)");
    int pos = 0;

    pos = hyperLinks.search(data, 0);
    if(pos >= 0)
    {
        data.replace(hyperLinks, "<A HREF=\"" + hyperLinks.cap(0) + "\">" + hyperLinks.cap(0) + "</A>");
    }
    pos = mailLinks.search(data, 0);
    if(pos >= 0)
    {
        data.replace(mailLinks, "<A HREF=\"mailto:" + mailLinks.cap(0) + "\">" + mailLinks.cap(0) + "</A>");
    }
    pos = ftpLinks.search(data, 0);
    if(pos >= 0)
    {
        data.replace(ftpLinks, "<A HREF=\"" + ftpLinks.cap(0) + "\">" + ftpLinks.cap(0) + "</A>");
    }
    return data;
}
///////////////////////////////////////
//
//  Console::requestWidgetFeatureList
//
///////////////////////////////////////
void Console::requestWidgetFeatureList( void )
{
    emit updateWidgetFeatureList( myFeatureList );

    QString caption = i18n( "&Console" );
    emit fetchMenu( myMenu, caption );
}
///////////////////////////////////////
//
//      Console::menuFunct
//
///////////////////////////////////////
void Console::menuFunct( int funct )
{
  switch( funct )
  {
    case MENU_PASTE:
      myLineEdit->paste();
      break;
    case MENU_SETTINGS_DIALOG:
      mySettingsDialog = new dlg_settings( this, "settings", myResource );
      connect( mySettingsDialog, SIGNAL( redrawBoard() ), myResource->pixCache, SLOT( resetCache() ) );
      connect( mySettingsDialog, SIGNAL( resetServer() ), myResource->Core, SLOT( resetServer() ) );

      mySettingsDialog->show();
      break;
    default:
      break;
  }
}
