/***************************************************************************
                          console.h  -  description
                             -------------------
    begin                : Sun May 27 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONSOLE_H
#define CONSOLE_H

/* Qt */
#include <qstringlist.h>
#include <qvbox.h>
#include <qlayout.h>
/* Local */
#include "definitions.h"
#include "resource.h"
#include "command.h"

/**
  *@author Troy Corbin Jr.
  */

class KLineEdit;
class KnightsTextView;
class KPopupMenu;
class dlg_settings;

class Console : public QVBox
{
    Q_OBJECT

public:

    enum Direction
    {
        Up      = 1,
        Down    = 0
    };

    Console(QWidget *parent=0, const char *name=0, resource *Rsrc=0);
    ~Console();
    void                buildAccel( void );

public slots:
    void                recvCMD( const Command& );
    void                getText( const QString& );
    void                menuFunct( int );
    void                historyBackward( void );
    void                historyForward( void );
    void                pageUp( void );
    void                pageDown( void );
    void                getFocus( void );
    void                getFocus( const QChar& );
    void                setBG( void );
    void                replyPrivate( void );
    void                replyChannel( void );
    void                kibitz( void );
    void                whisper( void );

    void                requestWidgetFeatureList( void ); /* For The Tab System */

signals:
    void                sendCMD( const Command& );
    void                updateWidgetFeatureList( const WFL& ); /* For The Tab System */
    void                fetchMenu( KPopupMenu*, QString& ); /* For The Tab System */

private:
    QString             insertTags( QString );
    void                appendText( const QString& );

    resource            *myResource;
    dlg_settings        *mySettingsDialog;
    int                 margin;
    bool                SelfMoving;
    WFL                 myFeatureList;

    KnightsTextView     *myTextView;
    KLineEdit           *myLineEdit;
    KPopupMenu          *myMenu;

    QStringList           history;
    QStringList::Iterator historyIT;
    QString               lastPrivateSource;
    QString               lastChannelSource;
    QString               myName; // QString version for easy reference
    QString               myFilterRules;
};

#endif
