/***************************************************************************
                          core.cpp  -  description
                             -------------------
    begin                : Tue Oct 2 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* KDE */
#include <kmessagebox.h>

/* Qt */
#include <qpushbutton.h>

/* Local */
#include "core.moc"
#include "match.h"
#include "io_engine.h"
#include "io_internet.h"
#include "match_param.h"

/* Tabs */
#include "tabmanager.h"

///////////////////////////////////////
//
//      core::core
//
///////////////////////////////////////
core::core(QObject *parent, const char *name, resource *Rsrc ) : QObject(parent,name)
{
    myResource = Rsrc;
    currentMatch = NULL;
    matchIndex = 0;
    myCoreClock.start( 100, false );
}

///////////////////////////////////////
//
//      core::~core
//
///////////////////////////////////////
core::~core()
{
    myCoreClock.stop();
}

///////////////////////////////////////
//
//      core::createNewIO
//
///////////////////////////////////////
void core::createNewIO( int type, int ID, int side, int str )
{
    QStringList::Iterator IT;
    io_internet *internetioPtr = NULL;
    io_engine *engineioPtr = NULL;
    QWidget *tabPtr = NULL;

    switch( type )
    {
        /* PLAYERTCP */
    case PLAYERTCP:
        if( ID == Null )
        {
            /* Create a NEW internetIO */
            internetioPtr = new io_internet( this, myResource );
            myIOMap.add( ID, internetioPtr );
            internetioPtr->setID( ID );
            connect(((io_internet*)internetioPtr), SIGNAL(sendCMD(const Command&)), this, SLOT(command2Match(const Command&)));
            connect(this, SIGNAL(showConnect(void)), ((io_internet*)internetioPtr), SLOT(showConnectDialog(void)));

            /*
                Recreate the consoles that were last open. Every other string in ICS_Console_Filters is the name
                of a new console we need to open.
            */
            for( IT = myResource->ICS_Console_Filters.begin(); IT != myResource->ICS_Console_Filters.end(); IT++ )
            {
              tabPtr = new Console( 0, (*IT).ascii(), myResource );
              myTabMap.add( ID, tabPtr );
              myResource->tabManager->addTab( tabPtr, (*IT) );
              connect( tabPtr, SIGNAL( sendCMD( const Command& ) ), internetioPtr, SLOT( recvCMD( const Command& ) ) );
              connect( internetioPtr, SIGNAL( sendCMD( const Command& ) ), tabPtr, SLOT( recvCMD( const Command& ) ) );
              IT++;
            }

            /*
               Create ICS Related Tabs
            */
            createNewTab( MENU_NEW_SEEK_LIST );

            myResource->tabManager->showTab( myTabMap.find(ID) );
            emit(showConnect());
        }
        else
        {
            /* Use an existing internetIO */
            internetioPtr = (io_internet*)myIOMap.find(Null);
            if( internetioPtr == NULL )
            {
                kdWarning() << "core::createNewIO: Trying to connect a match to an internetIO that doesn't exsist." << endl;
            }
            else
            {
                myIOMap.add( ID, internetioPtr );
            }
        }
        break;
        /* PLAYEREXAMINE */
    case PLAYEREXAMINE:
        /* examine mode needs only 1 io mapped to it */
        if( ID == Null )
        {
            /* Create a NEW internetIO */
            internetioPtr = new io_internet( this, myResource );
            myIOMap.add( ID, internetioPtr );
            internetioPtr->setID( ID );
            connect( internetioPtr, SIGNAL( sendCMD( const Command& ) ), this, SLOT( command2Match( const Command& ) ) );
        }
        else
        {
            internetioPtr = (io_internet*)myIOMap.find(ID);
            if( internetioPtr == NULL )
            {
                /* no io mapped to the ID yet, use the internetio */
                /* Use an existing internetIO */
                internetioPtr = (io_internet*)myIOMap.find(Null);
                if( internetioPtr == NULL )
                {
                    kdWarning() << "core::createNewIO: Trying to connect a match to an internetIO that doesn't exsist." << endl;
                }
                else
                {
                    myIOMap.add( ID, internetioPtr );
                }
            }
            /* this ID is already mapped we don't need another one */
        }
        break;
        /* PLAYERPC */
    case PLAYERPC:
        engineioPtr = new io_engine( 0, myResource );
        myIOMap.add( ID, engineioPtr );
        engineioPtr->setID( ID );
        connect( engineioPtr, SIGNAL( sendCMD( const Command& ) ), this, SLOT( command2Match( const Command& ) ) );
        engineioPtr->Start( side );
        engineioPtr->recvCMD( Command( ID, CMD_Set_Difficulty, QString().setNum( str ) ) );
        break;
        /* PLAYEREMAIL */
    case PLAYEREMAIL: // Fall Through
    default:
        /* No IO needed */
        break;
    }
}

///////////////////////////////////////
//
//      core::newMatch(match_params* param);
//
///////////////////////////////////////
match* core::newMatch( match_param* param )
{
    match *match_ptr = NULL;
    int newID = ++matchIndex;

    /* Create the match */
    match_ptr = new match( this, param, myResource);
    match_ptr->setID( newID );
    myMatchMap.add( newID, match_ptr );

    /* Create IOs */
    createNewIO( param->type(WHITE), newID, WHITE, param->strength(WHITE) );
    createNewIO( param->type(BLACK), newID, BLACK, param->strength(BLACK) );
    /* Connect Signals */
    connect( match_ptr, SIGNAL( sendCMD( const Command& ) ), this, SLOT( command2IO( const Command& ) ) );
    connect( &myCoreClock, SIGNAL( timeout() ), match_ptr, SLOT( tick() ) );

    /* We may need to close some matches now */
    if( myResource->OPTION_Auto_Close_Last_ICS )
    {
        if( ( param->type(WHITE) == PLAYERTCP ) || ( param->type(BLACK) == PLAYERTCP ) )
        {
            closeICSMatches( match_ptr );
        }
    }

    /* Save our configuration settings */
    myResource->ConfigWrite();
    currentMatch = match_ptr;
    return match_ptr;
}

///////////////////////////////////////
//
//  core::setMatch
//
///////////////////////////////////////
void core::setMatch( int ID )
{
    /*
      Disconnect the current match from the display
    */
    if( currentMatch != NULL )
    {
        currentMatch->setCurrent( false );
    }
    /*
                Now display the new match, and connect it's signals to the GUI
    */
    currentMatch = myMatchMap.find( ID );
    currentMatch->setCurrent( true );
}
///////////////////////////////////////
//
//      core::clearMatch
//
///////////////////////////////////////
void core::clearMatch( const int &id )
{
    match *departingMatch;
    io_base *ioPtr;

    departingMatch = myMatchMap.find(id);

    /* A bad ID was passed */
    if( departingMatch == NULL )
    {
        kdError() << "core::clearMatch: Bad matchID " << id << " could not be cleared." << endl;
        return;
    }

    /* Remove IOs in this match... */
    ioPtr = myIOMap.find(id);
    while( ioPtr != NULL )
    {
        myIOMap.remove( id );
        if( ioPtr->getType() != io_base::INTERNET )
        {
            delete ioPtr;
        }
        ioPtr = myIOMap.find(id);
    }

    /*
        ...and remove the match. Even though we created the match,
        we can not destroy it because the match called this method!
        The Tab system will destroy it for us.
    */
    myMatchMap.remove( id );
    disconnect( &myCoreClock, SIGNAL( timeout() ), departingMatch, SLOT( tick() ) );
    disconnect( departingMatch, SIGNAL( sendCMD( const Command& ) ), this, SLOT( command2IO( const Command& ) ) );
    return;
}
///////////////////////////////////////
//
//      core::command2IO
//
///////////////////////////////////////
void core::command2IO( const Command &command )
{
    io_base *io;

    io = myIOMap.find( command.getID() );
    while( io != NULL )
    {
        io->recvCMD(command);
        io = myIOMap.findNext();
    }
}
///////////////////////////////////////
//
//      core::command2Match
//
///////////////////////////////////////
void core::command2Match( const Command &command )
{
    QCustomEvent *e;
    match_param *paramPtr;
    match *matchPtr;

    matchPtr = myMatchMap.find( command.getID() );
    while( matchPtr != NULL )
    {
        /* Catch Commands Meant for Core */
        switch( command.getCommand() )
        {
        case CMD_Tell_Opponent:
            paramPtr = matchPtr->getParam();
            if( ( paramPtr->type( WHITE ) == PLAYERLOCAL ) ||
                    ( paramPtr->type( BLACK ) == PLAYERLOCAL ) )
            {
                KMessageBox::information( 0, command.getData(), i18n( "Tell Opponent" ) );
            }
            else
            {
                command2IO( command );
            }
            break;
        case CMD_Tell_Others:
            command2IO( command );
            break;
        case CMD_Tell_ICS:
            command2IO( command );
            break;
        case CMD_Tell_All:
            paramPtr = matchPtr->getParam();
            if( ( paramPtr->type( WHITE ) == PLAYERLOCAL ) ||
                    ( paramPtr->type( BLACK ) == PLAYERLOCAL ) )
            {
                KMessageBox::information( 0, command.getData(), i18n( "Tell All" ) );
            }
            command2IO( command );
            break;
        case CMD_Out_Of_Book:
            //                  engine[enginePtr - 1]->sendCMD( CMD_Check_Book );
            break;
        case CMD_Close_Match:
            /*
                Match must be killed from the tab system to ensure
                that it's GUI is properly deleted
            */
            e = new QCustomEvent( EVENT_Close_Tab );
            e->setData( matchPtr->getBoard() );
            QApplication::sendEvent( myResource->tabManager, e );
            delete e;
            break;
        default:
            /* Just send it to the Match */
            matchPtr->recvCMD( command );
            break;
        }
        matchPtr = myMatchMap.findNext();
    }
}
///////////////////////////////////////
//
//      core::isOnline
//
///////////////////////////////////////
bool core::isOnline( void )
{
    unsigned int index;
    for( index=0; index < myIOMap.count(); index++ )
    {
        if( myIOMap[index]->getType() == io_base::INTERNET )
        {
            return true;
        }
    }
    return false;
}

///////////////////////////////////////
//
//      core::onMove
//
///////////////////////////////////////
char core::onMove( void )
{
    if( currentMatch == NULL )
        return Null;
    return currentMatch->onMove();
}
///////////////////////////////////////
//
//  core::resetServer
//
///////////////////////////////////////
void core::resetServer( void )
{
    command2IO( Command( Null, CMD_Reset_Server ) );
}

///////////////////////////////////////
//
//  core::goOffline()
//
///////////////////////////////////////
void core::goOffline(void)
{
    unsigned int index(0);
    io_base *ioPtr(NULL);
    QWidget *tabPtr(NULL);

    while( index < myIOMap.count() )
    {
        if( myIOMap[index]->getType() == io_base::INTERNET )
        {
            ioPtr = myIOMap[index];
            if( myIOMap.keyAt(index) == Null )
            {
                myIOMap.remove( Null );
            }
        }
        index++;
    }

    /* Now close all ICS Matches */
    closeICSMatches();

    /* CLose all ICS related Tabs */
    for( index = 0; index < myTabMap.count(); index++ )
    {
        tabPtr = myTabMap[index];
        myResource->tabManager->removeTab( tabPtr );
    }
    myTabMap.clear();

    if( ioPtr != NULL )
    {
        delete ioPtr;
    }
}
///////////////////////////////////////
//
//  core::event
//
///////////////////////////////////////
bool core::event( QEvent *e )
{
    if( e->type() == EVENT_Del_IO_Net )
    {
        goOffline();
        myResource->tabManager->refreshMenus();
        return true;
    }
    return false;
}
///////////////////////////////////////
//
//  core::closeICSMatches()
//
///////////////////////////////////////
void core::closeICSMatches( const match *exception )
{
    unsigned int index(0);
    match *matchPtr(NULL);

    while( index < myIOMap.count() )
    {
        if( myIOMap[index]->getType() == io_base::INTERNET )
        {
            if( myIOMap.keyAt(index) != Null )
            {
                matchPtr = myMatchMap.find( myIOMap.keyAt(index) );
                if( matchPtr != exception )
                {
                    /*
                        Match must be killed from the tab system to ensure
                        that it's GUI is properly deleted
                    */
                    QCustomEvent e( EVENT_Close_Tab );
                    e.setData( matchPtr->getBoard() );
                    QApplication::sendEvent( myResource->tabManager, &e );
                    index--;
                }
            }
        }
        index++;
    }
}

void core::createNewTab(const int &type)
{
        QWidget *tab = 0;
        io_internet *io = (io_internet *) myIOMap.find(Null);

        switch(type)
        {
                case MENU_NEW_CONSOLE:
                        tab = new Console(0, "Console", myResource);
                        myTabMap.add(Null, tab);
                        myResource->tabManager->addTab(tab, i18n("Console"));
                        break;

                case MENU_NEW_SEEK_LIST:
                        tab = new SeekMatchTab(0, "seekList", myResource);
                        myTabMap.add(Null, tab);
                        myResource->tabManager->addTab(tab, i18n("Seek Match"));
                        break;

                case MENU_NEW_SEEK_GRAPH:
                        tab = new Challenge_Graph( 0, "seekGraph", myResource);
                        myTabMap.add(Null, tab);
                        myResource->tabManager->addTab(tab, i18n("Sought Matches Graph"));
                        break;

                default:
                        return;
        }

        connect(tab, SIGNAL(sendCMD(const Command &)), io,  SLOT(recvCMD(const Command &)));
        connect(io,  SIGNAL(sendCMD(const Command &)), tab, SLOT(recvCMD(const Command &)));
}

///////////////////////////////////////
//
//  core::load
//
///////////////////////////////////////
void core::load( const QString &url, const int &pos )
{
  newMatch( new match_param( myResource, PLAYERLOAD, PLAYERLOAD ) );
  currentMatch->load( url, pos );
}
