/***************************************************************************
                          core.h  -  description
                             -------------------
    begin                : Tue Oct 2 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CORE_H
#define CORE_H

#include <qobject.h>
#include <qtimer.h>
#include "definitions.h"
#include "resource.h"
#include "knightsmap.h"
#include "io_base.h"

class match;
class match_param;

/**
  *@author Troy Corbin Jr.

  Core is the link between the chess match and any non-GUI input sources.
  ( ie. engines, internet, email )
  */

class core : public QObject
{
    Q_OBJECT

public:

    typedef KnightsMap<int, match*> MatchMap;
    typedef KnightsMap<int, io_base*> IOMap;
    typedef KnightsMap<int, QWidget*> TabMap;

    core( QObject *parent=0, const char *name=0, resource *Rsrc=0 );
    ~core();
    bool                        isOnline( void );
    char                        onMove( void );
    void                        createNewIO( int type, int ID, int side=BLACK, int str=0 );
    void                        createNewTab( const int &type );
    void                        goOffline( void );
    bool                        event( QEvent *e );

public slots:
    match*                      newMatch( match_param* );
    void                        clearMatch( const int& );
    void                        setMatch( int );
    void                        command2IO( const Command& );
    void                        command2Match( const Command& );
    void                        resetServer( void );
    void                        load( const QString&, const int& );

signals:
    void                        serverDestroyed( void );
    void                        showConnect(void);

protected:
    bool                        findMatch( const int );
    void                        serverSelfDestruct( void );
    void                        closeICSMatches( const match* exception=NULL );

private:
    unsigned int                matchIndex;
    resource                    *myResource;
    QTimer                      myCoreClock;
    IOMap                       myIOMap;
    MatchMap                    myMatchMap;
    TabMap                      myTabMap;
    match                       *currentMatch;
};

#endif
