/***************************************************************************
                          definitions.h  -  description
                             -------------------
    begin                : Sun Jul 1 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef _DEFINITIONS_H
#define _DEFINITIONS_H

#ifndef TRUE
#  define TRUE 1
#  define FALSE 0
#endif

#ifndef _VERSION_
#  define _VERSION_ "0.6.4"
#endif

#define IMAGE_MAX 144
#define IMAGE_MIN 32

#define CONFIG_VERSION 2

#include <kdebug.h>
#include <qvaluelist.h>

/*
            These are used to define the game's states.
*/
const int WHITE            = 0x00;
const int BLACK            = 0x01;
const int WHITE_HELPER     = 0x02;
const int BLACK_HELPER     = 0x03;
const int BOTH             = 0x04;

const int PLAYERPC         = 0x00; // Chess Engine
const int PLAYEREMAIL      = 0x01; // Email Opponent
const int PLAYERTCP        = 0x02; // Internet Chess Engine
const int PLAYERLOAD       = 0x03; // Load from PGN

/* Add a mask of 0x10 to players that accept user input */
const int PLAYERLOCAL      = 0x10; // Human sitting at the PC
const int PLAYEREXAMINE    = 0x11; // ICS Examine Mode

const int ENGINE_WHITE     = 0x01;
const int ENGINE_WHITE_BK  = 0x02;
const int ENGINE_BLACK     = 0x04;
const int ENGINE_BLACK_BK  = 0x08;
const int ENGINE_TEST      = 0x10;

/* Menu Entries */

/*
      These represent the various choices found
      throughout the game's menubar.
*/

/* File Menu */
const int MENU_NEW_GAME         = 0x60000001;
const int MENU_LOAD             = 0x60000002;
const int MENU_SAVE             = 0x60000003;
const int MENU_SAVEAS           = 0x60000004;
const int MENU_PRINT            = 0x60000005;
const int MENU_QUIT             = 0x60000006;
const int MENU_CLOSE            = 0x60000008;
const int MENU_CLOSEALL         = 0x60000009;
const int MENU_CONNECT          = 0x6000000A;
const int MENU_NEW_CONSOLE      = 0x6000000B;
const int MENU_NEW_SEEK_LIST    = 0x6000000C;
const int MENU_NEW_SEEK_GRAPH   = 0x6000000D;
/* Options Menu */
const int MENU_BINDINGS_DIALOG  = 0x60000013;
const int MENU_SETTINGS_DIALOG  = 0x60000014;
const int MENU_INSTALL_THEMES   = 0x60000015;
/* New Games Menu */
const int MENU_SOLITARE     = 0x60000021;
const int MENU_VS_PC        = 0x60000022;
const int MENU_VS_TCPIP     = 0x60000023;
const int MENU_VS_EMAIL     = 0x60000024;
const int MENU_PC_VS_PC     = 0x60000025;
/* Match Menu */
const int MENU_RETRACT      = 0x60000031;
const int MENU_DRAW         = 0x60000032;
const int MENU_MOVE_NOW     = 0x60000033;
const int MENU_HINT         = 0x60000034;
const int MENU_PONDER       = 0x60000035;
const int MENU_PAUSE        = 0x60000036;
const int MENU_PAUSEALL     = 0x60000037;
const int MENU_ORIENTATION  = 0x60000038;
const int MENU_RESIGN       = 0x60000039;
const int MENU_CALL_FLAG    = 0x6000003A;
/* Draw Menu */
const int MENU_OFFER_DRAW   = 0x60000041;
const int MENU_ACCEPT_DRAW  = 0x60000042;
const int MENU_REJECT_DRAW  = 0x60000043;
const int MENU_IGNORE_DRAW  = 0x60000044;
/* Seek Menu */
const int MENU_SEEK         = 0x60000051;
const int MENU_FINGER       = 0x60000052;
const int MENU_TELL         = 0x60000053;
const int MENU_NOTIFY       = 0x60000054;
const int MENU_ACCEPT_MATCH = 0x60000055;
const int MENU_CENSOR       = 0x60000056;
const int MENU_HISTORY      = 0x60000057;
const int MENU_ASSESS       = 0x60000058;
/* View Menu */
const int MENU_COPY         = 0x60000061;
const int MENU_SELECT_ALL   = 0x60000062;
const int MENU_CLEAR_ALL    = 0x60000063;
const int MENU_FIND         = 0x60000064;
const int MENU_FIND_NEXT    = 0x60000065;
const int MENU_ZOOM_IN      = 0x60000066;
const int MENU_ZOOM_OUT     = 0x60000067;
const int MENU_PASTE        = 0x60000068;
/* Tutorial Menu */
const int MENU_OPEN_TUTORIAL= 0x60000071;
/* Window Menu */
const int MENU_SELECT_WINDOW= 0x60100000;
const int MENU_FULLSCREEN   = 0x60100001;

/* Messages */

/*
    The following constants define messages
      viewed in the statusbar.
*/
const int READY                 = 0x00000000;
const int WHITE_TURN            = 0x00000001;
const int BLACK_TURN            = 0x00000002;
const int WHITE_WIN             = 0x00000003;
const int BLACK_WIN             = 0x00000004;
const int GAME_DRAW             = 0x00000005;
const int GAME_50_MOVES         = 0x00000006;
const int WAITING               = 0x00000007;
const int ILLEGAL_MOVE          = 0x00000008;
const int NO_MOVE_WHILE_REVIEW  = 0x00000009;
const int LOAD_OK               = 0x00000010;
const int SAVE_OK               = 0x00000011;
const int PAUSED                = 0x00000012;
const int READING_FILE          = 0x00000013;
const int WHITE_DRAW_OFFER      = 0x00000014;
const int BLACK_DRAW_OFFER      = 0x00000015;
const int LOAD_ERROR            = 0x00000020;
const int SAVE_ERROR            = 0x00000021;
const int BOOK_ERROR_1          = 0x00000022;
const int BOOK_ERROR_2          = 0x00000023;
const int BOOK_ERROR_3          = 0x00000024;
const int BOOK_ERROR_4          = 0x00000025;
const int ENGINE_DIED_ERROR     = 0x00000026;
const int WHITE_RESIGN          = 0x00000027;
const int BLACK_RESIGN          = 0x00000028;
const int WHITE_FLAG            = 0x00000029;
const int BLACK_FLAG            = 0x00000030;
const int WHITE_CALL_FLAG       = 0x00000031;
const int BLACK_CALL_FLAG       = 0x00000032;
const int WHITE_CHECKMATE       = 0x00000033;
const int BLACK_CHECKMATE       = 0x00000034;
const int LOST_CONTACT          = 0x00000035;
const int COMMENT               = 0x00010000;
const int ERROR                 = 0x00100000;

/* Variations */
/*
        These define the variations on chess
        that can be supported.
*/
namespace GameType
{
const int Standard      = 0x00000001;
const int WildCastle    = 0x00000002;
const int NoCastle      = 0x00000004;
const int FischerRandom = 0x00000008;
const int BugHouse      = 0x00000010;
const int CrazyHouse    = 0x00000020;
const int Losers        = 0x00000040;
const int Suicide       = 0x00000080;
const int Giveaway      = 0x00000100;
const int TwoKings      = 0x00000200;
const int Atomic        = 0x00000400;
const int Kriegspiel    = 0x00000800;
const int ThreeCheck    = 0x00001000;
}

/* Protocols */

const int XBoard        = 0x02;
const int UCI           = 0x04;

/* Events */

/*
      These constants represent QEvent types.
*/

const int EVENT_Del_IO_Net    = 0x000003e9; // Shutdown io_network and delete
const int EVENT_Shutdown_App  = 0x000003ea; // Request shutdown of Knights
const int EVENT_Close_Tab     = 0x000003eb; // Request shutdown of a Tab


/* Position Notes */

/*
      These represent the various NOTES that can appear on a
      given position
*/

const char NOTE_NONE          = 0x00;
const char NOTE_HIGHLIGHT     = 0x01;
const char NOTE_SELECT        = 0x02;
const char NOTE_MOVE          = 0x10;
const char NOTE_CASTLE        = 0x11;
const char NOTE_PAWN_DOUBLE   = 0x12;
const char NOTE_ATTACK        = 0x20;
const char NOTE_ENPASSANT     = 0x21;

/* Chessmen */

/*
      These represent the various chessmen, or Types of chessmen
      found on the playing board.
*/
const char Null               = -1;
const char King               = 0;
const char Queen              = 1;
const char Bishop             = 2;
const char Knight             = 3;
const char Rook               = 4;
const char Pawn               = 5;

/* ICS Game Modes */

const char ICS_Normal         = 0;
const char ICS_Examine        = 1;
const char ICS_Observe        = 2;
const char ICS_Movelist       = 3;

typedef struct ThemeHeader
{
    QString name;
    QString version;
    QString author;
    QString authorEmail;
    QString authorWWW;
    QString notes;
};

typedef struct ChessMove
{
    signed char fromFile;
    signed char fromRank;
    signed char toFile;
    signed char toRank;
    signed char Promote;
    signed char ManTaken;
    char SAN[8];
    char CAN[8];
    unsigned char NAG;
    /* Following used for ICS Only */
    signed char ICS_Mode;
    signed char ICS_PawnPushFile;
    int  ICS_MoveCounter;
    bool ICS_OnMove;
    bool ICS_ClockTicking;
};

typedef struct Challenge
{
	QString whiteName;
	QString blackName;
	QString whiteRating;
	QString blackRating;
	int baseTime;
	int incTime;
	int oddsBaseTime;
	int oddsIncTime;
	bool rated;
	QString ICSTypeName;
	bool changed;
};

typedef struct Chessman
{
    bool Army;
    signed char Type;
    signed char File; // HACK ALERT! Used as Quantity for Sideboard Chessmen!
    signed char Rank;
};

typedef struct Position
{
    signed char File;
    signed char Rank;
    signed char ManPtr;
    signed char Note;
};

/* Time Control Period */
typedef struct TCP
{
    int Moves;
    int Seconds;
    int Increment;
};

typedef QValueList<TCP> TCPList;

/*
    Widget Feature List

    Used to notify the tab system of features to be activated
    based on the child widget's needs. Child must support:
        SIGNAL( updateWidgetFeatureList(const WFL&) )
        SLOT( requestWidgetFeatureList() )

      ContextMenu - If true, widget maintains a KPopupMenu to be placed on
                    the top menu. Widget is responsable for allocating and
                    deleting the menu, populating it, connecting to it's
                    SIGNALs, etc. Tab system only places the menu and
                    labels it based on the QString provided via fetchMenu.
                    Widget must support:
                        SIGNAL( fetchMenu( KPopupMenu*, QString& ) )

      ReqPermissionForClose - If true, widget is requesting notification
                              of it's demise so that it may cleanup. Widget
                              may also reject the close action by returning
                              a false value. Widget must support the following
                              SIGNAL, but you do not emit it! Instead, connect
                              it to your own internal SLOT and use the bool to
                              approve or reject the close.
                                SIGNAL( requestClose( bool& ) )

      EnableSave - If true, widget has saveable contents and wants the File->Save
                   option enabled. Widget must support the following to recieve
                   notification that the save option has been selected:
                       SLOT( recvCMD( const Command& ) )

      EnableSaveAs - Like above, this option enables the File->SaveAs menu item.
                     Widget must support the following to recieve notification
                     that the save option has been selected:
                       SLOT( recvCMD( const Command& ) )

      EnablePrint - Same as the Save values above, but for Print functionality.
                     Widget must support the following to recieve notification
                     that the print option has been selected:
                       SLOT( recvCMD( const Command& ) )
*/
typedef struct WFL
{
  bool ContextMenu;
  bool ReqPermissionForClose;
  bool EnableSave;
  bool EnableSaveAs;
  bool EnablePrint;
};

#endif
