/***************************************************************************
                          dlg_challenge.cpp  -  description
                             -------------------
    begin                : Sun Jan 20 2002
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qregexp.h>
#include "dlg_challenge.moc"

dlg_challenge::dlg_challenge(QWidget *parent, const char *name, resource *Rsrc ) :
        KDialogBase(parent, name, true, i18n("You've Been Challenged"), User1|User2, Ok,
                    true, i18n("Accept"), i18n("Decline") )
{
    myResource = Rsrc;

    BOX_Parent = makeVBoxMainWidget();

    BOX_Headline = new QHBox( BOX_Parent );
    LABEL_WhiteImage = new QLabel( BOX_Headline );
    LABEL_Headline = new QLabel( BOX_Headline );
    LABEL_BlackImage = new QLabel( BOX_Headline );
    COMBO_Rated = new KComboBox( BOX_Parent );
    BOX_Time = new QHBox( BOX_Parent );
    BOX_White = new QGroupBox( 2, Qt::Horizontal, "-", BOX_Time );
    BOX_WhiteBase = new QSpinBox( BOX_White );
    LABEL_WhiteBase = new QLabel( BOX_White );
    BOX_WhiteInc = new QSpinBox( BOX_White );
    LABEL_WhiteInc = new QLabel( BOX_White );
    BOX_Black = new QGroupBox( 2, Qt::Horizontal, "-", BOX_Time );
    BOX_BlackBase = new QSpinBox( BOX_Black );
    LABEL_BlackBase = new QLabel( BOX_Black );
    BOX_BlackInc = new QSpinBox( BOX_Black );
    LABEL_BlackInc = new QLabel( BOX_Black );
    BUTTON_TimeOdds = new QCheckBox( i18n("Time Odds Match"),BOX_Parent );

    LABEL_Headline->setAlignment( Qt::AlignCenter );
    LABEL_WhiteImage->setAlignment( Qt::AlignCenter );
    LABEL_BlackImage->setAlignment( Qt::AlignCenter );
    COMBO_Rated->setEditable( false );
    COMBO_Rated->insertItem( i18n("Unrated"), 0 );
    COMBO_Rated->insertItem( i18n("Rated"), 1 );

    BOX_WhiteBase->setMinValue( 0 );
    BOX_WhiteInc->setMinValue( 0 );
    BOX_WhiteBase->setMaxValue( 300 );
    BOX_WhiteInc->setMaxValue( 300 );
    BOX_WhiteBase->setSuffix( i18n(" min.") );
    BOX_WhiteInc->setSuffix( i18n(" sec.") );
    LABEL_WhiteBase->setText( i18n( "Base Time" ) );
    LABEL_WhiteInc->setText( i18n( "Increment" ) );

    BOX_BlackBase->setMinValue( 0 );
    BOX_BlackInc->setMinValue( 0 );
    BOX_BlackBase->setMaxValue( 300 );
    BOX_BlackInc->setMaxValue( 300 );
    BOX_BlackBase->setSuffix( i18n(" min.") );
    BOX_BlackInc->setSuffix( i18n(" sec.") );
    LABEL_BlackBase->setText( i18n( "Base Time" ) );
    LABEL_BlackInc->setText( i18n( "Increment" ) );

    showButton( User1, true );
    showButton( User2, true );
    enableButton( User1, true );
    enableButton( User2, true );
    connect(this, SIGNAL(user1Clicked()), this, SLOT(accept()));
		connect(this, SIGNAL(user2Clicked()), this, SLOT(decline()));
		connect(this, SIGNAL(user1Clicked()), this, SLOT(slotDelayedDestruct()));
		connect(this, SIGNAL(user2Clicked()), this, SLOT(slotDelayedDestruct()));
    show();
}

dlg_challenge::~dlg_challenge()
{
	// Clean up the Challenge pointer passed to us
	delete challenge;
}

void dlg_challenge::setValues( Challenge *chal )
{
	challenge = chal;
	LABEL_Headline->setText(i18n("%1 %2 vs. %3 %4\nin a %5 match.").arg( challenge->whiteName )
	                                                               .arg( challenge->whiteRating )
	                                                               .arg( challenge->blackName )
	                                                               .arg( challenge->blackRating )
	                                                               .arg( challenge->ICSTypeName ) );

	if( challenge->rated )
	{
		COMBO_Rated->setCurrentItem(1);
	}
	else
	{
		COMBO_Rated->setCurrentItem(0);
	}

	if( challenge->whiteRating == "----" || challenge->whiteRating == "++++" ||
	    challenge->blackRating == "----" || challenge->blackRating == "++++" )
	{
		COMBO_Rated->setEnabled(false);
	}

	/* Set Times */
	BOX_WhiteBase->setValue( challenge->baseTime );
	BOX_WhiteInc->setValue( challenge->incTime );
	BOX_BlackBase->setValue( challenge->oddsBaseTime );
	BOX_BlackInc->setValue( challenge->oddsIncTime );
	BOX_White->setTitle( i18n("Time Controls") );
	BOX_Black->setTitle( i18n("Time Controls") );

	if( challenge->oddsBaseTime )
	{
		BUTTON_TimeOdds->setChecked( true );
		BOX_Black->setEnabled( true );
	}
	else
	{
		BUTTON_TimeOdds->setChecked( false );
		BOX_Black->setEnabled( false );
	}

	/* Set Images */
	match_param *param = new match_param( myResource );
	param->setName( WHITE, challenge->whiteName );
	param->setName( BLACK, challenge->blackName );
	LABEL_WhiteImage->setPixmap( param->image( WHITE ) );
	LABEL_BlackImage->setPixmap( param->image( BLACK ) );
	delete param;

	/* Make Connections */
	connect(COMBO_Rated, SIGNAL(activated(int)), this, SLOT(slot_changed(int)));
	connect(BOX_WhiteBase, SIGNAL(valueChanged(int)), this, SLOT(slot_changed(int)));
	connect(BOX_WhiteInc, SIGNAL(valueChanged(int)), this, SLOT(slot_changed(int)));
	connect(BOX_BlackBase, SIGNAL(valueChanged(int)), this, SLOT(slot_changed(int)));
	connect(BOX_BlackInc, SIGNAL(valueChanged(int)), this, SLOT(slot_changed(int)));
	connect(BUTTON_TimeOdds, SIGNAL(stateChanged(int)), this, SLOT(slot_changed(int)));
	connect(BUTTON_TimeOdds, SIGNAL(toggled(bool)), this, SLOT(slot_timeOdds(bool)));

	repaint();
}

void dlg_challenge::setValues( void )
{
	if( COMBO_Rated->currentItem() == 0 )
	{
		challenge->rated = false;
	}
	else
	{
		challenge->rated = true;
	}

	challenge->baseTime = BOX_WhiteBase->value();
	challenge->incTime = BOX_WhiteInc->value();

	if(BUTTON_TimeOdds->isChecked())
	{
		challenge->oddsBaseTime = BOX_BlackBase->value();
		challenge->oddsIncTime = BOX_BlackInc->value();
	}
	else
	{
		challenge->oddsBaseTime = 0;
		challenge->oddsIncTime = 0;
	}
}

void dlg_challenge::slot_changed(int)
{
	setButtonText(User1, i18n("Counter Offer"));
	challenge->changed = true;
}

void dlg_challenge::slot_timeOdds(bool state)
{
    BOX_Black->setEnabled(state);
}

void dlg_challenge::accept()
{
	setValues();
	emit SIGNAL_acceptParams( challenge );
}

void dlg_challenge::decline()
{
	emit SIGNAL_decline( challenge );
}

