/***************************************************************************
                          dlg_login.cpp  -  description
                             -------------------
    begin                : Sat Sep 29 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlg_login.moc"

dlg_login::dlg_login(QWidget *parent, const char *name, resource *rsrc) :
        KDialogBase(parent, name, true, i18n("Play Online"), Help|Ok|Cancel, Ok,true )
{
    serverList::Iterator i;
    serverList::Iterator currentServer;
    int counter = 0;

    myResource = rsrc;

    BOX_Parent = makeVBoxMainWidget();

    /* Select which server to connect to */
    GROUP_Select_Server = new QGroupBox( 1, Qt::Vertical, i18n( "Please Select a Service" ), BOX_Parent );

    COMBO_Select_Server = new KComboBox ( GROUP_Select_Server );
    COMBO_Select_Server->setEditable( false );

    /* fill the combobox with servers */
    for(i = myResource->servers.begin(); i != myResource->servers.end(); i++)
    {
        COMBO_Select_Server->insertItem((*i).Name);
        if((*i).CurrentRef == true)
        {
            COMBO_Select_Server->setCurrentItem(counter);
            currentServer = i;
        }
        counter++;
    }
    /* connect to right signal to the combobox */
    connect( COMBO_Select_Server, SIGNAL( activated(const QString &) ), this, SLOT( slotUpdateUser(const QString &) ) );

    GROUP_Username = new QGroupBox( 1, Qt::Horizontal, i18n("User info"), BOX_Parent);
    BOX_ALIGN = new QHBox( GROUP_Username );
    BOX_TEXT = new QVBox( BOX_ALIGN );
    TEXT_Login = new QLabel( BOX_TEXT );
    TEXT_Password = new QLabel( BOX_TEXT );

    TEXT_Login->setText( i18n("Account Name:") );
    TEXT_Password->setText( i18n("Password:") );

    BOX_EDIT = new QVBox( BOX_ALIGN );
    EDIT_Login = new KLineEdit( BOX_EDIT );
    EDIT_Password = new KLineEdit( BOX_EDIT );

    CHECKBOX_GUEST = new QCheckBox(i18n("Log in as guest"), GROUP_Username);
    EDIT_Password->setEchoMode( QLineEdit::Password );

    connect(CHECKBOX_GUEST, SIGNAL(toggled(bool)), this, SLOT(slotGuestToggle(bool)));

    KGuiItem OK( i18n("Connect"), QString::null, i18n("Connect to the chess server using this name and password.") );

    /* Init the buttons */
    showButtonCancel( true );
    showButtonOK( true );
    showButton( Help, true );

    enableButtonCancel( true );
    enableButtonOK( true );
    enableButton( Help, true );
    setButtonOK( OK );
    setHelp("prefs");
    disableResize();
    show();
    /* make sure the username for the default server is displayed */
    slotUpdateUser((*currentServer).Name);
}
///////////////////////////////////////
//
//  dlg_login::~dlg_login
//
///////////////////////////////////////
dlg_login::~dlg_login()
{}
///////////////////////////////////////
//
//  dlg_login::slotOk
//
///////////////////////////////////////
void dlg_login::slotOk( void )
{
    if(BOX_ALIGN->isEnabled())
    {
        emit login(EDIT_Login->text(), EDIT_Password->text());
    }
    else
    {
        emit login("guest", "");
    }
    emit okClicked();
    slotDelayedDestruct();
}

///////////////////////////////////////
//
//  dlg_login::slotGuestToggle
//
///////////////////////////////////////
void dlg_login::slotGuestToggle( bool on )
{
    BOX_ALIGN->setEnabled(!on);
}

///////////////////////////////////////
//
//  dlg_login::slotUpdateUser
//
///////////////////////////////////////
void dlg_login::slotUpdateUser(const QString &name)
{
    /* this function retrieves the correct serverResource from
       the resources and uses that to fill in the username and
       password fields */
    serverList::Iterator i;

    for(i = myResource->servers.begin(); i != myResource->servers.end(); i++)
    {
        if((*i).CurrentRef)
        {
            (*i).CurrentRef = false;
        }
        if((*i).Name == name )
        {
            (*i).CurrentRef = true;
            EDIT_Login->setText((*i).UserName);
            EDIT_Password->setText((*i).Password);
            server = &(*i);
        }
    }
}

///////////////////////////////////////
//
//  dlg_login::slotUpdateUser
//
///////////////////////////////////////
void dlg_login::disableServerSelect()
{
    COMBO_Select_Server->setEnabled(false);
}


