/***************************************************************************
                          dlg_promote.cpp  -  description
                             -------------------
    begin                : Fri Jul 13 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "definitions.h"
#include "dlg_promote.moc"
#include "knightspixcache.h"
#include <qimage.h>

dlg_promote::dlg_promote(QWidget *parent, const char *name, resource *Rsrc  ) :
        QDialog( parent, name, true, Qt::WStyle_Customize | Qt::WStyle_DialogBorder )
{
    Resource = Rsrc;
    Queen = new QPushButton( this );
    Bishop = new QPushButton( this );
    Knight = new QPushButton( this );
    Rook = new QPushButton( this );
    Description = new QLabel( this );
}

dlg_promote::~dlg_promote()
{
    delete Queen;
    delete Bishop;
    delete Knight;
    delete Rook;
    delete Description;
}

void dlg_promote::Init( bool Army )
{
    KnightsPixCache *cache = Resource->pixCache;
    QPixmap buffer, temp;
    int margin(2);
    int size = 32;

    buffer.resize( size, size );
    Description->setFixedSize( size * 2 , 20 );
    Queen->setFixedSize( size, size );
    Bishop->setFixedSize( size, size );
    Knight->setFixedSize( size, size );
    Rook->setFixedSize( size, size );
    setFixedSize(       ( margin * 4 ) + ( size * 2 ),
                  ( margin * 6 ) + ( size * 2 ) + 20 );

    Description->move( margin, margin );
    Queen->move( margin, ( margin * 3 ) + 20 );
    Bishop->move( ( margin * 3 ) + size, ( margin * 3 ) + 20 );
    Knight->move( margin, ( margin * 5 ) + 20 + size );
    Rook->move( ( margin * 3 ) + size, ( margin * 5 ) + 20 + size );

    /* Queen */
    cache->find( QString("orig_%1_light").arg(size), buffer );
    if( Army == WHITE )
    {
        cache->find( QString("orig_%1_wq").arg(size), temp );
    }
    else
    {
        cache->find( QString("orig_%1_bq").arg(size), temp );
    }
    bitBlt( &buffer, 0, 0, &temp, 0, 0, -1, -1, Qt::CopyROP, false);
    Queen->setPixmap( buffer );
    Queen->show();

    /* Bishop */
    cache->find( QString("orig_%1_dark").arg(size), buffer );
    if( Army == WHITE )
    {
        cache->find( QString("orig_%1_wb").arg(size), temp );
    }
    else
    {
        cache->find( QString("orig_%1_bb").arg(size), temp );
    }
    bitBlt( &buffer, 0, 0, &temp, 0, 0, -1, -1, Qt::CopyROP, false);
    Bishop->setPixmap( buffer );
    Bishop->show();

    /* Knight */
    cache->find( QString("orig_%1_dark").arg(size), buffer );
    if( Army == WHITE )
    {
        cache->find( QString("orig_%1_wn").arg(size), temp );
    }
    else
    {
        cache->find( QString("orig_%1_bn").arg(size), temp );
    }
    bitBlt( &buffer, 0, 0, &temp, 0, 0, -1, -1, Qt::CopyROP, false);
    Knight->setPixmap( buffer );
    Knight->show();

    /* Rook */
    cache->find( QString("orig_%1_light").arg(size), buffer );
    if( Army == WHITE )
    {
        cache->find( QString("orig_%1_wr").arg(size), temp );
    }
    else
    {
        cache->find( QString("orig_%1_br").arg(size), temp );
    }
    bitBlt( &buffer, 0, 0, &temp, 0, 0, -1, -1, Qt::CopyROP, false);
    Rook->setPixmap( buffer );
    Rook->show();

    Description->setText( i18n( "Promote your pawn to..." ) );
    Description->show();
    connect( Queen, SIGNAL( clicked() ), this, SLOT( queenClick() ) );
    connect( Bishop, SIGNAL( clicked() ), this, SLOT( bishopClick() ) );
    connect( Knight, SIGNAL( clicked() ), this, SLOT( knightClick() ) );
    connect( Rook, SIGNAL( clicked() ), this, SLOT( rookClick() ) );
    Queen->setFocus();
    setCaption( i18n( "Pawn Promotion" ) );
}

void dlg_promote::queenClick( void )
{
    done( 'q' );
}

void dlg_promote::bishopClick( void )
{
    done( 'b' );
}

void dlg_promote::knightClick( void )
{
    done( 'n' );
}

void dlg_promote::rookClick( void )
{
    done( 'r' );
}

