//
// C++ Implementation: dlg_uciconfig
//
// Description:
//
//
// Author: Troy Corbin Jr. <troy@knights-chess.com>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "dlg_uciconfig.moc"
#include "io_engine.h"
#include <qgroupbox.h>
#include <qvbox.h>
#include <qcheckbox.h>
#include <klineedit.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qspinbox.h>

///////////////////////////////////////
//
//      dlg_uciconfig::dlg_uciconfig()
//
///////////////////////////////////////
dlg_uciconfig::dlg_uciconfig(QWidget *parent, const char *name, resource *Rsrc, engineList::Iterator eIT )
 : KDialogBase( parent,
                name,
                true,
                i18n("UCI Engine Options"),
                Help|Ok|Apply|Cancel,
                Ok,
                true )
{
  myResource = Rsrc;
  IT = eIT;

  (*IT).CurrentRef |= ENGINE_TEST;
  engine = new io_engine( this, myResource );
  connect( engine, SIGNAL( sendCMD( const Command& ) ), this, SLOT( getFromEngine( const Command& ) ) );
  engine->Start( BOTH );

  BOX_Parent = makeVBoxMainWidget();
  optionList = new OptionList;

	/* Init the buttons */
	showButtonCancel( true );
	showButtonOK( true );
	showButtonApply( true );
	showButton( Help, true );

	enableButtonCancel( true );
	enableButtonOK( true );
	enableButtonApply( true );
	enableButton( Help, true );
}
///////////////////////////////////////
//
//      dlg_uciconfig::~dlg_uciconfig()
//
///////////////////////////////////////
dlg_uciconfig::~dlg_uciconfig()
{
  (*IT).CurrentRef -= ENGINE_TEST;
  delete engine;
  delete optionList;
}
///////////////////////////////////////
//
//      dlg_uciconfig::getFromEngine()
//
///////////////////////////////////////
void dlg_uciconfig::getFromEngine( const Command &cmd )
{
  if( cmd.getCommand() == CMD_UCI_Options )
  {
    engineOptions = QStringList::split( ":':", cmd.getData() );
    presetOptions = QStringList::split( ":':", (*IT).AdvancedOptions );

    for( unsigned int a=0; a < engineOptions.count(); a++ )
    {
      kdWarning() << engineOptions[a] << endl;
      UCIOption newOption;

      // Catch the Widget's Name
      int pos = engineOptions[a].find( " type " );
      newOption.name = engineOptions[a].mid( 5, pos - 5 );

      // Set 'pos' = the section after 'type'
      QStringList parsed = QStringList::split( ' ', engineOptions[a] );
      pos = 2;
      while( parsed[pos] != "type" )
      {
        pos++;
        if( pos > 100 )
          break;
      }

      QString type = parsed[++pos];
      pos++;
      if( type == "check" ) // Is Type Check
      {
        newOption.boxPtr = new QGroupBox( 1, Qt::Horizontal, newOption.name, BOX_Parent );
        newOption.ptr = new QCheckBox( newOption.name, newOption.boxPtr );
        if( parsed[pos] == "default" )
        {
          pos++;
          if( parsed[pos] == "true" )
          {
            ((QCheckBox*)newOption.ptr)->setChecked( true );
          }
          else
          {
            ((QCheckBox*)newOption.ptr)->setChecked( false );
          }
        }
      }
      else if( type == "spin" ) // Is Type Spin
      {
        newOption.boxPtr = new QGroupBox( 1, Qt::Horizontal, newOption.name, BOX_Parent );
        int myDefault(1), myMin(1), myMax(1);
        while( 1 )
        {
          if( parsed[pos] == "min" )
          {
            pos++;
            myMin = parsed[pos].toInt();
          }
          else if( parsed[pos] == "max" )
          {
            pos++;
            myMax = parsed[pos].toInt();
          }
          else if( parsed[pos] == "default" )
          {
            pos++;
            myDefault = parsed[pos].toInt();
          }
          pos++;
          if( (unsigned)pos > parsed.count() )
          {
            newOption.ptr = new QSpinBox( myMin, myMax, 1, newOption.boxPtr );
            kdWarning() << "Setting 1" << endl;
            ((QSpinBox*)newOption.ptr)->setValue( myDefault );
            break;
          }
        }
      }
      else if( type == "combo" ) // Is Type Combo
      {
        newOption.boxPtr = new QGroupBox( 1, Qt::Horizontal, newOption.name, BOX_Parent );
        newOption.ptr = new QComboBox( false, newOption.boxPtr );
        while( 1 )
        {
          if( ( parsed[pos] == "var" ) || ( parsed[pos] == "default" ) )
          {
            pos++;
            ((QComboBox*)newOption.ptr)->insertItem( parsed[pos] );
          }
          pos++;
          if( (unsigned)pos > parsed.count() )
          {
            break;
          }
        }
      }
      else if( type == "button" ) // Is Type Button
      {
        newOption.boxPtr = NULL;
        newOption.ptr = new QPushButton( newOption.name, BOX_Parent );
      }
      else if( type == "string" ) // Is Type String
      {
        newOption.boxPtr = new QGroupBox( 1, Qt::Horizontal, newOption.name, BOX_Parent );
        if( parsed[pos] == "default" )
        {
          pos++;
          newOption.ptr = new KLineEdit( parsed[pos], newOption.boxPtr );
        }
        else
        {
          newOption.ptr = new KLineEdit( "<empty>", newOption.boxPtr );
        }
      }
      optionList->append( newOption );
    }
    setMainWidget( BOX_Parent );
    show();
  }
}
