/***************************************************************************
                          io_internet.cpp  -  description
                             -------------------
    begin                : Thu Aug 16 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* KDE */
#include <ksock.h>
#include <kprocess.h>
#include <kmessagebox.h>

/* Qt */
#include <qobject.h>
#include <qwidget.h>
#include <qregexp.h>
#include <qdns.h>
#include <qsocket.h>

/* Local */
#include "io_internet.moc"
#include "resource.h"
#include "dlg_login.h"
#include "dlg_challenge.h"
#include "match.h"
#include "core.h"
#include "audio.h"
#include "../config.h"
#include "server_base.h"
#include "server_fics.h"
#include "server_icc.h"

static const int TXT_Std = 0;
static const int TXT_Pvt = 1;
static const int TXT_Ch  = 2;
static const int TXT_Sht = 3;
static const int TXT_Wsp = 4;
static const int TXT_Not = 5;

io_internet::io_internet(core *parent, resource *rsrc)
{
    myResource = rsrc;
    challenge = NULL;
    myTimeseal = NULL;
		myServerHandler = NULL;
    socket = NULL;
		readBuffer = NULL;
    lineBuffer = "";
    Log = NULL;

    // set type
    this->myType = io_base::INTERNET;

    // initialize various variables
    this->myParent = parent;
    connected = false; // we are not connected to a server
}

///////////////////////////////////////////////////////////
// Destructor
///////////////////////////////////////////////////////////
io_internet::~io_internet()
{
    /* Close and remove socket */
    if( socket != NULL )
    {
        if(socket->socket() != -1)
        {
            send("quit", false);
        }
        delete socket;
				socket = NULL;
    }
    /* Close and remove logfile */
    if( Log )
    {
        Log->close();
        delete Log;
        Log = NULL;
    }
    /* Remove Timeseal */
    if( myTimeseal != NULL )
    {
        myTimeseal->kill();
        delete myTimeseal;
				myTimeseal = NULL;
    }
		if( readBuffer != NULL )
		{
			delete[] readBuffer;
			readBuffer = NULL;
		}
		if( myServerHandler != NULL )
		{
			delete myServerHandler;
			myServerHandler = NULL;
		}
}

void io_internet::connectToServer()
{
    serverList::iterator i;

    qApp->activeWindow()->setCursor( myResource->CURSOR_Thinking );

    /* Get and parse server config from resource */
    myServer = NULL;
    for( i = myResource->servers.begin(); i != myResource->servers.end(); i++ )
    {
        if( (*i).CurrentRef )
        {
            myServer = &(*i);
        }
    }
    if(myServer == NULL)
    {
        /* No server configured. Notify User and die gracefully */
        qApp->activeWindow()->setCursor( myResource->CURSOR_Standard );
        kdError() << "internetio::internetio: Can not find server resource CurrentRef " << endl;
        KMessageBox::sorry( (QWidget*)myParent, i18n( "There are no services configured.\nPlease make sure you have at least one service configured." ), i18n( "Cannot find a service."));
        QApplication::postEvent( myResource->Core, new QCustomEvent( EVENT_Del_IO_Net ) );
        return;
    }

		switch(myServer->ServerType)
		{
			case server_base::FICS :
				myServerHandler = new server_fics(myResource, this);
				break;
			case server_base::ICC :
				myServerHandler = new server_icc(myResource, this);
				break;
			default :
				break;
		}

		//connect the SIGNALS to the correct slots
		connect(this, SIGNAL(emitData(const QString&)), myServerHandler, SLOT(processData(const QString&)));
		connect(myServerHandler, SIGNAL( newChallenge( Challenge* ) ), this, SLOT( newChallenge( Challenge* ) ) );
		connect(myServerHandler, SIGNAL(newMatch(const match_param*, int)), this, SLOT(newMatch(const match_param*, int)));
		connect(myServerHandler, SIGNAL(sendMove(const ChessMove&, const QString&, int, int, int)), this, SLOT(move(const ChessMove&, const QString&, int, int, int)));
		connect(myServerHandler, SIGNAL(sendCMD(const Command&)), this, SLOT(recvHandlerCMD(const Command&)));

    if(myServer->Port == 0)
    {
        myServer->Port = 5000;
    }
    if(!myServer->Timeseal.isEmpty())
    {
        myTimeseal = new KProcess();
        (*myTimeseal) << myServer->Timeseal << myServer->URL << QString().setNum( myServer->Port ) << QString( "-p" ) << QString().setNum(myServer->Port + 1);
        if(!myTimeseal->start())
        {
            /* Couldn't start Timeseal. Notify User and die gracefully */
            qApp->activeWindow()->setCursor( myResource->CURSOR_Standard );
            KMessageBox::sorry( (QWidget*)myParent, i18n( "Knights can not start the Timeseal program.\nPlease make sure you have the correct path and filename configured." ), i18n( "Cannot start Timeseal."));
            QApplication::postEvent( myResource->Core, new QCustomEvent( EVENT_Del_IO_Net ) );
            return;
        }
        //This sleep allows timeseal to load before we try to connect to it.
        sleep(2);

        socket = new KSocket("127.0.0.1", myServer->Port + 1, 30);
        if(socket->socket() == -1)
        {
            /* Couldn't connect to server. Notify User and die gracefully */
            qApp->activeWindow()->setCursor( myResource->CURSOR_Standard );
            KMessageBox::sorry( (QWidget*)myParent, i18n( "Knights is unable to connect to the service.\n Please make sure your internet connection is working correctly and try again."), i18n( "Cannot connect to service."));
            QApplication::postEvent( myResource->Core, new QCustomEvent( EVENT_Del_IO_Net ) );
            return;
        }
				readBuffer = new char[READ_BUFFER_SIZE];
    }
    else
    {
        socket = new KSocket(myServer->URL, myServer->Port, 30);
        if(socket->socket() == -1)
        {
            /* Couldn't connect to server. Notify User and die gracefully */
            qApp->activeWindow()->setCursor( myResource->CURSOR_Standard );
            KMessageBox::sorry( (QWidget*)myParent, i18n( "Knights is unable to connect to the service.\n Please make sure your internet connection is working correctly and try again."), i18n( "Cannot connect to service."));
            QApplication::postEvent( myResource->Core, new QCustomEvent( EVENT_Del_IO_Net ) );
            return;
        }
				readBuffer = new char[READ_BUFFER_SIZE];
    }

    socket->enableRead(true);

    /* connect a signal to readReady */
    connect(socket, SIGNAL(readEvent(KSocket *)), this, SLOT(readCommand(KSocket *)));

    /* setup the seekTimer and turn it off by default */
    seekTimer = new QTimer(this);
    connect(seekTimer, SIGNAL(timeout()), myServerHandler, SLOT(processSeekTimer()));

    /* succesfully connected to the server, turn connected on and flush the command buffer */
    connected = true;

    /* ...log file */
    if( !myServer->LogFile.isEmpty() )
    {
        Log = new QFile( myServer->LogFile );
        if( !Log->open( IO_WriteOnly | IO_Append ) )
        {
            if( !Log->open( IO_WriteOnly ) )
            {
                kdError() << "Can not open the logfile located at " << myServer->LogFile << endl;
            }
        }
    }
    qApp->activeWindow()->setCursor( myResource->CURSOR_Standard );
}

void io_internet::recvCMD(const Command& command)
{
    QString error_message;

    if(!connected)
    {
        myCommandBuffer.push_back(command);
        return;
    }

    switch(((Command)command).getCommand())
    {
    case CMD_Move:
        send(((Command)command).getMove().SAN, false);
        break;
    case CMD_Reset_Server:
        myServerHandler->sendUserSettings();
        break;
    case CMD_Toggle_Seek:
        if(seekTimer->isActive())
        {
            /* timer running stop it */
            seekTimer->stop();
            emit sendCMD( Command( 0, CMD_Hide_Sought_List ) );
        }
        else
        {
            /* timer not running start it */
            seekTimer->start(myResource->Seek_Timer * 100);
            /* send a sought now */
            myServerHandler->processSeekTimer();
        }
        break;
    case CMD_Player_Finger:
        send("$finger " + ((Command)command).getData(), false);
        break;
    case CMD_Add_Friend:
        send("$addlist notify " + ((Command)command).getData(), false);
        break;
    case CMD_Ignore_Player:
        send("$addlist censor " + ((Command)command).getData(), false);
        break;
    case CMD_Player_History:
        send("$history " + ((Command)command).getData(), false);
        break;
    case CMD_Start_Match:
        send("$play " + ((Command)command).getData(), false);
        break;
    case CMD_Assess:
        send("$assess " + ((Command)command).getData(), false);
        break;
    case CMD_Set_Input:
        sendCMD( command );
        break;
    case CMD_Send_To_ICS:
        if(((Command)command).getData().contains(QRegExp("^\\s*(:?\\.|tell)")))
        {
            writeToConsole(((Command)command).getData(), "K_PVT");
        }
        send(((Command)command).getData(), false);
        break;
    case CMD_Examine_Forward:
        send("$forward 1", false);
        break;
    case CMD_Examine_Backward:
        send("$backward 1", false);
        break;
    case CMD_White_Resign:
    case CMD_Black_Resign:
        send("$resign", false);
        break;
    case CMD_Offer_Draw:
        send("$draw", false);
        break;
    case CMD_Reject_Draw:
        send("$decline t draw", false);
        break;
    case CMD_Result_White:
    case CMD_Result_Black:
    case CMD_Result_Draw:
        /* do nothing, the gui determined a game was over */
        break;
    case CMD_White_Called_Flag:
    case CMD_Black_Called_Flag:
        send("$flag", true);
        break;
    case CMD_Seek_Match:
        send("$seek " + ((Command) command).getData(), true);
        break;
		case CMD_Pause:
		case CMD_Resume:
			/* Ignore GUI commands about pauses */
			break;
    default:
        /* do nothing unknown command */
        kdWarning() << "InternetIO::sendCMD received an unknown command: " << ((Command)command).getCommand() << endl;
    }
}

///////////////////////////////////////
//
//  io_internet::send
//
///////////////////////////////////////
void io_internet::send(const QString& msg, bool dontLog)
{
    QString tmp(msg);
    ssize_t len;

    if( tmp.right(1) != "\n" )
    {
        tmp += "\n";
    }

    socket->enableWrite(true);
    len = write(socket->socket(), tmp.latin1(), tmp.length() );
    socket->enableWrite(false);

    if( Log && !dontLog)
    {
        Log->writeBlock( QString("<< ").latin1(), 3 );
        Log->writeBlock( tmp.latin1(), tmp.length() );
    }
    if( len < (signed)tmp.length() )
    {
        kdWarning() << "io_internet::Send: Failed to write full block of data to socket." << endl;
    }
}

///////////////////////////////////////
//
//      io_internet::readCommand
//
///////////////////////////////////////
void io_internet::readCommand(KSocket* eventSocket)
{
	if( eventSocket != socket )
	{
		kdWarning() << "io_internet::readCommand triggered by unknown socket @ " << &eventSocket << endl;
		return;
	}

    QString tmp;
    QStringList lines;

		ssize_t result = read( socket->socket(), readBuffer, READ_BUFFER_SIZE - 1 );
		if( result == -1 )
		{
			kdWarning() << "io_internet::readCommand: Error reading from socket." << endl;
			return;
		}
		readBuffer[result] = 0;
		tmp.setLatin1( readBuffer, result );

    tmp = lineBuffer + tmp;
    if( Log )
    {
        Log->writeBlock( tmp.latin1(), tmp.length() );
        Log->flush();
    }

    lines = QStringList::split( "\n", tmp, false);
    if( !( tmp.endsWith("\n\r") || tmp.endsWith("\n") ) )
    {
        lineBuffer = ( *( --lines.end() ) );
        lines.pop_back();
    }
    else
    {
        lineBuffer = "";
    }
    for(QStringList::iterator i = lines.begin(); i != lines.end(); i++)
    {
      emit emitData(*i);
    }
}

///////////////////////////////////////
//
//  io_internet::writeToConsole
//
///////////////////////////////////////
void io_internet::writeToConsole(const QString& text, const QString& tag)
{
    QString myText = text;
    /* Replace misc characters with rich-text friendly counterparts */
    myText.replace( QRegExp("\\x003c"), "&#60;" );
    myText.replace( QRegExp("\\x007c"), "&#124;" );
    myText.remove( "\n" );
    emit sendCMD(Command( 0, CMD_Append_To_Console, "<" + tag + ">" + myText + "</" + tag + ">"));
}

///////////////////////////////////////
//
//  io_internet::updateSoughtList
//
///////////////////////////////////////
void io_internet::updateSoughtList(const QString& soughtLine)
{
    /* "ADS DISPLAYED" MESSAGE */
    if(soughtLine.contains(QRegExp("\\d+\\s+ads? displayed.")))
    {
        emit sendCMD( Command( 0, CMD_Show_Sought_List ) );
    }
    else
    {
        emit sendCMD( Command( 0, CMD_Add_Sought_Match, soughtLine ) );
    }
}

///////////////////////////////////////
//
//  io_internet::selfDestruct()
//
///////////////////////////////////////
void io_internet::selfDestruct()
{
    /* cause the io_internet to delete itself */
    QApplication::postEvent( myResource->Core, new QCustomEvent( EVENT_Del_IO_Net ) );
}

///////////////////////////////////////
//
//  io_internet::setUserInfo(const QString&, const QString&)
//
///////////////////////////////////////
void io_internet::setUserInfo(const QString& userName, const QString& password)
{
    this->userName = (QString)userName;
    this->password = (QString)password;
}

///////////////////////////////////////
//
//  io_internet::sendUserName()
//
///////////////////////////////////////
void io_internet::sendUserName(void)
{
    send(userName, false);
}

///////////////////////////////////////
//
//  io_internet::sendPassword()
//
///////////////////////////////////////
void io_internet::sendPassword(void)
{
    send(password, true);
}

///////////////////////////////////////
//
//  io_internet::newChallenge()
//
///////////////////////////////////////
void io_internet::newChallenge( Challenge *chal )
{
    myResource->play( Sound::Challenge );
    if(challenge != NULL)
    {
        delete challenge;
    }
    challenge = new dlg_challenge( 0, "Challenge", myResource );
    connect(challenge, SIGNAL(destroyed()), this, SLOT(clearChallenge()));
    connect(challenge, SIGNAL( SIGNAL_acceptParams( Challenge* ) ), myServerHandler, SLOT( acceptChallenge( Challenge* ) ) );
    connect(challenge, SIGNAL( SIGNAL_decline( Challenge* ) ), myServerHandler, SLOT( declineChallenge( Challenge* ) ) );
    connect(myServerHandler, SIGNAL(withDrawChallenge()), challenge, SLOT(slotDelayedDestruct()));
    connect(myServerHandler, SIGNAL(withDrawChallenge()), this, SLOT(clearChallenge()));
    challenge->setValues( chal );
}

///////////////////////////////////////
//
//  io_internet::getUserName()
//
///////////////////////////////////////
QString io_internet::getUserName(void)
{
    return userName;
}

///////////////////////////////////////
//
//  io_internet::clearChallenge()
//
///////////////////////////////////////
void io_internet::clearChallenge(void)
{
    challenge = NULL;
}

///////////////////////////////////////
//
//  io_internet::newMatch(const match_param*, int ficsGameID)
//
///////////////////////////////////////
void io_internet::newMatch(const match_param* param, int gameID)
{
    fics_to_knights[gameID] = ((core*)myParent)->newMatch((match_param*)param)->getID();
}

///////////////////////////////////////
//
//  io_internet::isActiveGame(int ficsGameID)
//
///////////////////////////////////////
bool io_internet::isActiveGame(int gameID)
{
    return ( fics_to_knights[gameID] == 0 );
}

///////////////////////////////////////
//
//  io_internet::move(const ChessMove&, const QString&, int, int, int);
//
///////////////////////////////////////
void io_internet::move(const ChessMove& move, const QString& dataLine, int gameID, int whiteTime, int blackTime)
{
    Command command(fics_to_knights[gameID], CMD_Move, whiteTime, blackTime, move);
    command.setData((QString &)dataLine);
    emit sendCMD(command);
}

///////////////////////////////////////
//
//  io_internet::recvHandlerCMD(const Command&)
//
///////////////////////////////////////
void io_internet::recvHandlerCMD(const Command& command)
{
    Command myCommand(command);
    switch(((Command)command).getCommand())
    {
    case CMD_Move:
    case CMD_Black_Resign:
    case CMD_White_Called_Flag:
    case CMD_Result_White:
    case CMD_White_Resign:
    case CMD_Black_Called_Flag:
    case CMD_Result_Black:
    case CMD_Result_Draw:
    case CMD_Lost_Contact:
    case CMD_Set_Src_Tell:
    case CMD_Set_Src_Channel:
    case CMD_Close_Match:
    case CMD_Receive_Piece:
    case CMD_Sideboard_All:
        myCommand.setID(fics_to_knights[command.getID()]);
        emit sendCMD(myCommand);
        break;
    case CMD_Hide_Sought_List:
        emit sendCMD(command);
    }
}

///////////////////////////////////////
//
//  io_internet::stopTimer(const Command&)
//
///////////////////////////////////////
void io_internet::stopTimer()
{
    if(seekTimer->isActive())
    {
        /* timer running stop it */
        seekTimer->stop();
    }
}

///////////////////////////////////////
//
//  io_internet::showConnectDialog()
//
///////////////////////////////////////
void io_internet::showConnectDialog()
{
    // create the login dialog and show it to the user
    loginDlg = new dlg_login( NULL, "LoginDialog", myResource);
    connect(loginDlg, SIGNAL( okClicked() ), this, SLOT( connectToServer() ) );
    connect(loginDlg, SIGNAL( cancelClicked() ), this, SLOT( selfDestruct() ) );
    connect(loginDlg, SIGNAL( login(const QString&, const QString&) ), this, SLOT( setUserInfo(const QString&, const QString&) ) );
}


