/***************************************************************************
                          io_internet.h  -  description
                             -------------------
    begin                : Thu Aug 16 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef IO_INTERNET_H
#define IO_INTERNET_H

/* Local */
#include "definitions.h"
#include "io_base.h"
#include "match_param.h"
#include "command.h"
#include "core.h"

/* Tabs */
#include "tabmanager.h"
#include "console.h"
#include "challenge_graph.h"
#include "seekmatchtab.h"

#include <qfile.h>

class KSocket;
class KProcess;
class resource;
class core;
class dlg_login;
class dlg_challenge;


/**
  *@author Troy Corbin Jr / Alexander Wels.
  */

struct MatchRecord
{
    int         knightsID;
    int         serverID;
    bool        observing;
    bool        examining;
    bool        finished;
};

const int READ_BUFFER_SIZE = 1024; /* 64k is max ip packet size */

class io_internet : public io_base
{
    Q_OBJECT
public:
    io_internet(core *parent = NULL, resource *Rsrc = NULL);
    ~io_internet();
    void                      writeToConsole(const QString& text, const QString& tag);
    void                      updateSoughtList(const QString& text);
    QString                   getUserName();
    bool                      isActiveGame(int);  /* returns true if the fics id passed in is an active game */
    void                      stopTimer(); /* stop the seek timer if it is active */
public slots:
    void                      newChallenge( Challenge* );
    void                      clearChallenge();
    void                      newMatch(const match_param*, int);
    void                      move(const ChessMove&, const QString&, int, int, int);
    void                      recvHandlerCMD(const Command&);
    void                      sendUserName();
    void                      sendPassword();
    void                      recvCMD(const Command& command);
    void                      readCommand(KSocket *);
    void                      send(const QString&, bool dontLog);
    void                      connectToServer( void );
    void                      selfDestruct( void );
    void                      setUserInfo(const QString&, const QString&);
    void                      showConnectDialog( void );
signals:
    void                      sendCMD(const Command& command);
    void                      setLastPrivateSource( const QString& );
    void                      setLastChannelSource( const QString& );
    void                      emitData(const QString& line);

protected:
    dlg_challenge             *challenge;

private:

    dlg_login                 *loginDlg;
    /* These are used to handle the automagic seek command */
    QTimer                    *seekTimer;

    /* These are used for the connection & logon */
    QFile                     *Log;
    KSocket                   *socket;
    KProcess                  *myTimeseal;
    struct serverResource     *myServer;
		char											*readBuffer;
    QString                   lineBuffer;
    bool                      connected;
    QString                   userName;
    QString                   password;

    /* communication with the server */
    QStringList               command_list;    // this is a temp buffer while waiting for the movelist
    QMap<match_param *, int>  fics_ids;
    QMap<int, int>            fics_to_knights; // maps fics id's to knights id's
    QValueList<Command>       myCommandBuffer; // this is a buffer for all the commands io_internet received
    // while not connected to a server
    server_base               *myServerHandler;// the server handler parses all the server specific stuff and
    // passes generic information back to io_internet so that it can handle
    // the generic information without caring to which type of server it is
    // connected.
};

#endif
