/***************************************************************************
                          knightspixcache.cpp  -  description
                             -------------------
    begin                : Mon Aug 20 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "knightspixcache.moc"
#include "definitions.h"
#include <qwmatrix.h>

extern bool qt_use_xrender;
extern bool qt_has_xft;

KnightsPixCache::KnightsPixCache() : QObject( 0, "KnightsPixCache" )
{
    bytesTotal = 1024 * 1024;
    bytesUsed = 0;
}
KnightsPixCache::~KnightsPixCache()
{
    clear();
    bytesUsed = 0;
}
///////////////////////////////////////
//
//  KnightsPixCache::resetCache
//
///////////////////////////////////////
void KnightsPixCache::resetCache( void )
{
    clear();
    emit themeChanged();
}
///////////////////////////////////////
//
//  KnightsPixCache::cacheLimit
//
///////////////////////////////////////
unsigned int KnightsPixCache::cacheLimit( void )
{
    return ( bytesTotal / 1024 );
}
///////////////////////////////////////
//
//  KnightsPixCache::setCacheLimit
//
///////////////////////////////////////
void KnightsPixCache::setCacheLimit( const unsigned int &limit )
{
    bytesTotal = limit * 1024;
    if( bytesUsed > bytesTotal )
    {
        cleanCache( 0 );
    }
}
///////////////////////////////////////
//
//  KnightsPixCache::clear
//
///////////////////////////////////////
void KnightsPixCache::clear( void )
{
    list.clear();
    bytesUsed = 0;
}
///////////////////////////////////////
//
//      KnightsPixCache::cleanCache
//
///////////////////////////////////////
void KnightsPixCache::cleanCache( const unsigned int &space )
{
    pixmapList::Iterator        tmpIT;
    register unsigned int age;

    while( bytesUsed + space > bytesTotal )
    {
        age = 0;
        IT = list.begin();
        if( IT == list.end() )
        {
            return;
        }

        while( IT != list.end() )
        {
            if( (*IT).age > age )
            {
                age = (*IT).age;
                tmpIT = IT;
            }
            ++IT;
        }

        bytesUsed -= (*tmpIT).bytes;
        list.remove( tmpIT );
    }
}
///////////////////////////////////////
//
//      KnightsPixCache::add
//
///////////////////////////////////////
void KnightsPixCache::add
    ( const QString &label, const QPixmap &pixmap )
{
    unsigned int tmp;
    cacheItem newItem;

    if( pixmap.isNull() )
    {
        return;
    }
    tmp = ( pixmap.width() * pixmap.height() * pixmap.depth() ) >> 3;
    if( ( bytesUsed + tmp ) > bytesTotal )
        cleanCache(tmp);

    newItem.label = label;
    newItem.bytes = tmp;
    newItem.item = pixmap;
    newItem.age = 0;
    list.append( newItem );
    bytesUsed += tmp;
}
///////////////////////////////////////
//
//      KnightsPixCache::find
//
///////////////////////////////////////
bool KnightsPixCache::find( const QString &label, QPixmap &pixmap )
{
    bool status(false);
    for( IT = list.begin(); IT != list.end(); ++IT )
    {
        if( (*IT).label == label )
        {
            pixmap = (*IT).item;
            (*IT).age = 0;
            status = true;
        }
        else
            (*IT).age++;
    }
    if( status == false )
    {
        if( label.startsWith( "orig_" ) )
        {
            resizeOriginal( label, pixmap );
            add
                ( label, pixmap );
            return true;
        }
    }
    return status;
}
///////////////////////////////////////
//
//      KnightsPixCache::resizeOriginal
//
///////////////////////////////////////
void KnightsPixCache::resizeOriginal( const QString &label, QPixmap &pixmap )
{
    QStringList list = QStringList::split( QString("_"), label );
    QImage tmpImage;

    /* Get size from the label */
    int size = list[1].toInt();
    if( size < IMAGE_MIN )
    {
        size = IMAGE_MIN;
    }
    if( size > IMAGE_MAX )
    {
        size = IMAGE_MAX;
    }

    if( list[2] == "light" )
    {
        tmpImage = SquareLight.smoothScale( size, size );
    }
    else if ( list[2] == "dark" )
    {
        tmpImage = SquareDark.smoothScale( size, size );
    }
    else if ( list[2] == "select" )
    {
        tmpImage = HighlightSelect.smoothScale( size, size );
    }
    else if ( list[2] == "move" )
    {
        tmpImage = HighlightMove.smoothScale( size, size );
    }
    else if ( list[2] == "attack" )
    {
        tmpImage = HighlightAttack.smoothScale( size, size );
    }
    else if ( list[2] == "bk" )
    {
        tmpImage = BlackKing.smoothScale( size, size );
    }
    else if ( list[2] == "bq" )
    {
        tmpImage = BlackQueen.smoothScale( size, size );
    }
    else if ( list[2] == "bb" )
    {
        tmpImage = BlackBishop.smoothScale( size, size );
    }
    else if ( list[2] == "bn" )
    {
        tmpImage = BlackKnight.smoothScale( size, size );
    }
    else if ( list[2] == "br" )
    {
        tmpImage = BlackRook.smoothScale( size, size );
    }
    else if ( list[2] == "bp" )
    {
        tmpImage = BlackPawn.smoothScale( size, size );
    }
    else if ( list[2] == "wk" )
    {
        tmpImage = WhiteKing.smoothScale( size, size );
    }
    else if ( list[2] == "wq" )
    {
        tmpImage = WhiteQueen.smoothScale( size, size );
    }
    else if ( list[2] == "wb" )
    {
        tmpImage = WhiteBishop.smoothScale( size, size );
    }
    else if ( list[2] == "wn" )
    {
        tmpImage = WhiteKnight.smoothScale( size, size );
    }
    else if ( list[2] == "wr" )
    {
        tmpImage = WhiteRook.smoothScale( size, size );
    }
    else if ( list[2] == "wp" )
    {
        tmpImage = WhitePawn.smoothScale( size, size );
    }
    else if ( list[2] == "border" )
    {
        tmpImage = Border.smoothScale( size * 9, size * 9 );
    }
    else if ( list[2] == "flipborder" )
    {
        QWMatrix matrix;
        matrix.rotate( 180.0f );
        tmpImage = Border.smoothScale( size * 9, size * 9).xForm( matrix );
    }
    else if ( list[2] == "on" )
    {
        tmpImage = BorderLightOn.smoothScale( size / 2, size / 2 );
    }
    else if ( list[2] == "off" )
    {
        tmpImage = BorderLightOff.smoothScale( size / 2, size / 2 );
    }

    /*
        Add transparency, if requested

        Borrowed and modified from KDE's KIconEffect::semiTransparent()
    */
    /* Get alpha from the label */
    if( ( list.count() > 3 ) && ( list[3].toFloat() != 1.0f ) )
    {
        tmpImage.setAlphaBuffer(true);
        float trans = list[3].toFloat();
        int x, y;

        /*
            Precalculate all possible new alpha values. This creates a bigger time
            saving as the image gets bigger. ( Slower on images ~16x16, or 256 pixels )
        */
        unsigned char alphaArray[256];
        for( x = 0; x < 256; x++ )
        {
            alphaArray[x] = (unsigned char)( (float)x * trans );
        }

        if( tmpImage.depth() == 32 )
        {
            int width  = tmpImage.width();
            int height = tmpImage.height();

            if( qt_use_xrender && qt_has_xft )
            {
                for( y = 0; y < height; y++ )
                {
#ifdef WORDS_BIGENDIAN
                    uchar *line = (uchar*)tmpImage.scanLine(y);
#else

                    uchar *line = (uchar*)tmpImage.scanLine(y) + 3;
#endif

                    for( x = 0; x < width; x++ )
                    {
                        *line = alphaArray[*line];
                        line += 4;
                    }
                }
            }
            else
            {
                for( y = 0; y < height; y++ )
                {
                    QRgb *line = (QRgb *) tmpImage.scanLine(y);
                    for( x = 0; x < width; x++ )
                    {
                        line[x] = qRgba( qRed(line[x]), qGreen(line[x]), qBlue(line[x]), alphaArray[qAlpha(line[x])] );
                    }
                    //          for( x = ( y % 2 ); x < width; x += 2 )
                    //          {
                    //            line[x] &= 0x00ffffff;
                    //          }
                }
            }
        }
        else
        {
            kdWarning() << "KnightsPixCache::resizeOriginal: Can not convert transparency on non-32bit images." << endl;
        }
    }

    pixmap.convertFromImage( tmpImage );
    return;
}
