/***************************************************************************
                          knightspixcache.h  -  description
                             -------------------
    begin                : Mon Aug 20 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KNIGHTSPIXCACHE_H
#define KNIGHTSPIXCACHE_H

#include <qvaluelist.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qobject.h>

/**
  *@author Troy Corbin Jr.
  */

typedef struct cacheItem
{
    QString				label;
    QPixmap				item;
    unsigned int	bytes;
    unsigned int	age;
};

typedef QValueList<cacheItem> pixmapList;

class KnightsPixCache : public QObject
{
    Q_OBJECT
public:
    QImage								SquareLight;
    QImage								SquareDark;
    QImage								HighlightSelect;
    QImage								HighlightMove;
    QImage								HighlightAttack;
    QImage								BlackKing;
    QImage								BlackQueen;
    QImage								BlackBishop;
    QImage								BlackKnight;
    QImage								BlackRook;
    QImage								BlackPawn;
    QImage								WhiteKing;
    QImage								WhiteQueen;
    QImage								WhiteBishop;
    QImage								WhiteKnight;
    QImage								WhiteRook;
    QImage								WhitePawn;
    QImage								Border;
    QImage								BorderLightOn;
    QImage								BorderLightOff;
    QPixmap								Background;

    KnightsPixCache();
    ~KnightsPixCache();
    void									setCacheLimit( const unsigned int &limit );
    unsigned int					cacheLimit( void );
    void									clear( void );
    void									add
    ( const QString &label, const QPixmap &pixmap );
    bool									find( const QString &label, QPixmap &pixmap );
    void									resizeOriginal( const QString &label, QPixmap &pixmap );

signals:

    void									themeChanged( void );

public slots:
    void									resetCache( void );

private:

    pixmapList						list;
    pixmapList::Iterator	IT;

    unsigned int					bytesUsed;
    unsigned int					bytesTotal;

    void									cleanCache( const unsigned int &space );
};

#endif
