/***************************************************************************
                          logic.h  -  description
                             -------------------
    begin                : Sat Sep 29 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LOGIC_H
#define LOGIC_H

#include <qstring.h>
#include "definitions.h"
#include "resource.h"
#include "match_param.h"

/**
  *@author Troy Corbin Jr.
  */

const unsigned char _ARRAY_SIZE_ = 74;
const unsigned char CF_King      = 0x01;
const unsigned char CF_RookQ     = 0x02;
const unsigned char CF_RookK     = 0x04;

class logic
{

public:
    signed char OnMove;
    Position    current[_ARRAY_SIZE_];
    Chessman    chessman[_ARRAY_SIZE_];
    ChessMove   chessMove;

    /* This is used for the 50 move rule */
    unsigned char MoveCounter;

    logic( resource *Rsrc=0, match_param *param=0 );
    ~logic();
    void        init( const int &Var=GameType::Standard );

    inline int  variation( void ) const
    {
        return Variation;
    }

    void        setBoard( const QString &board=QString::null, const short ppf=-2 );
    void        setBoardFromFen( const QString &fen );

    QString     board( void );
    bool        move( void );
    void        hashMoves( const char &Man, const bool &Recursion=false );
    int         pointer( const char &File, const char &Rank ) const;
    int         recievePiece( const ChessMove &receved );
    void        recieveSideboard( const QString &side );

    bool        isChessman( const char ChessmanPtr );
    bool        isCheck( const bool Army );
    bool        isLegal( const bool Army );
    bool        isDraw( const bool Army );

    bool        parseCAN( const bool Army );
    bool        parseSAN( void );
    void        writeCAN( void );
    void        writeSAN( void );

protected:
    int         calcPointer( const char File, const char Rank );
    void        _hashMoves( const bool &Recursion=false );
    void        checkNormal( void );
    void        checkSuicide( void );
    void        clearBoard( void );
    int         getKing( const bool Army );
    void        init_Standard( void );
    void        init_Suicide( void );
    void        init_House( void );
    inline void copyPositions( Position *Src, Position *Dst )
    {
        memcpy( Dst, Src, sizeof(Position) * _ARRAY_SIZE_ );
    }
    inline void copyChessmen( Chessman *Src, Chessman *Dst )
    {
        memcpy( Dst, Src, sizeof(Chessman) * _ARRAY_SIZE_ );
    }

private:
    resource    *Resource;
    match_param *Param;
    int         Variation;
    Position    hash[_ARRAY_SIZE_];
    Position    hashBackup[_ARRAY_SIZE_];
    Chessman    chessmanBackup[_ARRAY_SIZE_];
    unsigned char CastleFlag[2];
    unsigned char enPassant[2];
    unsigned char ManPtr;
};

#endif
