/***************************************************************************
                          match.h  -  description
                             -------------------
    begin                : Mon Jul 2 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MATCH_H
#define MATCH_H

#include <qobject.h>
#include "resource.h"
#include "match_param.h"
#include "command.h"
#include "chessclock.h"
#include "logic.h"
#include "pgn.h"

class board_base;
class core;
class KPopupMenu;

/**
  *@author Troy Corbin Jr.

  Match is the nexus around which other classes gather to create a game of chess.
  The GUI ( board_base ), the communications ( core ), the logic ( logic ), and
  the recordkeeper ( pgn ) all tie in to each other through this class.
  */

class match : public QObject
{
    Q_OBJECT

private:
    resource      *myResource;
    board_base    *Board;
    pgn           *Record;
    logic         *Logic;
    chessclock    *Clock;
    KPopupMenu    *m_match;
    KPopupMenu    *m_draw;
    ChessMove     chessMove;
    ChessMove     preMove;
    bool          preMoved;
    bool          Paused;
    bool          Loading;
    bool          JustMoved;
    bool          Current;
    int           myID;
    int           StatusCode;
    int           loadTimer;
    QString       StatusMessage;
    char          ICSGameMode;
    char          Draw_Offered[2];
    WFL           myFeatureList;

public:
    match( core *parent, match_param *param, resource *Rsrc );
    ~match();
    bool          save( const bool Prompt, const bool SaveAs );
    char          inputOnMove( bool reverse=false );
    char          input( char army );
    QString       url( void );
    void          retract( void );
    void          setPaused( const bool State );
    void          setModified( const bool &State );
    /* Inline Function */
    int           getID( void )
    {
        return myID;
    }
    void          setID( const int &ID )
    {
        myID = ID;
    }
    char          onMove( void )
    {
        return Logic->OnMove;
    }
    bool          flag( const bool &Army )
    {
        return Clock->Flag[Army];
    }
    void          setCurrent( const bool &State )
    {
        Current = State;
    }
    void          print( void )
    {
        Record->print();
    }
    bool          loading( void )
    {
        return Loading;
    }
    int           centiseconds( const bool &Army )
    {
        return Clock->getCentiseconds( Army );
    }
    QStringList*  notation( void )
    {
        return Record->notation();
    }
    void          setStatusMessage( const int &ID, const QString &MSG=QString::null )
    {
        emit setStatusBar( ID, MSG );
    }
    match_param*  getParam( void )
    {
        return Record->Param;
    }
    QWidget*      getBoard( void )
    {
        return (QWidget*)Board;
    }

public slots:
    void          recvCMD( const Command &cmd );
    void          destroyMatch( bool& );
    void          tick( void );
    bool          load( const QString URL, const int pos=0 );
    void          loadSpecial( void );
    void          slot_Select( int );
    void          slot_Preview( int );
    void          slot_flagFell( const bool );
    void          review( int );
    void          parseMatchParam( void );
    void          requestWidgetFeatureList( void );

protected slots:
    void          recvMenuAction( int );
    bool          move( ChessMove );

signals:
    void          setNotation( void );
    void          setStatusBar( const int &ID, const QString &MSG=QString::null );
    void          sendCMD( const Command& );
    void          updateWidgetFeatureList(const WFL&);

protected:
    bool          move( void );
    void          clearSelections( void );
    void          drawPosition( const int );
    void          playSound( const int );
    void          _retract( void );
    void          createMenus( void );
};

#endif
