/***************************************************************************
                          matchhistory.cpp  -  description
                             -------------------
    begin                : Thu Sep 4 2003
    copyright            : (C) 2003 by The Knights Project
    email                : knights-general@lists.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>
#include "matchhistory.h"
#include "command.h"

MatchHistory::MatchHistory() : QObject(0,"MatchHistory")
{}
MatchHistory::~MatchHistory()
{}
///////////////////////////////////////
//
//  MatchHistory::add
//
///////////////////////////////////////
/** Use for recorded chess moves ( ie. from PGN ) */
void MatchHistory::add
    ( const ChessMove &move, const QString &board )
{
    add
        ( move, board, 0, 0 );
}
/** Use for recieving sideboard pieces */
void MatchHistory::add
    ( const ChessMove &move, const int &whiteTime, const int &blackTime )
{
    MatchEvent me;
    me.move = move;
    me.wholeBoard = QString("");
    me.type = Sideboard;
    me.whiteTime = whiteTime;
    me.blackTime = blackTime;
    eventList.append( me );
}
/** Use for flag events */
void MatchHistory::add
    ( const int &whiteTime, const int &blackTime )
{
    MatchEvent me;
    Command::clearMove( &me.move );
    me.wholeBoard = QString("");
    me.type = Flag;
    me.whiteTime = whiteTime;
    me.blackTime = blackTime;
    eventList.append( me );
}
/** Use for actual chess moves */
void MatchHistory::add
    ( const ChessMove &move, const QString &board, const int &whiteTime, const int &blackTime )
{
    MatchEvent me;
    me.move = move;
    me.wholeBoard = board;
    me.type = Move;
    me.whiteTime = whiteTime;
    me.blackTime = blackTime;
    eventList.append( me );
}
/** Use if you have an event structure ready to go */
void MatchHistory::add
    ( const MatchEvent &event )
{
    eventList.append( event );
}
///////////////////////////////////////
//
//  MatchHistory::undo
//
///////////////////////////////////////
void MatchHistory::undo( void )
{
    if( eventList.count() )
    {
        eventList.remove( eventList.fromLast() );
    }
}
///////////////////////////////////////
//
//  MatchHistory::count
//
///////////////////////////////////////
unsigned int MatchHistory::count( const EventType &et )
{
    int count = 0;
    if( et == Unknown )
    {
        count = eventList.count();
    }
    else
    {
        for( unsigned int tmp = 0; tmp < eventList.count(); tmp++ )
        {
            if( eventList[tmp].type == et )
            {
                ++count;
            }
        }
    }
    return count;
}
///////////////////////////////////////
//
//  MatchHistory::first
//
///////////////////////////////////////
MatchHistory::MatchEvent& MatchHistory::first( const EventType &et )
{
    if( et != Unknown )
    {
        for( unsigned int tmp = 0; tmp < eventList.count(); tmp++ )
        {
            if( eventList[tmp].type == et )
            {
                return eventList[tmp];
            }
        }
        kdWarning() << "MatchHistory::first: No event of the given EventType: " << (int)et << endl;
    }
    return eventList.first();
}
///////////////////////////////////////
//
//  MatchHistory::last
//
///////////////////////////////////////
MatchHistory::MatchEvent& MatchHistory::last( const EventType &et )
{
    if( et != Unknown )
    {
        for( int tmp = eventList.count() - 1; tmp > -1; tmp-- )
        {
            if( eventList[tmp].type == et )
            {
                return eventList[tmp];
            }
        }
        kdWarning() << "MatchHistory::last: No event of the given EventType: " << (int)et << endl;
        kdWarning() << "MatchHistory::last: Current number of Events: " << eventList.count() << endl;
    }
    return eventList.last();
}
///////////////////////////////////////
//
//  MatchHistory::fetch
//
///////////////////////////////////////
MatchHistory::MatchEvent& MatchHistory::fetch( unsigned int i, const EventType &et )
{
    if( et != Unknown )
    {
        unsigned int index = 0;
        for( unsigned int tmp = 0; tmp < eventList.count(); tmp++ )
        {
            if( eventList[tmp].type == et )
            {
                if( index == i )
                {
                    return eventList[tmp];
                }
                index++;
            }
        }
        kdWarning() << "MatchHistory::fetch: Index out of range for given EventType: " << (int)et << endl;
    }
    return eventList[i];
}
