/***************************************************************************
                          matchhistory.h  -  description
                             -------------------
    begin                : Thu Sep 4 2003
    copyright            : (C) 2003 by The Knights Project
    email                : knights-general@lists.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MATCHHISTORY_H
#define MATCHHISTORY_H

#include <qobject.h>
#include <qstring.h>
#include <qvaluelist.h>
#include "definitions.h"

/**
  *@author The Knights Team
  */

class MatchHistory : public QObject
{
    Q_OBJECT

public:

    enum EventType
    {
        Unknown,
        Move,
        Sideboard,
        Flag
    };

    typedef struct MatchEvent
    {
        EventType type;
        ChessMove move;
        int       whiteTime;
        int       blackTime;
        QString   wholeBoard;
        QString   comment;
        QString   RAV;
    };

protected:

    typedef QValueList<MatchEvent> MatchEventList;

    MatchEventList    eventList;

public:
    MatchHistory();
    ~MatchHistory();

    /** Use to get a count of all events in this History of the
        given type, or of all types if none specified */
    unsigned int      count( const EventType &et=Unknown );

    /** Use to clear this History */
    void              clear( void )
    {
        eventList.clear();
    }

    /** Use to retrieve a specific event, based on the specified
        type and the given index. If no type is given, all types
        will be considered. */
    MatchEvent&       fetch( unsigned int i, const EventType &et=Unknown );

    /** Use to retrieve the first event of the specified type,
        or simply the first event if no type is specified. */
    MatchEvent&       first( const EventType &et=Unknown );

    /** Use to retrieve the last event of the specified type,
        or simply the first event if no type is specified. */
    MatchEvent&       last( const EventType &et=Unknown );

    /** Use for actual chess moves */
    void              add
        ( const ChessMove &move, const QString &board, const int &whiteTime, const int &blackTime );

    /** Use for recorded chess moves ( ie. from PGN ) */
    void              add
        ( const ChessMove &move, const QString &board );

    /** Use for recieving sideboard pieces */
    void              add
        ( const ChessMove &move, const int &whiteTime, const int &blackTime );

    /** Use for flag events */
    void              add
        ( const int &whiteTime, const int &blackTime );

    /** Use if you have an event structure ready to go */
    void              add
        ( const MatchEvent &event );

    /** Undo last event. Use for 'Retract' feature */
    void              undo( void );
};

#endif
