/***************************************************************************
                          menubar.cpp  -  Knights MenuBar implementation
                             -------------------
    begin                : Mon Mar 3 2003
    copyright            : (C) 2003-2005 by The Knights Project
    email                : knights-general@lists.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kpopupmenu.h>
#include <khelpmenu.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kicontheme.h>
#include <kaboutdialog.h>
#include <kstddirs.h>
#include <kaction.h>
#include <qfile.h>
#include <qtextstream.h>

#include "core.h"
#include "menubar.moc"
#include "resource.h"

MenuBar::MenuBar(QWidget *parent, const char *name, resource *resource) :
  KMenuBar(parent, name),
  m_resource(resource),
  m_fileMenu(0),
  m_settingsMenu(0),
  m_tutorialMenu(0),
  m_windowMenu(0),
  m_newTabMenu(0),
  m_customMenu(0)
{
  createAboutDialog();

  m_helpHelpMenu = new KHelpMenu(this, 0, true);

  connect(m_helpHelpMenu, SIGNAL(showAboutApplication()), m_aboutKnightsDialog, SLOT(show()));
  m_helpMenu = m_helpHelpMenu->menu();

  m_actionCollection = new KActionCollection(this, "actionCollection");

  createMenus();
}

MenuBar::~MenuBar()
{
  destroyMenus();
  m_actionCollection->clear();

  delete m_actionCollection;
  delete m_helpHelpMenu;
  delete m_aboutKnightsDialog;
}

void MenuBar::changeContextMenu(KPopupMenu *menu, QString &caption)
{
  m_customMenu = menu;
  m_customCaption = caption;
}

void MenuBar::removeContextMenu()
{
  m_customMenu = 0;
  m_customCaption = QString::null;
}

void MenuBar::resetBar()
{
  clear();
  insertItem(i18n("&File"), m_fileMenu);
  if(m_customMenu)
  {
    insertItem(m_customCaption, m_customMenu);
  }
  insertItem(i18n("&Settings"), m_settingsMenu);
  insertItem(i18n("&Window"), m_windowMenu);
  // insertItem(i18n("&Tutorials"), m_tutorialMenu);
  insertItem(i18n("&Help"), m_helpMenu);
}

void MenuBar::createMenus()
{
  destroyMenus();

  createFileMenu();
  createSettingsMenu();
  createWindowMenu();

  resetBar();
}

void MenuBar::destroyMenus()
{
  clear();

  delete m_fileMenu;
  delete m_settingsMenu;
  delete m_tutorialMenu;
  delete m_windowMenu;
  m_fileMenu = 0;
  m_settingsMenu = 0;
  m_tutorialMenu = 0;
  m_windowMenu = 0;
}

void MenuBar::createAboutDialog()
{
  m_aboutKnightsDialog = new KAboutDialog(KAboutDialog::AbtTabbed,
                                          i18n("Knights"),
                                          KDialogBase::Close,
                                          KDialogBase::Close,
                                          this,
                                          "aboutKnightsDialog");
  m_aboutKnightsDialog->addTextPage(i18n("&About"),                readTextFile("appdata", "about.kml"), true, 14);
  m_aboutKnightsDialog->addTextPage(i18n("A&uthors"),              readTextFile("appdata", "authors.kml"), true, 14);
  m_aboutKnightsDialog->addTextPage(i18n("&Thanks To"),            readTextFile("appdata", "thanks.kml"), true, 14);
  m_aboutKnightsDialog->addLicensePage(i18n("&License Agreement"), readTextFile("data", "LICENSES/GPL_V2"), 14);
}

void MenuBar::createFileMenu()
{
  m_fileMenu = new KPopupMenu(this , "fileMenu");

  m_fileMenu->insertItem(KGlobal::iconLoader()->loadIconSet("filenew", KIcon::Small),
                         i18n("&New Match..."),     this, SIGNAL(selected(int)), CTRL+Key_N, MENU_NEW_GAME);
  m_fileMenu->insertItem(KGlobal::iconLoader()->loadIconSet("fileopen", KIcon::Small),
                         i18n("&Load Match..."),    this, SIGNAL(selected(int)), CTRL+Key_L, MENU_LOAD);

  m_fileMenu->insertSeparator();

  m_fileMenu->insertItem(KGlobal::iconLoader()->loadIconSet("filesave", KIcon::Small),
                         i18n("&Save Match"),       this, SIGNAL(selected(int)), CTRL+Key_S, MENU_SAVE);
  m_fileMenu->insertItem(KGlobal::iconLoader()->loadIconSet("filesaveas", KIcon::Small),
                         i18n("Save Match &As..."), this, SIGNAL(selected(int)), CTRL+Key_A, MENU_SAVEAS);

  m_fileMenu->insertSeparator();

  if(m_resource->Core->isOnline())
  {
    m_fileMenu->insertItem(KGlobal::iconLoader()->loadIconSet("connect_no", KIcon::Small),
                           i18n("Disconnect from Server"), this, SIGNAL(selected(int)), CTRL+Key_C, MENU_CONNECT);
  }
  else
  {
    m_fileMenu->insertItem(KGlobal::iconLoader()->loadIconSet("connect_creating", KIcon::Small),
                           i18n("Connect to Server"),      this, SIGNAL(selected(int)), CTRL+Key_C, MENU_CONNECT);
  }

  m_fileMenu->insertSeparator();

  m_fileMenu->insertItem(KGlobal::iconLoader()->loadIconSet("fileprint", KIcon::Small),
                         i18n("&Print..."),         this, SIGNAL(selected(int)), CTRL+Key_P, MENU_PRINT);

  m_fileMenu->insertSeparator();

  m_fileMenu->insertItem(KGlobal::iconLoader()->loadIconSet("fileclose", KIcon::Small),
                         i18n("&Close Tab"),        this, SIGNAL(selected(int)), CTRL+Key_W, MENU_CLOSE);
  m_fileMenu->insertItem(KGlobal::iconLoader()->loadIconSet("exit", KIcon::Small),
                         i18n("&Quit"),             this, SIGNAL(selected(int)), CTRL+Key_Q, MENU_QUIT);
}

void MenuBar::createSettingsMenu()
{
  m_settingsMenu = new KPopupMenu(this, "settingsMenu");

  m_settingsMenu->insertItem(i18n("&Install Themes"),            this, SIGNAL(selected(int)), 0, MENU_INSTALL_THEMES);
  m_settingsMenu->insertItem(KGlobal::iconLoader()->loadIconSet("key_bindings", KIcon::Small),
                             i18n("Configure &Key Bindings..."), this, SIGNAL(selected(int)), 0, MENU_BINDINGS_DIALOG);
  m_settingsMenu->insertItem(KGlobal::iconLoader()->loadIconSet("configure", KIcon::Small),
                             i18n("&Configure Knights..."),      this, SIGNAL(selected(int)), 0, MENU_SETTINGS_DIALOG);
}

void MenuBar::createTutorialMenu()
{
  m_tutorialMenu = new KPopupMenu(this, "tutorialMenu");
}

void MenuBar::createWindowMenu()
{
  if(m_newTabMenu != 0)
  {
    delete m_newTabMenu;
  }

  m_newTabMenu = new KPopupMenu(this, "newTabMenu");

  m_newTabMenu->insertItem(KGlobal::iconLoader()->loadIconSet("filenew", KIcon::Small),
                           i18n("New &Match..."), this, SIGNAL(selected(int)), CTRL+Key_N, MENU_NEW_GAME);

  if(m_resource->Core->isOnline())
  {
    m_newTabMenu->insertItem(i18n("New &Console"),      this, SIGNAL(selected(int)), 0, MENU_NEW_CONSOLE);
    m_newTabMenu->insertItem(i18n("New Matches &List"), this, SIGNAL(selected(int)), 0, MENU_NEW_SEEK_LIST);
  }

  if(m_windowMenu != 0)
  {
    delete m_windowMenu;
  }

  m_windowMenu = new KPopupMenu( this, "m_window");
  m_windowMenu->setCheckable( true );
  m_windowMenu->insertItem(KGlobal::iconLoader()->loadIconSet("tab_new", KIcon::Small),
                           i18n("&New Tab" ),           m_newTabMenu);
  m_windowMenu->insertItem(KGlobal::iconLoader()->loadIconSet("tab_remove", KIcon::Small),
                           i18n("&Close Current Tab" ), this, SIGNAL(selected(int)), CTRL+Key_W, MENU_CLOSE);
  m_windowMenu->insertItem(KGlobal::iconLoader()->loadIconSet("window_fullscreen", KIcon::Small),
                           i18n("&Fullscreen" ),        this, SIGNAL(selected(int)), CTRL+SHIFT+Key_F, MENU_FULLSCREEN);
  m_windowMenu->insertSeparator();
}

QString MenuBar::readTextFile(const QString &dataType, const QString &filename)
{
  QString f(locate(dataType, filename));
  QString text;
  if(!f.isEmpty())
  {
    QFile file(f);
    if(file.open(IO_ReadOnly))
    {
      QTextStream str(&file);
      text = str.read();
    }
  }
  text.replace("%imagepath%", KGlobal::dirs()->findResourceDir("appdata", "splash.png"));
  return text;
}

void MenuBar::createWindowMenu(QStringList &list, const int &current)
{
  createWindowMenu();
  for(uint i = 0; i < list.count(); i++)
  {
    m_windowMenu->insertItem(list[i], this, SIGNAL(selected(int)), 0, MENU_SELECT_WINDOW + i);
    if((signed) i == current)
    {
      m_windowMenu->setItemChecked(MENU_SELECT_WINDOW + i, true);
    }
  }
  resetBar();
}

void MenuBar::setItemEnabled(const int &item, const bool &enabled)
{
  m_fileMenu->setItemEnabled(item, enabled);
}
