/***************************************************************************
                          pgn.h  -  description
                             -------------------
    begin                : Mon Jul 30 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PGN_H
#define PGN_H

#include <klocale.h>
#include <kio/netaccess.h>
#include <qobject.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qvaluelist.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpainter.h>
#include "definitions.h"
#include "resource.h"
#include "knightsmap.h"
#include "match_param.h"

/**
  *@author Troy Corbin Jr.
  */

typedef KnightsMap<QString, QString> PGNTags;
class tab_pgnView;
class MatchHistory;

class pgn : public QObject
{
    Q_OBJECT
public:

    /* The match_param */
    match_param         *Param;

    /* PGN Tags now stored in a KnightsMap */
    PGNTags             tags;

    QString             CurrentURL;
    int                 File_Position; // Used to scan PGN file
    QStringList         Move_Data;

    /* These are used in unfinished save games */
    int                 whiteTime;
    int                 blackTime;
    TCPList             whiteTCP;
    TCPList             blackTCP;

    unsigned int        currentIndex;
    MatchHistory        *history;

    pgn( resource *Rsrc=0, match_param *param=NULL );
    ~pgn();
    void                clear( void );
    void                init( void );
    int                 scan( void );
    bool                load( const int pos=0 );
    bool                save( QString& URL );
    bool                open( const QString& URL );
    bool                sendEmail( const int& onMove );
    void                print( void );
    QStringList*        notation( const int format=1 );
    void                close( void );
    static QString      getNAG( int num );
    QString             caption( void );
    void                refresh( void ); // Update PGNView because a move was made

signals:
    void                processMove( ChessMove );
    void                processSpecial( void );
    void                requestReview( int );

public slots:
    bool                loadNext( void );
    void                parseMatchParam( void );
    void                childViewDestroyed( void );
    void                parseURL( const QString& );

private:
    resource            *Resource;
    tab_pgnView         *pgnView;
    QString             tempFile;
    QString             currentLine;
    QFile               File;
    QTextStream         Input;

    QChar               getch( void );
    QString             getword( void );
    void                parseTag( void );
    void                parseAnnotation( void );
    void                parseRAV( void );
    void                parseKnightsData( void );
};

#endif
