/***************************************************************************
                          seekmatchtab.cpp  -  Allows the user to seek matches.
                             -------------------
    begin                : Sat Sep 14 2002
    copyright            : (C) 2003 by Troy Corbin Jr.
                           (C) 2005 by Arend van Beelen jr.
    email                : tcorbin@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kiconloader.h>
#include <kpopupmenu.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qregexp.h>
#include <qspinbox.h>

#include "core.h"
#include "resource.h"
#include "seekmatchtab.moc"
#include "seekmatchwidget.h"

SeekMatchTab::SeekMatchTab(QWidget *parent, const char *name, resource *resource) :
  SeekMatchWidget(parent, name),
  m_resource(resource)
{
  m_seekMenu = new KPopupMenu(this);
  m_seekMenu->setCheckable(true);
  m_seekMenu->insertItem(i18n("Accept This Match"),  this, SLOT(menuItemClicked(int)), 0, MENU_ACCEPT_MATCH);
  m_seekMenu->insertItem(i18n("Tell..."),            this, SLOT(menuItemClicked(int)), 0, MENU_TELL);
  m_seekMenu->insertItem(i18n("Assess..."),          this, SLOT(menuItemClicked(int)), 0, MENU_ASSESS);
  m_seekMenu->insertItem(i18n("Player Info"),        this, SLOT(menuItemClicked(int)), 0, MENU_FINGER);
  m_seekMenu->insertItem(KGlobal::iconLoader()->loadIconSet("history", KIcon::Small),
                         i18n("Player History"),     this, SLOT(menuItemClicked(int)), 0, MENU_HISTORY);
  m_seekMenu->insertSeparator();
  m_seekMenu->insertItem(i18n("Add to Friends"),     this, SLOT(menuItemClicked(int)), 0, MENU_NOTIFY);
  m_seekMenu->insertItem(i18n("Ignore This Player"), this, SLOT(menuItemClicked(int)), 0, MENU_CENSOR);

  connect(soughtMatchesList, SIGNAL(contextMenuRequested(QListViewItem *, const QPoint &, int)),
          SLOT(showSeekMenu(QListViewItem *, const QPoint &, int)));
  connect(soughtMatchesList, SIGNAL(doubleClicked(QListViewItem *)),
          SLOT(selectMatch(QListViewItem *)));
  connect(showSeeksButton, SIGNAL(toggled(bool)),
          SLOT(showSeeksButtonToggled()));
  connect(showGraphButton, SIGNAL(clicked()),
          SLOT(showGraphButtonClicked()));
  connect(seekButton, SIGNAL(clicked()),
          SLOT(seekButtonClicked()));
}

void SeekMatchTab::recvCMD(const Command &command)
{
  switch(command.getCommand())
  {
    case CMD_Add_Sought_Match:
      addSoughtMatch(command.getData());
      break;

		/* We must block signals when toggling the Seek button here. Otherwise
		   pressing the button when multiple SeekMatchTabs are open will lead
			 to infinite toggle loops. */
    case CMD_Show_Sought_List:
      updateSoughtMatchesList();
			if( !showSeeksButton->isOn() )
			{
				blockSignals( true );
				showSeeksButton->setOn( true );
				blockSignals( false );
			}
      break;

		/* Ditto */
    case CMD_Hide_Sought_List:
      soughtMatchesList->clear();
			if( showSeeksButton->isOn() )
			{
				blockSignals( true );
				showSeeksButton->setOn( false );
				blockSignals( false );
			}
      break;
  }
}

void SeekMatchTab::updateSoughtMatchesList()
{
  soughtMatchesList->clear();
  for(uint i = 0; i < m_sf1.count(); i++)
  {
    new QListViewItem(soughtMatchesList,
                      m_sf1[i],
                      m_sf2[i],
                      m_sf3[i],
                      m_sf4[i],
                      i18n("%1 min.").arg(m_sf5[i]),
                      i18n("%1 sec.").arg(m_sf6[i]),
                      m_sf7[i]);
  }
  m_sf1.clear();
  m_sf2.clear();
  m_sf3.clear();
  m_sf4.clear();
  m_sf5.clear();
  m_sf6.clear();
  m_sf7.clear();
}

void SeekMatchTab::addSoughtMatch(const QString &source)
{
  QStringList fields = QStringList::split(QChar(' '), source, false);
  m_sf1 << fields[2];
  m_sf2 << fields[1];
  m_sf3 << fields[6];
  m_sf4 << fields[5];
  m_sf5 << fields[3];
  m_sf6 << fields[4];
  m_sf7 << fields[0];
}

void SeekMatchTab::showSeekMenu(QListViewItem *item, const QPoint &position, int)
{
  if(item == 0)
  {
    return;
  }

  soughtMatchesList->setSelected(item, true);
  m_selectedPlayerName = item->text(0).replace(QRegExp("\\(.+\\)"), QString(""));
  m_selectedMatchId = item->text(6).toInt();

  m_seekMenu->popup(position);
}

void SeekMatchTab::selectMatch(int matchId)
{
  if(matchId)
  {
    emit sendCMD(Command(0, CMD_Start_Match, QString::number(matchId)));
  }
}

void SeekMatchTab::selectMatch(QListViewItem *item)
{
  selectMatch(item->text(6).toInt());
}

void SeekMatchTab::menuItemClicked(int item)
{
  switch(item)
  {
    case MENU_FINGER:
      emit sendCMD(Command(0, CMD_Player_Finger, m_selectedPlayerName));
      break;

    case MENU_TELL:
      emit sendCMD(Command(0, CMD_Set_Input, QString("tell %1 ").arg(m_selectedPlayerName)));
      break;

    case MENU_NOTIFY:
      emit sendCMD(Command(0, CMD_Add_Friend, m_selectedPlayerName));
      break;

    case MENU_CENSOR:
      emit sendCMD(Command(0, CMD_Ignore_Player, m_selectedPlayerName));
      break;

    case MENU_HISTORY:
      emit sendCMD(Command(0, CMD_Player_History, m_selectedPlayerName));
      break;

    case MENU_ACCEPT_MATCH:
      emit sendCMD(Command(0, CMD_Start_Match, QString::number(m_selectedMatchId)));
      break;

    case MENU_ASSESS:
      emit sendCMD(Command(0, CMD_Assess, m_selectedPlayerName));
      break;
  }
}

void SeekMatchTab::showSeeksButtonToggled()
{
  emit sendCMD(Command(0, CMD_Toggle_Seek));
}

void SeekMatchTab::showGraphButtonClicked()
{
  m_resource->Core->createNewTab(MENU_NEW_SEEK_GRAPH);
}

void SeekMatchTab::seekButtonClicked()
{
  QString command = QString("%1 %2 %3").arg(baseTimeSpin->value()).arg(incrementSpin->value()).arg(ratedCheckbox->isChecked() ? "rated" : "unrated");

  switch(colorCombo->currentItem())
  {
    case 1:
      command += " white";
      break;

    case 2:
      command += " black";
      break;
  }

  switch(matchTypeCombo->currentItem())
  {
    case 1:
      command += " crazyhouse";
      break;

    case 2:
      command += " suicide";
      break;
  }

  emit sendCMD(Command(0, CMD_Seek_Match, command));
}
