/***************************************************************************
                          server_base.h  -  description
                             -------------------
    begin                : Tue Jul 29 2003
    copyright            : (C) 2003 by Alexander Wels
    email                : dutchy@sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _SERVER_BASE_H
#define _SERVER_BASE_H

#include "command.h"
#include "resource.h"
#include <qobject.h>

class resource;
class match_param;

class server_base : public QObject
{
    Q_OBJECT

public:
    enum parserType
    {
        STYLE1 = 0,
        STYLE2 = 1,
        STYLE3 = 2,
        STYLE4 = 3,
        STYLE5 = 4,
        STYLE6 = 5,
        STYLE7 = 6,
        STYLE8 = 7,
        STYLE9 = 8,
        STYLE10 = 9,
        STYLE11 = 10,
        STYLE12 = 11,
        STYLE13 = 12
    };

    enum serverType
    {
        FICS = 0,
        ICC = 1
    }; /* don't know of any other types at this moment, maybe chess.net? */

    enum loginStages
    {
        LOGIN_STAGE_NAME,
        LOGIN_STAGE_PASSWORD,
        LOGIN_STAGE_LOGGED_IN
    };

    server_base();
    server_base(const resource *rsrc);
    virtual ~server_base();
    virtual int           getServerType() = 0;                    // depends on what the subclass supports
    virtual void          sendUserSettings() = 0;
public slots:
    virtual void          processData(const QString& line) = 0;   // sub classes have to implement this.
    virtual void          acceptChallenge( Challenge* ) = 0;      // accept the currently outstanding challenge.
    virtual void          declineChallenge( Challenge* ) = 0;   // decline the currently outstanding challenge.
    virtual void          processSeekTimer( void ) = 0;           // process the timer.
signals:
    void                  sendCMD(const Command& command);
    void                  newChallenge( Challenge* );
    void                  withDrawChallenge();
    void                  newMatch(const match_param*, int);
    void                  sendMove(const ChessMove&, const QString&, int, int, int);
protected:

    resource              *myResource;
    int                   retryCount;
    void                  safeStrCpy( char*, QString& );
};

#endif /* _SERVER_BASE_H */

