/***************************************************************************
                          server_fics.cpp  -  description
                             -------------------
    begin                : Wed Jul 30 2003
    copyright            : (C) 2003 by Alexander Wels
    email                : dutchy@sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "server_fics.moc"

/* KDE */
#include <kmessagebox.h>
#include <kdebug.h>

/* Qt */
#include <qregexp.h>

/* Local */
#include "resource.h"
#include "dlg_login.h"
#include "dlg_challenge.h"
#include "match.h"
#include "audio.h"

/////////////////////////////////////////////////
//
//  server_fics::server_fics(const resource *rsrc, const io_internet* parent)
//
/////////////////////////////////////////////////
server_fics::server_fics(const resource *rsrc, const io_internet* parent)
{
    myResource = (resource*)rsrc;
    myParent = (io_internet*)parent;
    loginStage = LOGIN_STAGE_NAME;
}

/////////////////////////////////////////////////
//
//  server_fics::~server_fics()
//
/////////////////////////////////////////////////
server_fics::~server_fics()
{
    myResource = NULL;
    myParent = NULL;
}

/////////////////////////////////////////////////
//
//  server_fics::processData(const QString& line)
//
/////////////////////////////////////////////////
void server_fics::processData(const QString& line)
{
	QString myLine = line;

	/* Remove Bells, prompts, and other garbage */
	myLine = myLine.remove( QRegExp("\\x0007") );
	myLine = myLine.remove( "\r" );
	myLine = myLine.remove( "\f" );
	myLine = myLine.remove("fics% ");

	if( !myLine.isEmpty() )
	{
		if(loginStage != LOGIN_STAGE_LOGGED_IN)
		{
			parseLoginData(myLine);
		}
		else
		{
			switch(parseMode)
			{
				case NORMAL_MODE:
				case AWAITING_MOVE_LIST_MODE:
						parseNormalMode(myLine);
						break;
				case UPDATE_SOUGHT_MODE:
						parseUpdateSoughtMode(myLine);
						break;
				case OBSERVE_GAME_MODE:
						parseObserveGame(myLine);
						break;
				case CHALLENGE_MODE:
						break;
				case PARSE_MOVE_LIST_MODE:
						parseMoveList(myLine);
						break;
				case NEW_GAME_MODE:
						parseNewGame(myLine);
						break;
				case VARIABLE_LIST_MODE:
						parseVariableList(myLine);
						break;
			}
		}
	}
}

///////////////////////////////////////
//
//  server_fics::parseLoginData
//
///////////////////////////////////////
void server_fics::parseLoginData(const QString& data)
{
    QStringList lines;

    if(loginStage == LOGIN_STAGE_NAME)
    {
        if(data.contains("If you are not a registered player, enter guest or a unique ID."))
        {
            myParent->sendUserName();
            loginStage = LOGIN_STAGE_PASSWORD;
        }
    }
    else if(loginStage == LOGIN_STAGE_PASSWORD)
    {
        if(data.contains("If not, just hit return to try another name."))
        {
            myParent->sendPassword();
        }
        else if(data.contains("**** Invalid password! ****"))
        {
            if(retryCount > 2)
            {
                //too many retries, display error and disconnect from the server.
                KMessageBox::sorry( NULL, i18n( "Knights is unable to connect to the server.\n Too many retries."), i18n( "Cannot connect to server."));
                myParent->selfDestruct();
                return;
            }
            retryCount++;
            dlg_login *loginDlg = new dlg_login( NULL, "LoginDialog", myResource);
            loginDlg->disableServerSelect();
            connect(loginDlg, SIGNAL(okClicked()), myParent, SLOT(sendUserName()));
            connect(loginDlg, SIGNAL(cancelClicked()), myParent, SLOT(selfDestruct()));
            connect(loginDlg, SIGNAL(login(const QString&, const QString&)), myParent, SLOT(setUserInfo(const QString&, const QString&)));
        }
        else if(data.contains("**** Starting FICS session as "))
        {
            loginStage = LOGIN_STAGE_LOGGED_IN;
            parseMode = NORMAL_MODE;
            sendUserSettings();
        }
        else if(data.contains("you may use this name to play unrated games"))
        {
            QRegExp guestName("Logging you in as \"(\\w*)\"");
            int pos = guestName.search(data);
            if(pos > -1)
            {
                myParent->setUserInfo(guestName.cap(1), "");
            }
            myParent->send("\n", false);
        }
    }
    lines = QStringList::split( "\n", data, false );
    for(QStringList::iterator i = lines.begin(); i != lines.end(); i++)
    {
        myParent->writeToConsole((*i), "K_STD");
    }
}

///////////////////////////////////////
//
//  server_fics::parseNormalMode
//
///////////////////////////////////////
void server_fics::parseNormalMode(const QString& line)
{
    int i(0), j(0);
		QString myLine = line;
    QString tmp;

    if( myLine.startsWith("<12> ") )
    {
        // a game move
        parseStyle12(myLine, PARSE12_MODE_MOVE);
    }
    // CHALLENGE
    else if( myLine.startsWith("Challenge: "))
    {
        parseChallenge(myLine);
    }
    else if(myLine.endsWith(" withdraws the match offer") ||
            myLine.contains(QRegExp("Challenge from \\w+ removed.")) ||
            myLine.startsWith("You decline the match offer from ")
           )
    {
        emit withDrawChallenge();
        myParent->writeToConsole(myLine, "K_STD");
        lastTag = "K_STD";
    }
    else if(myLine.startsWith("<b1> "))
    {
        // a bughouse piece has been passed or a piece has been captured in crazyhouse
        parseReceivePiece(myLine);
    }
    else if(myLine.startsWith("\\"))
    {
        myParent->writeToConsole(myLine, lastTag);
    }
    // DRAW OFFER
    else if(myLine.endsWith(" offers you a draw.") )
    {
        myParent->writeToConsole(myLine, "K_WSP");
        lastTag = "K_WSP";
    }
    else if(myLine.startsWith("You are now observing game"))
    {
        parseMode = OBSERVE_GAME_MODE;
    }
    else if(myLine.startsWith("Movelist for game"))
    {
        parseMode = PARSE_MOVE_LIST_MODE;
    }
    // SOUGHT GAME
    else if(myLine.contains("seeking"))
    {
        myParent->writeToConsole(myLine, "K_SHT");
    }
    // Important System Messages: Use Whisper Color
    else if(myLine.contains(QRegExp("declines the draw request\\.$" )))
    {
        myParent->writeToConsole(myLine, "K_WSP");
        lastTag = "K_WSP";
    }
    else if( myLine.contains( QRegExp(".+rating adjustment:.+" ) ) )
    {
        myParent->writeToConsole(myLine, "K_WSP");
        lastTag = "K_WSP";
    }
    // SHOUTS
    else if( myLine.contains( QRegExp("c?t?s?-?shouts: ") ) )
    {
        myParent->writeToConsole(myLine, "K_SHT");
        lastTag = "K_SHT";
    }
    // NOTIFY
    else if((myLine.contains(QRegExp("\\s*Notification:"))) ||
            (myLine.contains(QRegExp("\\s*Present company includes:"))) ||
            (myLine.contains(QRegExp("\\s*Your arrival was noted by:"))))
    {
        myParent->writeToConsole(myLine, "K_NOT");
        lastTag = "K_NOT";
        myResource->play( Sound::Notification );
    }
    // CHANNEL TELLS
    else if(myLine.contains(QRegExp("^\\s*\\w+(?:\\(.+\\))*\\(\\d+\\):")))
    {
        // First grab the channel # so we can auto-respond later
        j = myLine.find(QString("):"));
        i = myLine.findRev(QString("("), j) + 1;
        emit sendCMD( Command( 0, CMD_Set_Src_Channel, myLine.mid(i, j - i) ) );
        myParent->writeToConsole(myLine, "K_CH");
        lastTag = "K_CH";
    }
    else if(myLine.contains(QRegExp("^\\{?Game \\d+")) && (myLine.contains("Creating", true) || myLine.contains("Continuing", true)))
    {
        parseNewGame(myLine);
        parseMode = NEW_GAME_MODE;
    }
    else if((myLine.contains(QRegExp("^\\{?Game \\d+")) || myLine.contains(QRegExp("Game \\d+"))) &&
            (
                myLine.contains(" forfeits by disconnection", true) ||
                myLine.contains(" forfeits on time", true) ||
                myLine.contains(" resigns", true) ||
                myLine.contains(" Game drawn by mutual agreement ", true) ||
                myLine.contains(", has lost contact or quit.", true) ||
                myLine.contains(" checkmated", true) ||
                myLine.contains("lost connection", true) ||
                myLine.contains("has no material to mate", true) ||
                myLine.contains("Game aborted by mutual agreement", true) ||
                myLine.contains("Game adjourned by mutual agreement", true) ||
                myLine.contains("Neither player has mating material", true) ||
                myLine.contains("Your opponent has aborted the game on move one", true) ||
                myLine.contains("has no material to mate", true) ||
                myLine.contains("Game drawn by stalemate", true)
            )
           )
    {
        myParent->writeToConsole(myLine, "K_STD");
        sendEndOfGameCommand(myLine);
    }
    // PRIVATE TELL
    else if(myLine.contains(QRegExp("^\\w+(?:\\(.+\\))* tells you: .*")))
    {
        // First grab the user name so we can auto-respond later
        QRegExp rx("^(\\w+)(?:\\(.+\\))* tells you: .*");
        rx.search(myLine);
        emit sendCMD(Command(0, CMD_Set_Src_Tell, rx.cap(1)));
        myResource->play( Sound::Tell );
        myParent->writeToConsole(myLine, "K_PVT");
        lastTag = "K_PVT";
    }
    // SAY
    else if(myLine.contains( QRegExp("^\\s*\\w+(:?\\(.+\\))* says: .*")))
    {
        myResource->play(Sound::Say);
        myParent->writeToConsole(myLine, "K_PVT");
        lastTag = "K_PVT";
        return;
    }
    // WHISPER & KIBITZ
    else if(myLine.contains(QRegExp("^\\s*\\w+(:?\\(.+\\))+\\[\\d+\\] whispers: .*")) || myLine.contains(QRegExp("^\\s*\\w+(:?\\(.+\\))+\\[\\d+\\] kibitzes: .*")))
    {
        myParent->writeToConsole(myLine, "K_WSP");
        lastTag = "K_WSP";
    }
    else if(myLine.startsWith("You are no longer examining game "))
    {
        QRegExp rx("You are no longer examining game (\\d+)");
        rx.search(myLine);
        Command command(rx.cap(1).toInt(), CMD_Close_Match);
        emit sendCMD(command);
    }
    else if(myLine.contains(QRegExp("^\\s*\\d{1,3}\\s+(?:\\d{1,4}|\\+\\+\\+\\+|\\-\\-\\-\\-)\\s+\\w{3,17}(\\(C\\))?\\s+\\d{1,3}\\s+\\d{1,3}")))
    {
        parseMode = UPDATE_SOUGHT_MODE;
        parseUpdateSoughtMode(myLine);
    }
		// Getting Variables for a player
		else if( myLine.startsWith( "Variable settings of " ) )
		{
			parseVariableList( myLine );
		}
    else
    {
        myParent->writeToConsole(myLine, "K_STD");
        lastTag = "K_STD";
    }
}

///////////////////////////////////////
//
//  server_fics::parseUpdateSoughtMode
//
///////////////////////////////////////
void server_fics::parseUpdateSoughtMode(const QString& line)
{
    myParent->updateSoughtList(line);
    if(line.contains(QRegExp("\\d+\\s+ads? displayed.")))
    {
        parseMode = NORMAL_MODE;
    }
}

///////////////////////////////////////
//
//  server_fics::parseChallenge
//
///////////////////////////////////////
void server_fics::parseChallenge(const QString& line)
{
    QStringList list;
    Challenge *challenge = new Challenge;

    myParent->writeToConsole(line, "K_STD");

    QString myLine = line;
    myLine.remove( "(" );
    myLine.remove( ")" );
    list = QStringList::split(" ", myLine, false);

    challenge->whiteName = list[1].upper();
    challenge->blackName = list[3].upper();
    challenge->whiteRating = list[2];
    challenge->blackRating = list[4];
    playerRating.insert( challenge->whiteName, list[2], true );
    playerRating.insert( challenge->blackName, list[4], true );
		confirmInterface( list[1] );
		confirmInterface( list[3] );

    if( list[5] == "rated" )
    {
      challenge->rated = true;
    }
    else
    {
      challenge->rated = false;
    }

    challenge->ICSTypeName = list[6];

		challenge->baseTime = list[7].toInt();
		challenge->incTime = list[8].remove(".").toInt();

		if( list.count() > 9 )
		{
			challenge->oddsBaseTime = list[9].toInt();
			challenge->oddsIncTime = list[10].toInt();
		}
		else
		{
			challenge->oddsBaseTime = 0;
			challenge->oddsIncTime = 0;
		}

    if(list[6] == "wild")
    {
        kdDebug() << line << endl;
        list[6] = "w" + list[11].mid(5,1);
    }

		challenge->changed = false;

    emit newChallenge( challenge );
}

///////////////////////////////////////
//
//  server_fics::parseNewGame()
//
///////////////////////////////////////
void server_fics::parseNewGame(const QString& line)
{
    QStringList fields;

    if(line.startsWith("<12> "))
    {
        // a game move
        parseStyle12(line, PARSE12_MODE_NEW);
        emit sendCMD(Command(0, CMD_Hide_Sought_List));
        myParent->stopTimer();
        parseMode = NORMAL_MODE;
    }
    else
    {
        fields = QStringList::split( QChar(' '), line, false );
        determineGameType(fields[7]);
    }
}

///////////////////////////////////////
//
//  server_fics::parseObserverGame()
//
///////////////////////////////////////
void server_fics::parseObserveGame(const QString& line)
{
    QStringList fields;

    if(line.startsWith("<12> "))
    {
        fields = QStringList::split( QChar(' '), line, false );
        //This only works for normal starting positions, CHANGE ME.
        QString newGame = "<12> rnbqkbnr pppppppp -------- -------- -------- -------- PPPPPPPP RNBQKBNR W -1 1 1 1 1 0 ";
        newGame += fields[16];
        newGame += " ";
        newGame += fields[17];
        newGame += " ";
        newGame += fields[18];
        newGame += " 1 2 12 39 39 120 120 1 none (0:00) none 0 0 0";
        observeGameNr = fields[16].toInt();
        parseStyle12(newGame, PARSE12_MODE_NEW);
        if( ( gameType == GameType::BugHouse ) || ( gameType == GameType::CrazyHouse ) )
        {
          parseReceivePiece( QString( "<b1> game %1 white [] black []" ).arg(observeGameNr) );
        }
        myParent->send("moves", true);
        parseMode = AWAITING_MOVE_LIST_MODE;
    }
    else
    {
        fields = QStringList::split( QChar(' '), line, false );
        determineGameType(fields[7]);
        fields[3].remove("(").remove(")");
        fields[5].remove("(").remove(")");
        playerRating.insert(fields[2].upper(), fields[3], true);
        playerRating.insert(fields[4].upper(), fields[5], true);
        myParent->writeToConsole(line, "K_STD" );
    }
}

///////////////////////////////////////
//
//  server_fics::acceptChallenge()
//
///////////////////////////////////////
void server_fics::acceptChallenge( Challenge *challenge )
{
	QString match = "$match ";

	if( myParent->getUserName().upper() == challenge->whiteName )
	{
		match += challenge->blackName;
	}
	else
	{
		match += challenge->whiteName;
	}

	if( challenge->rated )
	{
		match += " rated ";
	}
	else
	{
		match += " unrated ";
	}

	match += QString::number( challenge->baseTime ) + " ";
	match += QString::number( challenge->incTime  );

	if( challenge->oddsBaseTime )
	{
		match += " " + QString::number( challenge->oddsBaseTime );
		match += " " + QString::number( challenge->oddsIncTime  );
	}

	if( challenge->ICSTypeName != "blitz" &&
	    challenge->ICSTypeName != "standard" &&
	    challenge->ICSTypeName != "lightning" )
	{
		match += " " + challenge->ICSTypeName;
	}

	myParent->send( match, false );
}

///////////////////////////////////////
//
//  server_fics::declineChallenge()
//
///////////////////////////////////////
void server_fics::declineChallenge( Challenge *challenge )
{
	if( challenge->whiteName == myParent->getUserName().upper() )
	{
		myParent->send("decline " + challenge->blackName , false);
	}
	else
	{
		myParent->send("decline " + challenge->whiteName , false);
	}
}


///////////////////////////////////////
//
//  server_fics::ParseStyle12
//
///////////////////////////////////////
void server_fics::parseStyle12(const QString& line, const unsigned int mode)
{
    struct ChessMove move;
    QStringList fields;
    QString position_line = "";
    QString tmp = "";
    match_param *param = NULL;
    Command::clearMove(&move);

    if( parseMode != AWAITING_MOVE_LIST_MODE )
    {
      switch(mode)
      {
        case PARSE12_MODE_NEW:
          {
              // a new game that we are playing, or observing
              param = new match_param(this->myResource);
              fields = QStringList::split( QChar(' '), line, false );

              // set white time control
              TCPList tmpListWhite(param->time(WHITE));
              TCP tmpTCPWhite = tmpListWhite[0];
              tmpTCPWhite.Seconds = fields[20].toInt() * 60;
              tmpTCPWhite.Increment = fields[21].toInt();
              tmpListWhite[0] = tmpTCPWhite;
              param->setTime(WHITE, tmpListWhite);

              // set black time control
              TCPList tmpListBlack(param->time(BLACK));
              TCP tmpTCPBlack = tmpListBlack[0];
              tmpTCPBlack.Seconds = fields[20].toInt() * 60;
              tmpTCPBlack.Increment = fields[21].toInt();
              tmpListBlack[0] = tmpTCPBlack;
              param->setTime(BLACK, tmpListBlack);

              if((myParent->getUserName().upper() == fields[17].upper()) &&
                      ((fields[19].toInt() == -1) || (fields[19].toInt() == 1)))
              {
                  // I am playing white
                  param->setType(WHITE, PLAYERLOCAL);
              }
              else if(fields[19].toInt() != 2)
              {
                  // I am not playing white
                  param->setType(WHITE, PLAYERTCP);
									confirmInterface( fields[17] );
              }

              if((myParent->getUserName().upper() == fields[18].upper()) &&
                      ((fields[19].toInt() == -1) || (fields[19].toInt() == 1)))
              {
                  // I am playing black
                  param->setType(BLACK, PLAYERLOCAL);
              }
              else if(fields[19].toInt() != 2)
              {
                  // I am not playing black
                  param->setType(BLACK, PLAYERTCP);
									confirmInterface( fields[18] );
              }

              if(fields[19].toInt() == 2)
              {
                  param->setType(WHITE, PLAYEREXAMINE);
                  param->setType(BLACK, PLAYEREXAMINE);
              }
              param->setVariation(gameType);
              param->setName(WHITE, fields[17]);
              param->setName(BLACK, fields[18]);
              param->setRating(WHITE, playerRating[fields[17].upper()]);
              param->setRating(BLACK, playerRating[fields[18].upper()]);
              // tell core to connect us to a new match
              emit newMatch(param, fields[16].toInt());
          }
        case PARSE12_MODE_MOVE:
          {
              fields = QStringList::split( QChar(' '), line, false );
              // various ICS stuff for ChessMove
              move.ICS_ClockTicking = fields[31].toInt();
              move.ICS_PawnPushFile = fields[10].toShort();
              move.ICS_MoveCounter = fields[15].toInt();
              if(fields[9] == "W")
              {
                  move.ICS_OnMove = BLACK;
              }
              else
              {
                  move.ICS_OnMove = WHITE;
              }

              switch( fields[19].toInt() )
              {
                  // Examining a game
              case 2:
                  move.ICS_Mode = ICS_Examine;
                  if(myParent->isActiveGame(fields[16].toInt()))
                  {
                      // no new game started yet, call parsestyle 12 with a different mode
                      parseStyle12(line, PARSE12_MODE_NEW);
                      return;
                  }
                  break;
                  // Observing a game
              case -2:
              case 0:
                  move.ICS_Mode = ICS_Observe;
                  break;
                  // Playing a game
              default:
                  move.ICS_Mode = ICS_Normal;
                  break;
              }

              // Verbose Coordinate Notation of previous move ( USE AS CAN )
              tmp = fields[27].right( fields[27].length() - 2 ).replace( "-", "" );
              safeStrCpy( move.CAN, tmp );

              // SAN
              tmp = fields[29].replace( "+", "" ).replace( "#", "" );
              safeStrCpy( move.SAN, tmp );

              // fill the line for the command
              position_line += fields[1]; // Internal Rank #7
              position_line += fields[2]; // Internal Rank #6
              position_line += fields[3]; // Internal Rank #5
              position_line += fields[4]; // Internal Rank #4
              position_line += fields[5]; // Internal Rank #3
              position_line += fields[6]; // Internal Rank #2
              position_line += fields[7]; // Internal Rank #1
              position_line += fields[8]; // Internal Rank #0
              // Can White Castle Short (boolean)
              position_line += fields[11];
              // Can White Castle Long (boolean)
              position_line += fields[12];
              // Can Black Castle Short (boolean)
              position_line += fields[13];
              // Can Black Castle Long (boolean)
              position_line += fields[14];
              emit sendMove(move, (const QString&)position_line, fields[16].toInt(), fields[24].toInt() * 100, fields[25].toInt() * 100);
              break;
          }
        default:
          break;
      }
    }
}


///////////////////////////////////////
//
//  server_fics::parseMoveList(QString data)
//
///////////////////////////////////////
void server_fics::parseMoveList(const QString& line)
{
    if(line.contains(QRegExp("^\\s+\\d+\\.\\s+")))
    {
        QStringList two_plys;
        struct ChessMove move;
        Command command;
        int move_counter = 0;
        two_plys = QStringList::split( " ", line );
        move_counter = two_plys[0].left(two_plys[0].length() - 1).toInt(); // remove the .

        // white
        Command::clearMove(&move);
        move.ICS_MoveCounter = move_counter;
        move.ICS_Mode = ICS_Movelist;
        move.ICS_OnMove = WHITE;
        two_plys[1] = two_plys[1].replace( "+", "" ).replace( "#", "" );
        safeStrCpy(move.SAN, two_plys[1] );
        command.setCommand((int&)CMD_Move);
        //This is kind of hack to pass the gameID to io_internet
        command.setID(observeGameNr);
        command.setMove(move);
        emit sendCMD(command);

        // black
        if(two_plys.size() > 3)
        {
            Command::clearMove(&move);
            move.ICS_MoveCounter = move_counter + 1;
            move.ICS_Mode = ICS_Movelist;
            move.ICS_OnMove = BLACK;
            two_plys[3] = two_plys[3].replace( "+", "" ).replace( "#", "" );
            safeStrCpy(move.SAN, two_plys[3] );
            command.setCommand((int&)CMD_Move);
            //This is kind of hack to pass the gameID to io_internet
            command.setID(observeGameNr);
            command.setMove(move);
            emit sendCMD(command);
        }
    }
    else if(line.contains("{Still in progress}"))
    {
        myParent->send("$refresh", true);
        parseMode = NORMAL_MODE;
    }
}

///////////////////////////////////////
//
//  server_fics::sendEndOfGameCommand(const QString& line)
//
///////////////////////////////////////
void server_fics::sendEndOfGameCommand(const QString& line)
{
    QStringList fields;
    Command command;

    fields = QStringList::split(" ", line, false );
    if(fields[1].endsWith(":"))
    {
        fields[1] = fields[1].left(fields[1].length() - 1);
    }
    command.setID(fields[1].toInt());
    fields[fields.count() - 1] = fields[fields.count() - 1].stripWhiteSpace();
    if(fields[fields.count() - 1] == "1-0" )
    {
        if(fields[fields.count() - 2].contains("resigns"))
        {
            command.setCommand((int&)CMD_Black_Resign);
        }
        else if(fields[fields.count() - 2].contains("time"))
        {
            command.setCommand((int&)CMD_White_Called_Flag);
        }
        else
        {
            command.setCommand((int&)CMD_Result_White);
        }
    }
    else if(fields[fields.count() - 1] == "0-1")
    {
        if(fields[fields.count() - 2].contains("resigns"))
        {
            command.setCommand((int&)CMD_White_Resign);
        }
        else if(fields[fields.count() - 2].contains("time"))
        {
            command.setCommand((int&)CMD_Black_Called_Flag);
        }
        else
        {
            command.setCommand((int&)CMD_Result_Black);
        }
    }
    else if(fields[fields.count() - 1] == "1/2-1/2")
    {
        command.setCommand((int&)CMD_Result_Draw);
    }
    else
    {
        command.setCommand((int&)CMD_Lost_Contact);
    }
    emit sendCMD(command);
}

///////////////////////////////////////
//
//  server_fics::sendUserSettings()
//
///////////////////////////////////////
void server_fics::sendUserSettings()
{
    myParent->send("set style 12", false);
    myParent->send(QString("set interface Knights %1").arg(_VERSION_), false);
    myParent->send(QString("set private %1").arg(myResource->OPTION_Private), false);
    myParent->send(QString("set kibitz %1").arg(myResource->OPTION_Kibitz), false);
    myParent->send(QString("set tell %1").arg(myResource->OPTION_Tell), false);
    myParent->send(QString("set shout %1").arg(myResource->OPTION_Shout), false);
    myParent->send(QString("set seek %1").arg(myResource->OPTION_Seek), false);
    myParent->send(QString("set tolerance %1").arg(myResource->OPTION_Profanity), false);
}

///////////////////////////////////////
//
//  server_fics::parseReceivePiece(const QString& line)
//
///////////////////////////////////////
void server_fics::parseReceivePiece(const QString& line)
{
    QStringList fields;
    Command command;
    struct ChessMove move;

    if( parseMode != AWAITING_MOVE_LIST_MODE )
    {
      fields = QStringList::split(" ", line, false );
      command.setID(fields[2].toInt());
      if(fields.size() > 7)
      {
          Command::clearMove(&move);
          safeStrCpy( move.SAN, fields[8] );
          command.setMove(move);
          command.setCommand(CMD_Receive_Piece);
      }
      else
      {
          QString whiteSideBoard = fields[4];
          QString blackSideBoard = fields[6];
          QString commandData = QString("%1%2%3%4%5")
                                .arg(whiteSideBoard.contains('Q', false), 1, 36)
                                .arg(whiteSideBoard.contains('B', false), 1, 36)
                                .arg(whiteSideBoard.contains('N', false), 1, 36)
                                .arg(whiteSideBoard.contains('R', false), 1, 36)
                                .arg(whiteSideBoard.contains('P', false), 1, 36);
          commandData += QString("%1%2%3%4%5")
                        .arg(blackSideBoard.contains('Q', false), 1, 36)
                        .arg(blackSideBoard.contains('B', false), 1, 36)
                        .arg(blackSideBoard.contains('N', false), 1, 36)
                        .arg(blackSideBoard.contains('R', false), 1, 36)
                        .arg(blackSideBoard.contains('P', false), 1, 36);
          command.setCommand(CMD_Sideboard_All);
          command.setData(commandData);
      }
      emit sendCMD(command);
    }
}

///////////////////////////////////////
//
//  server_fics::determineGameType(const QString&)
//
///////////////////////////////////////
void server_fics::determineGameType(const QString& type)
{
    if(type == "blitz" || type == "lightning" || type == "standard")
    {
        gameType = GameType::Standard;
    }
    if(type == "crazyhouse")
    {
        gameType = GameType::CrazyHouse;
    }
    if(type == "suicide")
    {
        gameType = GameType::Suicide;
    }
    if(type == "bughouse")
    {
        gameType = GameType::BugHouse;
    }
    if(type == "losers")
    {
        gameType = GameType::Losers;
    }
}

///////////////////////////////////////
//
//  server_fics::processSeekTimer()
//
///////////////////////////////////////
void server_fics::processSeekTimer()
{
    /* timer timed out, send out a sought */
    myParent->send("$sought all", false);
}

///////////////////////////////////////
//
//      server_fics::parseVariableList
//
///////////////////////////////////////
void server_fics::parseVariableList( const QString &line )
{
	/*
	   By identifying other clients running Knights, we can implement special features
	   between clients. For example, if 2 Knights clients are playing each other, they can
	   send each other thier player images for use in Challenge dialogs, PGN reports, etc.
	*/
	QStringList fields;
	fields = QStringList::split(" ", line, false );
	parseMode = VARIABLE_LIST_MODE;

	if( fields[0] == "Variable" )
	{
		varSubject = fields[3].remove( ":" );
	}
	else if( fields[0].startsWith( "time=" ) )
	{
	}
	else if( fields[0].startsWith( "inc=" ) )
	{
	}
	else if( fields[0].startsWith( "rated=" ) )
	{
	}
	else if( fields[0].startsWith( "open=" ) )
	{
	}
	else if( fields[0].startsWith( "pgn=" ) )
	{
	}
	else if( fields[0].startsWith( "bugopen=" ) )
	{
	}
	else if( fields[0].startsWith( "mailmess=" ) )
	{
	}
	else if( fields[0].startsWith( "tourney=" ) )
	{
	}
	else if( fields[0].startsWith( "provshow=" ) )
	{
	}
	else if( fields[0].startsWith( "autoflag=" ) )
	{
	}
	else if( fields[0].startsWith( "minmovetime=" ) )
	{
	}
	else if( fields[0].endsWith( "is in silence mode." ) )
	{
	}
	else if( fields[0] == "Prompt:" )
	{
	}
	else if( fields[0] == "f1:" )
	{
	}
	else if( fields[0] == "f2:" )
	{
	}
	else if( fields[0] == "f3:" )
	{
	}
	else if( fields[0] == "f4:" )
	{
	}
	else if( fields[0] == "f5:" )
	{
	}
	else if( fields[0] == "f6:" )
	{
	}
	else if( fields[0] == "f7:" )
	{
	}
	else if( fields[0] == "f8:" )
	{
	}
	else if( fields[0] == "f9:" )
	{
	}
	else if( fields[0] == "Formula:" )
	{
	}
	else if( fields[0] == "Bughouse" )
	{
	}
	else if( fields[0] == "Following:" )
	{
	}
	else if( fields[0] == "\\" )
	{
	}
	else if( fields[0] == "Interface:" )
	{
		if( fields[1] == "\"Knights" )
		{
			QString ver = fields[2].remove( "\"" );
			knightsVersion.insert( varSubject, ver, true );
			kdWarning() << "Detected Knights version " << ver << " being used by " << varSubject << endl;
		}
		else
		{
			knightsVersion.insert( varSubject, "0", true );
		}
	}
	else
	{
		parseMode = NORMAL_MODE;
		parseNormalMode( line );
	}
}
///////////////////////////////////////
//
//      server_fics::confirmInterface
//
///////////////////////////////////////
void server_fics::confirmInterface( const QString &name )
{
	if( knightsVersion[name].isEmpty() )
	{
		myParent->send( QString( "$variables " + name ), true);
	}
}
