/***************************************************************************
                          server_fics.h  -  description
                             -------------------
    begin                : Tue Jul 29 2003
    copyright            : (C) 2003 by Alexander Wels
    email                : dutchy@sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _SERVER_FICS_H
#define _SERVER_FICS_H

#include "resource.h"
#include "io_internet.h"
#include "server_base.h"

#include "qmap.h"

class resource;
class io_internet;

typedef QMap<QString, QString> RatingMap;
typedef QMap<QString, QString> KnightsVersionMap;

class server_fics : public server_base
{
    Q_OBJECT

    enum parse12Modes
    {
        PARSE12_MODE_NEW,
        PARSE12_MODE_MOVE
    };

	enum parseModes
	{
		NORMAL_MODE,
		UPDATE_SOUGHT_MODE,
		NEW_GAME_MODE,
		OBSERVE_GAME_MODE,
		CHALLENGE_MODE,
		AWAITING_MOVE_LIST_MODE,
		PARSE_MOVE_LIST_MODE,
		VARIABLE_LIST_MODE
	};

public:
    server_fics(const resource *rsrc, const io_internet* parent);
    ~server_fics();
    int           getServerType()
    {
        return server_base::FICS;
    }
    ;   // this is purely informational.
    void          sendUserSettings();

public slots:
    void          processData(const QString& line);              //process the data received from the io_internet.
    void          acceptChallenge( Challenge* );                 // accept the currently outstanding challenge.
    void          declineChallenge( Challenge* );                // decline the currently outstanding challenge.
    void          processSeekTimer( void );                      // process the timer.
signals:

protected:
    void          parseLoginData(const QString& line);
    void          parseNormalMode(const QString& line);
    void          parseUpdateSoughtMode(const QString& line);
    void          parseChallenge(const QString &line);
    void          parseNewGame(const QString& line);
    void          parseObserveGame(const QString& line);
    void          parseMoveList(const QString& line);
		void          parseVariableList( const QString& line );
    void          sendEndOfGameCommand(const QString& line);
    void          parseStyle12(const QString& line, const unsigned int mode);
    void          determineGameType(const QString& type);
    void          parseReceivePiece(const QString& line);
		void          confirmInterface( const QString& name );
private:
    server_fics();
    server_fics(const resource *rsrs);

    int               loginStage;    /* the current stage of the login process */
    int               observeGameNr; /* the game number we are observing */
    int               gameType;      /* determines the type of game we are going to play/ob/exa */
    QString           lastTag;
    parseModes        parseMode;     /* which parse mode are we in */
    io_internet       *myParent;     /* the io_internet that instantiated me */
    RatingMap         playerRating;  /* stores the latest known rating for each known player */
		KnightsVersionMap knightsVersion;/* stores version information on players detected running Knights */
		QString           varSubject;    /* Name of the user we're receiving variables about */
};

#endif /* _SERVER_FICS_H */

