/***************************************************************************
                          server_icc.cpp  -  description
                             -------------------
    begin                : Fri Sep 5 2003
    copyright            : (C) 2003 by The Knights Project
    email                : knights-general@lists.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "server_icc.moc"

/* Qt */
#include <qregexp.h>

/* KDE */
#include <kmessagebox.h>
#include <kdebug.h>

/* Local */
#include "resource.h"
#include "dlg_login.h"
#include "dlg_challenge.h"
#include "match.h"
#include "audio.h"

/////////////////////////////////////////////////
//
//  server_icc::server_icc(const resource *rsrc, const io_internet* parent)
//
/////////////////////////////////////////////////
server_icc::server_icc(const resource *rsrc, const io_internet* parent)
{
    this->myResource = (resource*)rsrc;
    this->myParent = (io_internet*)parent;
    this->loginStage = LOGIN_STAGE_NAME;
    this->level2SettingsSend = false;

    //Setup which datagrams we want to use.
    //First fill the bitArray with 0's.
    this->level2Setting.fill(false, DG_SET2 + 1);

    this->level2Setting.setBit(DG_WHO_AM_I);
    //  this->level2Setting.setBit(DG_SET2);
    this->level2Setting.setBit(DG_PERSONAL_TELL);
    this->level2Setting.setBit(DG_CHANNEL_TELL);
    this->level2Setting.setBit(DG_SHOUT);
    //  this->level2Setting.setBit(DG_SEEK);
    //  this->level2Setting.setBit(DG_SEEK_REMOVED);
    this->level2Setting.setBit(DG_MY_GAME_STARTED);
    this->level2Setting.setBit(DG_MY_GAME_RESULT);
    this->level2Setting.setBit(DG_MY_GAME_ENDED);
    this->level2Setting.setBit(DG_STARTED_OBSERVING);
    this->level2Setting.setBit(DG_STOP_OBSERVING);
    this->level2Setting.setBit(DG_SEND_MOVES);
    this->level2Setting.setBit(DG_KIBITZ);
}

/////////////////////////////////////////////////
//
//  server_icc::server_icc()
//
/////////////////////////////////////////////////
server_icc::server_icc()
{}

/////////////////////////////////////////////////
//
//  server_icc::~server_icc()
//
/////////////////////////////////////////////////
server_icc::~server_icc()
{
    this->myResource = NULL;
    this->myParent = NULL;
    this->loginStage = LOGIN_STAGE_NAME;
    this->level2SettingsSend = false;
}

///////////////////////////////////////
//
//  server_icc::sendUserSettings()
//
///////////////////////////////////////
void server_icc::sendUserSettings()
{
    myParent->send("set-quietly style 13", false);
    myParent->send(QString("set-quietly interface Knights %1").arg(_VERSION_), false);
    //  myParent->send(QString("set-quietly kib %1").arg(myResource->OPTION_Kibitz), false);
    //  myParent->send(QString("set-quietly tell %1").arg(myResource->OPTION_Tell), false);
    //  myParent->send(QString("set-quietly shout %1").arg(myResource->OPTION_Shout), false);
    //  myParent->send(QString("set-quietly seek %1").arg(myResource->OPTION_Seek), false);
    //  myParent->send(QString("set-quietly tol %1").arg(myResource->OPTION_Profanity), false);
    myParent->send(QString("set-quietly prompt 0"), false); //saves me having to deal with stripping the prompt.
}

/////////////////////////////////////////////////
//
//  server_icc::processData(const QString& line)
//
/////////////////////////////////////////////////
void server_icc::processData(const QString& line)
{
    QStringList datagrams;

    if(loginStage != LOGIN_STAGE_LOGGED_IN)
    {
        parseLoginData(line);
    }
    else
    {
        if(line.contains('\x019'))
        {
            kdDebug() << "[" << line << "]" << endl;
            datagrams = QStringList::split(QRegExp("\x019\\("), line, false);
            for(QStringList::iterator i = datagrams.begin(); i != datagrams.end(); i++)
            {
                parseDatagram((*i).left((*i).length() - 2));
            }
        }
        else
        {
            myParent->writeToConsole(line, "K_STD");
        }
    }
}

///////////////////////////////////////
//
//  server_icc::resetLogin
//
///////////////////////////////////////
void server_icc::resetLogin(void)
{
    loginStage = LOGIN_STAGE_NAME;
}

///////////////////////////////////////
//
//  server_icc::acceptChallenge()
//
///////////////////////////////////////
void server_icc::acceptChallenge( Challenge *challenge )
{
	QString match = "match ";

	if( myParent->getUserName() == challenge->whiteName )
	{
		match += challenge->blackName;
	}
	else
	{
		match += challenge->whiteName;
	}

	if( challenge->rated )
	{
		match += " rated ";
	}
	else
	{
		match += " unrated ";
	}

	match += challenge->baseTime + " ";
	match += challenge->incTime;

	if( challenge->oddsBaseTime )
	{
		match += " " + challenge->oddsBaseTime;
		match += " " + challenge->oddsIncTime;
	}

	if( challenge->ICSTypeName != "blitz" &&
	    challenge->ICSTypeName != "standard" &&
	    challenge->ICSTypeName != "lightning" )
	{
		match += " " + challenge->ICSTypeName;
	}
	myParent->send( match, false );
}

///////////////////////////////////////
//
//  server_icc::declineChallenge()
//
///////////////////////////////////////
void server_icc::declineChallenge( Challenge *challenge )
{
	if( challenge->whiteName == myParent->getUserName().upper() )
	{
		myParent->send("decline " + challenge->blackName , false);
	}
	else
	{
		myParent->send("decline " + challenge->whiteName , false);
	}
}

///////////////////////////////////////
//
//  server_icc::processSeekTimer()
//
///////////////////////////////////////
void server_icc::processSeekTimer()
{
    /* timer timed out, send out a sought */
    myParent->send("$sought all", false);
}

///////////////////////////////////////
//
//  server_icc::parseLoginData
//
///////////////////////////////////////
void server_icc::parseLoginData(const QString& data)
{
    QStringList lines;

    if(loginStage == LOGIN_STAGE_NAME)
    {
        if(data.contains("login: "))
        {
            if(!level2SettingsSend)
            {
                //construct the bitstring to send to the server.
                QString level2String = "level2settings=";
                for(unsigned int i = 0; i < level2Setting.size(); i++)
                {
                    level2String += level2Setting.at(i) ? "1" : "0";
                }
                kdDebug() << level2String << endl;
                myParent->send(level2String, true);
                level2SettingsSend = true;
            }
            myParent->sendUserName();
            loginStage = LOGIN_STAGE_PASSWORD;
        }
    }
    else if(loginStage == LOGIN_STAGE_PASSWORD)
    {
        if(data.contains("If it is yours, type the password."))
        {
            myParent->sendPassword();
        }
        else if(data.contains("Invalid password."))
        {
            dlg_login *loginDlg = new dlg_login( NULL, "LoginDialog", myResource);
            loginDlg->disableServerSelect();
            connect(loginDlg, SIGNAL(okClicked()), myParent, SLOT(sendUserName()));
            connect(loginDlg, SIGNAL(cancelClicked()), myParent, SLOT(selfDestruct()));
            connect(loginDlg, SIGNAL(login(const QString&, const QString&)), myParent, SLOT(setUserInfo(const QString&, const QString&)));
        }
        else if(data.contains("________"))
        {
            loginStage = LOGIN_STAGE_LOGGED_IN;
            sendUserSettings();
            lines = QStringList::split( QRegExp("\n\r?"), data, FALSE );
            for(QStringList::iterator i = lines.begin(); i != lines.end(); i++)
            {
                myParent->writeToConsole((*i), "K_STD");
            }
        }
    }
    lines = QStringList::split( QRegExp("\n\r?"), data, FALSE );
    for(QStringList::iterator i = lines.begin(); i != lines.end(); i++)
    {
        myParent->writeToConsole((*i), "K_STD");
    }
}

///////////////////////////////////////
//
//  server_icc::parseDatagram(const QString datagram)
//
///////////////////////////////////////
void server_icc::parseDatagram(const QString& datagram)
{
    QStringList params;

    params = QStringList::split(' ', datagram, false);
    switch(params[0].toInt())
    {
    case DG_WHO_AM_I:
        myParent->setUserInfo(params[1], "");
        break;
    case DG_PERSONAL_TELL:
        parsePersonalTell(params);
        break;
    case DG_CHANNEL_TELL:
        parseChannelTell(params);
        break;
    case DG_SHOUT:
        parseShout(params);
        break;
    case DG_SEEK:
        parseSeek(params);
        break;
    case DG_SEEK_REMOVED:
        parseSeekRemoved(params);
        break;
    case DG_STARTED_OBSERVING:
        parseStartObserving(params);
        break;
    default:
        kdDebug() << datagram << endl;
        break;
    }
}

///////////////////////////////////////
//
//  server_icc::parsePersonalTell(const QStringList& params)
//
///////////////////////////////////////
void server_icc::parsePersonalTell(QStringList& params)
{
    QString title = "";
    QString message = "";

    //figure out the titles.
    int i = 1;
    do
    {
        i++;
        title += "(" + params[i].remove("{").remove("}") + ")";
    }
    while(!params[i+1].startsWith("\x019{"));
    if(title == "()")  // no actual title.
    {
        title = "";
    }
    title += " ";
    //figure out the message
    i++;
    for(; i < (int)params.size() - 1; i++)
    {
        message += params[i].remove("\x019{").remove("\x019}") + " ";
    }
    //determine the tell types.
    switch(params[i].toInt())
    {
    case 0: //say
        myParent->writeToConsole(params[1] + title + "says " + message, "K_PVT");
        break;
    case 1: //tell
        myParent->writeToConsole(params[1] + title + "tells you: " + message, "K_PVT");
        break;
    case 2: //ptell
        myParent->writeToConsole(params[1] + title + "(your partner) tells you: " + message, "K_PVT");
        break;
    case 3: //qtell
        myParent->writeToConsole(params[1] + title + "qtells you: " + message, "K_PVT");
        break;
    case 4: //atell
        myParent->writeToConsole(params[1] + title + "atells you: " + message, "K_PVT");
        break;
    }
    emit sendCMD(Command(0, CMD_Set_Src_Tell, params[1]));
}

///////////////////////////////////////
//
//  server_icc::parseChannelTell(const QStringList& params)
//
///////////////////////////////////////
void server_icc::parseChannelTell(QStringList& params)
{
    QString title = "";
    QString message = "";

    //figure out the titles.
    QString channel = params[1];
    int i = 2;
    do
    {
        i++;
        title += "(" + params[i].remove("{").remove("}") + ")";
    }
    while(!params[i+1].startsWith("\x019{"));
    if(title == "()")  // no actual title.
    {
        title = "";
    }
    title += " ";
    //figure out the message
    i++;
    for(; i < (int)params.size() - 1; i++)
    {
        message += params[i].remove("\x019{").remove("\x019}") + " ";
    }
    //determine channel tell type.
    switch(params[i].toInt())
    {
    case 0: //default
    case 1: //drop through in case any of the others ever happen.
    case 2:
    case 3:
        myParent->writeToConsole(params[2] + title + "(" + channel + "): " + message, "K_CH");
        break;
    case 4: //atell
        myParent->writeToConsole(params[2] + title + "(" + channel + ") answers: ", "K_CH");
        break;
    }
    emit sendCMD(Command(0, CMD_Set_Src_Channel, channel));
}

///////////////////////////////////////
//
//  server_icc::parseShout(const QStringList params)
//
///////////////////////////////////////
void server_icc::parseShout(QStringList& params)
{
    QString title = "";
    QString message = "";

    //figure out the titles.
    int i = 1;
    do
    {
        i++;
        title += "(" + params[i].remove("{").remove("}") + ")";
    }
    while(!params[i+1].contains(QRegExp("\\d+")));
    if(title == "()")  // no actual title.
    {
        title = "";
    }
    title += " ";
    i++;
    int shoutType = i;
    //figure out the message
    i++;
    for(; i < (int)params.size(); i++)
    {
        message += params[i].remove("\x019{").remove("\x019}") + " ";
    }
    //determine shout types.
    switch(params[shoutType].toInt())
    {
    case 0: // regular shouts.
        myParent->writeToConsole(params[1] + title + "shouts: " + message, "K_SHT");
        break;
    case 1: // i-shouts
        myParent->writeToConsole("--> " + params[1] + title + message, "K_SHT");
        break;
    case 2: // s-shouts
        myParent->writeToConsole(params[1] + title + "s-shouts: " + message, "K_SHT");
        break;
    case 3: //announcements
        myParent->writeToConsole("**ANNOUNCEMENT** from " + params[1] + title + ": " + message, "K_SHT");
        break;
    }
}

///////////////////////////////////////
//
//  server_icc::parseSeek(QStringList params)
//
///////////////////////////////////////
void server_icc::parseSeek(QStringList& params)
{
}

///////////////////////////////////////
//
//  server_icc::parseRemoveSeek(QStringList params)
//
///////////////////////////////////////
void server_icc::parseSeekRemoved(QStringList& params)
{
}

///////////////////////////////////////
//
//  server_icc::parseStartObserving(QStringList params)
//
///////////////////////////////////////
void server_icc::parseStartObserving(QStringList& params)
{
    struct ChessMove move;
    match_param *param = NULL;
    Command::clearMove(&move);

    param = new match_param(this->myResource);

    // set white time control
    TCPList tmpListWhite(param->time(WHITE));
    TCP tmpTCPWhite = tmpListWhite[0];
    tmpTCPWhite.Seconds = params[7].toInt() * 60;
    tmpTCPWhite.Increment = params[8].toInt();
    tmpListWhite[0] = tmpTCPWhite;
    param->setTime(WHITE, tmpListWhite);

    // set black time control
    TCPList tmpListBlack(param->time(BLACK));
    TCP tmpTCPBlack = tmpListBlack[0];
    tmpTCPBlack.Seconds = params[9].toInt() * 60;
    tmpTCPBlack.Increment = params[10].toInt();
    tmpListBlack[0] = tmpTCPBlack;
    param->setTime(BLACK, tmpListBlack);

    //I am observing this game.
    param->setType(BLACK, PLAYERTCP);
    param->setType(WHITE, PLAYERTCP);

    if(params[11].toInt() == 0)
    {
        param->setType(WHITE, PLAYEREXAMINE);
        param->setType(BLACK, PLAYEREXAMINE);
    }
    param->setVariation(determineGameType(params[5]));
    param->setName(WHITE, params[2]);
    param->setName(BLACK, params[3]);

    int i = 12;
    while(!params[i].endsWith("}"))
    {
        i++;
    }
    if(params[i + 1] == "0")
    {
        params[i + 1] == "++++";
    }
    if(params[i + 2] == "0")
    {
        params[i + 2] == "++++";
    }
    param->setRating(WHITE, params[i + 1]);
    param->setRating(BLACK, params[i + 2]);
    // tell core to connect us to a new match
    emit newMatch(param, params[1].toInt());
}

///////////////////////////////////////
//
//  server_icc::determineGameType(const QString&)
//
///////////////////////////////////////
int server_icc::determineGameType(const QString& type)
{
    if(type == "blitz" || type == "lightning" || type == "standard")
    {
        return GameType::Standard;
    }
    if(type == "crazyhouse")
    {
        return GameType::CrazyHouse;
    }
    if(type == "suicide")
    {
        return GameType::Suicide;
    }
    if(type == "bughouse")
    {
        return GameType::BugHouse;
    }
    if(type == "losers")
    {
        return GameType::Losers;
    }
    //Don't know what the hell this is, assume normal chess rules.
    return GameType::Standard;
}

