/***************************************************************************
                          server_icc.h  -  description
                             -------------------
    begin                : Fri Sep 5 2003
    copyright            : (C) 2003 by The Knights Project
    email                : knights-general@lists.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SERVER_ICC_H
#define SERVER_ICC_H

#include "resource.h"
#include "io_internet.h"
#include "server_base.h"

#include "qbitarray.h"
#include "qstringlist.h"

class resource;
class io_internet;

/**
  *@author The Knights Team
  */

class server_icc : public server_base
{
    Q_OBJECT

    enum datagramTypes
    {
        DG_WHO_AM_I = 0,  // Form: (myname titles)
        DG_PLAYER_ARRIVED = 1,  // Form: (playername <sequence of other info about that player>) /* don't want this */
        DG_PLAYER_LEFT = 2, // Form: Form: (playername) /* don't want this */
        DG_BULLET = 3, // Form: (playername rating annotation)
        DG_BLITZ = 4, // Form: (playername rating annotation)
        DG_STANDARD = 5, // Form: (playername rating annotation)
        DG_WILD = 6, // Form: (playername rating annotation)
        DG_BUGHOUSE = 7, // Form: (playername rating annotation)
        DG_LOSERS = 88, // Form: (playername rating annotation)
        DG_TIMESTAMP = 8, //Form: (playername timestamp-client-number)
        DG_TITLES = 9, //Form: ?unknown?
        DG_OPEN = 10, //Form: 0/1
        DG_STATE = 11, //Form: (playername <type> <gamenumber>)
        DG_PLAYER_ARRIVED_SIMPLE = 55, //Form (name)
        DG_GAME_STARTED = 12, /*Form: (gamenumber whitename blackname wild-number rating-type rated
                                               white-initial white-increment black-initial black-increment
                                               played-game {ex-string} white-rating black-rating game-id
                                               white-titles black-titles irregular-legality irregular-semantics
                                               uses-plunkers fancy-timecontrol) */
        DG_GAME_RESULT = 13, //Form: (gamenumber become-examined game_result_code score_string2 description-string)
        DG_EXAMINED_GAME_IS_GONE = 14, //Form: (gamenumber)
        DG_MY_GAME_STARTED = 15, //See DG_GAME_STARTED for form.
        DG_MY_GAME_RESULT = 16, //See DG_GAME_RESULT for form.
        DG_MY_GAME_ENDED = 17, //See DG_GAME_ENDED for form.
        DG_STARTED_OBSERVING = 18, //See DG_GAME_STARTED for form.
        DG_STOP_OBSERVING = 19, //Form: (gamenumber).
        DG_ISOLATED_BOARD = 40, //Form: (gamenumber).
        DG_PLAYERS_IN_MY_GAME = 20, //Form: (gamenumber playername symbol kibvalue).
        DG_OFFERS_IN_MY_GAME = 21, //Form: (gamenumber wdraw bdraw wadjourn badjourn wabort babort wtakeback btakeback)
        DG_TAKEBACK = 22, //Form: (gamenumber takeback-count)
        DG_BACKWARD = 23, //Form: (gamenumber backup-count)
        DG_MOVE_ALGEBRAIC = 33,
        DG_MOVE_SMITH = 34,
        DG_MOVE_TIME = 35,
        DG_MOVE_CLOCK = 36,
        DG_SEND_MOVES = 24, //Form: (gamenumber algebraic-move smith-move time clock is-variation)
        DG_MOVE_LIST = 25, //Form: (gamenumber initial-position {white-move1-info}{black-move1-info}{white-move2-info} ...)
        DG_BUGHOUSE_HOLDINGS = 37, //Form: (gamenumber {white-holding} {black-holding})
        DG_BUGHOUSE_PASS = 57, //Form: (gamenumber color piece)
        DG_KIBITZ = 26, //Form: (gamenumber playername titles kib/whi ^Y{kib string^Y})
        DG_SET_CLOCK = 38, //Form: (gamenumber white-clock black-clock)
        DG_FLIP = 39, //Form: (gamenumber flip)
        DG_REFRESH = 41, //Form: (gamenumber)
        DG_ILLEGAL_MOVE = 42, //Form: (gamenumber movestring reason)
        DG_MY_RELATION_TO_GAME = 43, //Form: (gamenumber symbol)
        DG_PARTNERSHIP = 44, //Form: (name name forming)
        DG_JBOARD = 49, //Form: ?unknown?
        DG_FEN = 70, //Form: (gamenumber {FEN-string})
        DG_MSEC = 56, //Form: (gamenumber color msec running)
        DG_MORETIME = 61, //Form: (gamenumber color seconds)
        DG_CHANNEL_TELL = 28, //Form: (channel playername titles ^Y{tell string^Y} type)
        DG_PEOPLE_IN_MY_CHANNEL = 27, //Form: (channel playername come/go)
        DG_CHANNELS_SHARED = 46, //Form: (playername channels)
        DG_SEES_SHOUTS = 45, //Form: (playername bitvector)
        DG_MATCH = 29, /*Form: (challenger-name challenger-rating challenger-titles
                                        receiver-name   receiver-rating   receiver-titles
                                        wild-number rating-type is-it-rated is-it-adjourned
                                        challenger-time-control receiver-time-control
                                        challenger-color-request [assess-loss assess-draw assess-win]
                                        fancy-time-control)*/
        DG_MATCH_REMOVED = 30, //Form: (challenger-name receiver-name ^Y{Explanation string^Y})
        DG_SEEK = 50, /*Form: (index name titles rating provisional-status wild rating-type time
                                       inc rated color minrating maxrating autoaccept formula fancy-time-control)*/
        DG_SEEK_REMOVED = 51, // Form: (index reasoncode)
        DG_MY_RATING = 52, //Form: (bullet-rating blitz-rating standard-rating wild-rating bughouse-rating)
        DG_NEW_MY_RATING = 87, //Form: (rating-0 rating-1 rating-2 ...)
        DG_PERSONAL_TELL = 31, //Form: (playername titles ^Y{tell string^Y} type)
        DG_PERSONAL_TELL_ECHO = 62, //Form: (receivername ^Y{string^Y} type)
        DG_SHOUT = 32, //Form: (playername titles type ^Y{shout string^Y})
        DG_MY_VARIABLE = 47, //Form: (variable-name value)
        DG_MY_STRING_VARIABLE = 48, //Form: (variable-name value)
        DG_SOUND = 53, //Form: (code)
        DG_SUGGESTION = 63, //Form: (command-string text-string priority suggester subject id)
        DG_CIRCLE = 59, //Form: (gamenumber examiner coordinate)
        DG_ARROW = 60, //Form: (gamenumber examiner origin destination)
        DG_NOTIFY_ARRIVED = 64, //Form: See DG_PLAYER_ARRIVED
        DG_NOTIFY_LEFT = 65, //Form: See DG_PLAYER_LEFT
        DG_NOTIFY_OPEN = 66, //Form: See DG_OPEN
        DG_NOTIFY_STATE = 67, //Form: See DG_STATE
        DG_MY_NOTIFY_LIST = 68, //Form: (name yes/no)
        DG_LOGIN_FAILED = 69, //Form: (code explanation-string)
        DG_GAMELIST_BEGIN = 72, //Form: (command {parameters} nhits first last {summary})
        DG_GAMELIST_ITEM = 73, /*Form: (index id event date time white-name white-rating black-name black-rating rated rating-type
                                                wild init-time-W inc-W init-time-B inc-B eco status color mode {note} here)*/
        DG_IDLE = 74, //Form: (name idle-time time-since-read)
        DG_ACK_PING = 75, //Form: (name lag-in-ms)
        DG_RATING_TYPE_KEY = 76, //Form: (index string)
        DG_GAME_MESSAGE = 77, //Form: (gamenumber string)
        DG_UNACCENTED = 78,
        DG_STRINGLIST_BEGIN = 79, //Form: (nhits {description})
        DG_STRINGLIST_ITEM = 80, //Form: (string)
        DG_DUMMY_RESPONSE = 81,
        DG_CHANNEL_QTELL = 82, //Form: (channel name titles ^Y{qtell-string^Y})
        DG_PERSONAL_QTELL = 83, //Form: (playername titles ^Y{qtell-string^Y})
        DG_SET_BOARD = 84, //Form: (gamenumber 64-character-board side-to-move)
        DG_MATCH_ASSESSMENT = 85,
        DG_LOG_PGN = 86,
        DG_UNCIRCLE = 89, //Form: (gamenumber examiner coordinate)
        DG_UNARROW = 90, //Form: (gamenumber examiner origin destination)
        DG_WSUGGEST = 91, //Form: (URL text-string priority suggester subject id)
        DG_MESSAGELIST_BEGIN = 94, //Form: (command-string)
        DG_MESSAGELIST_ITEM = 95, //Form: (index sender time date message)
        DG_LIST = 96, //Form: (header rowstart item1 item2 ....)
        DG_SJI_AD = 97,
        DG_QRETRACT = 99, //Form: (id)
        DG_MY_GAME_CHANGE = 100, //Form: See DG_GAME_STARTED
        DG_POSITION_BEGIN = 101, //Form: (gamenumber {initial-FEN} nmoves-to-follow)
        DG_POSITION_BEGIN2 = 110,
        DG_PAST_MOVE = 111, //Same form as DG_SEND_MOVE
        DG_PGN_TAG = 112, //Form: (gamenumber tagname value)
        DG_IS_VARIATION = 113,
        DG_PASSWORD = 114, //Form: (new-password handle)
        DG_WILD_KEY = 116, //Form: (index string)
        DG_SET2 = 124
    };

public:
    server_icc(const resource *rsrc, const io_internet* parent);
    ~server_icc();
    int           getServerType()
    {
        return server_base::ICC;
    }
    ;   // this is purely informational.
    void          sendUserSettings();

public slots:
    void          processData(const QString& line);              //process the data received from the io_internet.
    void          resetLogin(void);                              //reset the login process.
    void          acceptChallenge( Challenge* );               // accept the currently outstanding challenge.
    void          declineChallenge( Challenge* );              // decline the currently outstanding challenge.
    void          processSeekTimer( void );                      // process the timer.

protected:
    int           determineGameType(const QString& type);
    void          parseLoginData(const QString& line);
    void          parseDatagram(const QString& datagram);
    void          parsePersonalTell(QStringList& params);
    void          parseChannelTell(QStringList& params);
    void          parseShout(QStringList& params);
    void          parseSeek(QStringList& params);
    void          parseSeekRemoved(QStringList& params);
    void          parseStartObserving(QStringList& params);
signals:

private:
    server_icc();

    io_internet       *myParent;     /* the io_internet that instantiated me */
    int               loginStage;    /* the current stage of the login process */
    QBitArray         level2Setting; /* a bit array that sets which level 2 datagram I want to receive */
    bool              level2SettingsSend;
};

#endif
