/***************************************************************************
                          tab_pgnview.cpp  -  description
                             -------------------
    begin                : Sat Jan 25 2003
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tab_pgnview.moc"
#include "tabmanager.h"
#include "resource.h"
#include "pgn.h"
#include "matchhistory.h"
#include <qfile.h>
#include <qregexp.h>
#include <qtextstream.h>
#include <kstddirs.h>
#include <ktempfile.h>

tab_pgnView::tab_pgnView(pgn *parent, resource *Rsrc ) : KnightsTextView(0,Rsrc)
{
    myParent = parent;
    whiteImage = NULL;
    blackImage = NULL;

		featureList.ContextMenu = false;
		featureList.ReqPermissionForClose = false;
		featureList.EnableSave = false;
		featureList.EnableSaveAs = false;
		featureList.EnablePrint = true;

    QString source = locate( "appdata", "pgn_template.kml" );
    QFile file( source );
    if( file.open( IO_ReadOnly ) )
    {
        QTextStream stream( &file );
        PGNTemplate = stream.read();
        file.close();
    }
    else
    {
        kdWarning() << "tab_pgnView::tab_pgnView: Can not find PGN Template" << endl;
    }
}

tab_pgnView::~tab_pgnView()
{
    if( whiteImage )
        delete whiteImage;
    if( blackImage )
        delete blackImage;
}
///////////////////////////////////////
//
//      tab_pgnView::requestWidgetFeatureList
//
///////////////////////////////////////
void tab_pgnView::requestWidgetFeatureList( void )
{
	emit( updateWidgetFeatureList( featureList ) );
}
///////////////////////////////////////
//
//      tab_pgnView::recvCMD
//
///////////////////////////////////////
void tab_pgnView::recvCMD( const Command &cmd )
{
	switch( cmd.getCommand() )
	{
		/* MENU_PRINT */
		case MENU_PRINT:
			myParent->print();
			break;
		default:
			break;
	}
}
///////////////////////////////////////
//
//      tab_pgnView::refresh
//
///////////////////////////////////////
void tab_pgnView::refresh( void )
{
  document = PGNTemplate;

    /* Replace macros with data */
    document.replace( QRegExp( "%site%" ), myParent->tags.find( "Site" ) );
    document.replace( QRegExp( "%date%" ), myParent->tags.find( "Date" ) );
    document.replace( QRegExp( "%round%" ), myParent->tags.find( "Round" ) );
    document.replace( QRegExp( "%result%" ), myParent->tags.find( "Result" ) );
    document.replace( QRegExp( "%white%" ), myParent->tags.find( "White" ) );
    document.replace( QRegExp( "%whitetitle%" ), myParent->tags.find( "WhiteTitle" ) );
    document.replace( QRegExp( "%whiteelo%" ), myParent->tags.find( "WhiteElo" ) );
    document.replace( QRegExp( "%whiteuscf%" ), myParent->tags.find( "WhiteUSCF" ) );
    document.replace( QRegExp( "%whitena%" ), myParent->tags.find( "WhiteNA" ) );
    document.replace( QRegExp( "%whitetype%" ), myParent->tags.find( "WhiteType" ) );
    document.replace( QRegExp( "%black%" ), myParent->tags.find( "Black" ) );
    document.replace( QRegExp( "%blacktitle%" ), myParent->tags.find( "BlackTitle" ) );
    document.replace( QRegExp( "%blackelo%" ), myParent->tags.find( "BlackElo" ) );
    document.replace( QRegExp( "%blackuscf%" ), myParent->tags.find( "BlackUSCF" ) );
    document.replace( QRegExp( "%blackna%" ), myParent->tags.find( "BlackNA" ) );
    document.replace( QRegExp( "%blacktype%" ), myParent->tags.find( "BlackType" ) );
    document.replace( QRegExp( "%time%" ), myParent->tags.find( "Time" ) );
    document.replace( QRegExp( "%utctime%" ), myParent->tags.find( "UTCTime" ) );
    document.replace( QRegExp( "%utcdate%" ), myParent->tags.find( "UTCDate" ) );
    document.replace( QRegExp( "%event%" ), myParent->tags.find( "Event" ) );
    document.replace( QRegExp( "%eventdate%" ), myParent->tags.find( "EventDate" ) );
    document.replace( QRegExp( "%eventsponsor"), myParent->tags.find( "EventSponsor" ) );
    document.replace( QRegExp( "%section%" ), myParent->tags.find( "Section" ) );
    document.replace( QRegExp( "%stage%" ), myParent->tags.find( "Stage" ) );
    document.replace( QRegExp( "%board%" ), myParent->tags.find( "Board" ) );
    document.replace( QRegExp( "%opening%" ), myParent->tags.find( "Opening" ) );
    document.replace( QRegExp( "%variation%" ), myParent->tags.find( "Variation" ) );
    document.replace( QRegExp( "%subvariation%" ), myParent->tags.find( "SubVariation" ) );
    document.replace( QRegExp( "%eco%" ), myParent->tags.find( "ECO" ) );
    document.replace( QRegExp( "%nic%" ), myParent->tags.find( "NIC" ) );
    document.replace( QRegExp( "%timecontrol%" ), myParent->tags.find( "TimeControl" ) );
    document.replace( QRegExp( "%termination%" ), myParent->tags.find( "Termination" ) );
    document.replace( QRegExp( "%setup%" ), myParent->tags.find( "SetUp" ) );
    document.replace( QRegExp( "%fen%" ), myParent->tags.find( "FEN" ) );
    document.replace( QRegExp( "%annotator%" ), myParent->tags.find( "Annotator" ) );
    document.replace( QRegExp( "%mode%" ), myParent->tags.find( "Mode" ) );
    document.replace( QRegExp( "%plycount%" ), myParent->tags.find( "PlyCount" ) );

    /* Get the White Player's Image */
    QPixmap wi = myResource->loadSCIDImage( myParent->tags.find( "White" ) );
    if( wi.isNull() )
    {
        if( myParent->tags.find( "WhiteType" ) == "program" )
            wi.load( locate("data", "knights/default-engine-portrait.jpg" ) );
        else
            wi.load( locate("data", "knights/default-portrait.jpg" ) );
    }
    whiteImage = new KTempFile();
    whiteImage->setAutoDelete( true );
    wi.save( whiteImage->name(), "PNG" );
    document.replace( QRegExp( "%whiteimage%" ), whiteImage->name() );

    /* Get the Black Player's Image */
    QPixmap bi = myResource->loadSCIDImage( myParent->tags.find( "Black" ) );
    if( bi.isNull() )
    {
        if( myParent->tags.find( "BlackType" ) == "program" )
            bi.load( locate("data", "knights/default-engine-portrait.jpg" ) );
        else
            bi.load( locate("data", "knights/default-portrait.jpg" ) );
    }
    blackImage = new KTempFile();
    blackImage->setAutoDelete( true );
    bi.save( blackImage->name(), "PNG" );
    document.replace( QRegExp( "%blackimage%" ), blackImage->name() );

    /* Obtain the move data */
    QString moves;
    QStringList *list;
    if( myParent->history->count() )
    {
        list = myParent->notation(3);
    }
    else
    {
        list = new QStringList( myParent->Move_Data );
    }

    /* Do some formatting and then merge the strings */
    for( QStringList::Iterator i = list->begin(); i != list->end(); i++ )
    {
        (*i).replace( QRegExp( "[^\\0040-\\0176]" ), "" );
        (*i).replace( QRegExp( "\\(" ), "<i>(" );
        (*i).replace( QRegExp( "\\)" ), ")</i>" );
        (*i).replace( QRegExp( "\\{" ), "<b>{" );
        (*i).replace( QRegExp( "\\}" ), "}</b>" );
    }
    moves = list->join( " " );
    delete list;

    int pos = document.find( "%moves%" );
    if( pos != -1 )
    {
        document.remove( (unsigned int)pos, 7 );
        document.insert( (unsigned int)pos, moves );
    }
    setText( document );
}
