/***************************************************************************
                          tabbox.h  -  description
                             -------------------
    begin                : Fri Sep 13 2002
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TABBOX_H
#define TABBOX_H

#include <qvbox.h>
#include "accel.h"
#include "definitions.h"

class QTabWidget;
class resource;
class MenuBar;
class Command;
class dlg_settings;
class KPopupMenu;

/**
  *@author Troy Corbin Jr.

  This class represents a top-level window in the Tabs scheme. It holds TabPages
  and is managed by the TabManager. This also is responsable for the menubar.
  */

class TabBox : public QVBox
{
    Q_OBJECT
public:
    TabBox( resource *rsrc=0 );
    virtual     ~TabBox();
    bool        shutdown( void );
    void        keyPressEvent( QKeyEvent* );
    void        addTab( QWidget *child, const QString &caption );
    bool        removeTab( QWidget *child, bool deleteChild=false );
    bool        isChild( QWidget *child );
    QString     getChildName( const int &index );
    void        showTab( QWidget *child );
    void        refreshMenu( void );
    void        changeCaption( QWidget *child, const QString &caption );
    int         count( void );

signals:
    void        newTabBox( TabBox* );
    void        saveTabGeometry( const QString&, const QSize& );
    void        focus( const QChar& );

    /**
        Requests a pointer to a KPopupMenu to be inserted into the Menubar.
        Your widget should have a SLOT( requestMenu() ) available to catch
        this, and you should answer with a SIGNAL( fetchMenu( KPopupMenu*, QString& ) )
        TabBox will handle the connections.
    */
    void        requestMenu( void );

public slots:
    void        hideEvent( QHideEvent* );
    void        showEvent( QShowEvent* );

protected slots:
    void        menuAction( int );
    void        currentChanged( QWidget* );

    /**
        This removes and deletes a Tab if it's a child of this widget.
        Never call this directly; It relies on QObject::sender()!
    */
    void        destroyChild( void );

    void        updateWindowMenu( void );
    void        recieveWidgetFeatureList( const WFL& );

protected:
    void        changeMyCaption( QWidget *child );

private:
    MenuBar             *myMenuBar;
    QTabWidget          *myTabs;
    Accel               *myAccel;
    resource            *myResource;
    dlg_settings        *SettingsDialog;
    bool                minimized;
};

#endif
