/***************************************************************************
                          tabgrip.cpp  -  description
                             -------------------
    begin                : Fri Sep 13 2002
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tabgrip.moc"
#include "resource.h"
#include <qpainter.h>
#include <qtooltip.h>

TabGrip::TabGrip(QWidget *parent, const char *name ) : QWidget(parent,name)
{
    setMaximumSize( 3200, 12 );
    setMinimumSize( 32, 12 );

    isDragging = false;
    couldDrag = false;

    setCursor( Qt::SizeAllCursor );
    QToolTip::add( this, i18n("Drag Tab") );

    show();
}
TabGrip::~TabGrip()
{}
///////////////////////////////////////
//
//      TabGrip::paintEvent
//
///////////////////////////////////////
void TabGrip::paintEvent( QPaintEvent* )
{
    QPainter paint( this );
    QColorGroup group( colorGroup() );
    paint.setPen( QPen( group.dark(), 2, Qt::DotLine ) );
    paint.drawLine( 3, 2, width() - 3, 2 );
    paint.drawLine( 5, 6, width() - 3, 6 );
    paint.drawLine( 3, 10, width() - 3, 10 );
    paint.setPen( QPen( group.light(), 2, Qt::DotLine ) );
    paint.drawLine( 4, 3, width() - 3, 3 );
    paint.drawLine( 6, 7, width() - 3, 7 );
    paint.drawLine( 4, 11, width() - 3, 11 );
}
///////////////////////////////////////
//
//      TabGrip::mousePressEvent
//
///////////////////////////////////////
void TabGrip::mousePressEvent( QMouseEvent *event )
{
    event->accept();
    if(event->button() == Qt::LeftButton)
    {
        couldDrag = true;
        offset = mapToGlobal( event->pos() );
        offset.setX( topLevelWidget()->x() - offset.x() );
        offset.setY( topLevelWidget()->y() - offset.y() );
    }
}
///////////////////////////////////////
//
//      TabGrip::mouseMoveEvent
//
///////////////////////////////////////
void TabGrip::mouseMoveEvent( QMouseEvent *event )
{
    /*
        By default, MouseMoveEvent is never called unless the user has
        a button held down, so this should only be called if we're dragging.
    */
    event->accept();
    if( couldDrag == true )
    {
        /* Dragging Page */
        isDragging = true;
        couldDrag = false;
        QApplication::setOverrideCursor( Qt::SizeAllCursor );
    }
}
///////////////////////////////////////
//
//      TabGrip::mouseReleaseEvent
//
///////////////////////////////////////
void TabGrip::mouseReleaseEvent( QMouseEvent *event )
{
    event->accept();
    if(event->button() == Qt::LeftButton)
    {
        couldDrag = false;
        if( isDragging )
        {
            QApplication::restoreOverrideCursor();
            isDragging = false;
            emit wasDragged( event->globalPos(), offset );
        }
    }
}
