/***************************************************************************
                          tabpage.cpp  -  description
                             -------------------
    begin                : Fri Sep 13 2002
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tabpage.moc"
#include "tabmanager.h"
#include "tabgrip.h"
#include "resource.h"
#include <kiconloader.h>
#include <kdebug.h>
#include <klocale.h>
#include <qlayout.h>
#include <qtoolbutton.h>
#include <qmetaobject.h>

///////////////////////////////////////
//
//      TabPage::constructor
//
///////////////////////////////////////
TabPage::TabPage( QWidget *parent, QWidget *child, resource *rsrc ) : QVBox(parent)
{
    myResource = rsrc;
    myChild = child;
    myMenu = NULL;
    myMenuCaption = "";

    /* Create ActionBar */
    actionBar = new QHBox( this );
    actionBar->setSizePolicy( QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed ) );
    actionBar->show();

    grip = new TabGrip( actionBar );
    connect( grip, SIGNAL( wasDragged(const QPoint&, const QPoint&) ), this, SLOT( tabDragged(const QPoint&, const QPoint&) ) );

    KIconLoader icons( QString( "knights" ) );
    QPixmap map = icons.loadIcon( QString("tab_remove"), KIcon::Small, 0, KIcon::DefaultState, 0, true );
    if( map.isNull() )
    {
        /* Keep for backward compatability with KDE 3.0 */
        map = icons.loadIcon( QString("fileclose"), KIcon::Small );
    }

    /* Create close button */
    closeButton = new QToolButton( actionBar, "closeButton" );
    closeButton->setIconSet( QIconSet( map ) );
    closeButton->setAutoRaise( true );
    closeButton->setTextLabel( i18n( "Close This Tab" ), true );
    closeButton->setSizePolicy( QSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed ) );
    connect( closeButton, SIGNAL( clicked() ), this, SIGNAL( requestDestruction() ) );

    /* Create ChildFeatures Object */
    childFeatures.ContextMenu = false;
    childFeatures.ReqPermissionForClose = false;
    childFeatures.EnableSave = false;
    childFeatures.EnableSaveAs = false;
    childFeatures.EnablePrint = false;
    /* Check for extended features in child widget */
    QMetaObject *childMeta = child->metaObject();
    QStrList strlist = childMeta->signalNames( true );
    if( strlist.find( "updateWidgetFeatureList(const WFL&)" ) != -1 )
    {
      connect( child, SIGNAL( updateWidgetFeatureList(const WFL&) ), this, SLOT( recieveWidgetFeatureList(const WFL&) ) );
      connect( this, SIGNAL( requestWidgetFeatureList() ), child, SLOT( requestWidgetFeatureList() ) );
      emit( requestWidgetFeatureList() );
    }

    myChild->reparent( this, QPoint( 0, 0 ), true );
    myChild->show();
    setMinimumSize( 256, 256 );
    show();
}
///////////////////////////////////////
//
//      TabPage::destructor
//
///////////////////////////////////////
TabPage::~TabPage()
{
}
///////////////////////////////////////
//
//      TabPage::recieveWidgetFeatureList
//
///////////////////////////////////////
void TabPage::recieveWidgetFeatureList( const WFL &newList )
{
	childFeatures = newList;
	disconnect( this, SLOT( fetchMenu( KPopupMenu*, QString& ) ) );

  if( childFeatures.ContextMenu == true )
  {
    /* Fetch the menu for this widget */
    connect( myChild, SIGNAL( fetchMenu( KPopupMenu*, QString& ) ), this, SLOT( fetchMenu( KPopupMenu*, QString& ) ) );
  }
  else
  {
    myMenu = NULL;
    myMenuCaption = "";
  }
  emit recievedWidgetFeatureList( newList );
}
///////////////////////////////////////
//
//      TabPage::tabDragged
//
///////////////////////////////////////
void TabPage::tabDragged( const QPoint &dest, const QPoint &offset )
{
    TabBox *myParent = parentTabBox();
    if( myParent != NULL )
    {
        QWidget *destWidget = QApplication::widgetAt( dest, true );
        /* Find a TabBox */
        while(1)
        {
            if( destWidget == NULL )
            {
                /* Create new TabBox */
                TabBox *newBox = new TabBox( myResource );
                newBox->resize( myParent->size() );
                newBox->move( dest + offset );
                newBox->show();

                /* Remove myself from the old Box */
                emit newParent( newBox );
                myParent->removeTab( this );

                /* Add myself to the new one */
                newBox->addTab( this, myCaption );
                myResource->tabManager->updateRelations( myChild, myParent, -1 );
                break;
            }
            if( QString( destWidget->className() ) == "TabBox" )
            {
                if( myParent != ((TabBox*)destWidget) )
                {
                    /* We can latch on here */
                    myParent->removeTab( this );
                    myResource->tabManager->updateRelations( myChild, myParent, -1 );
                    myResource->tabManager->updateRelations( myChild, (TabBox*)destWidget, 1 );
                    ((TabBox*)destWidget)->addTab( this, myCaption );
                    break;
                }
                else
                {
                    /* Didn't really drag it anywhere, so return */
                    break;
                }
            }
            destWidget = destWidget->parentWidget();
        }
        if( !myParent->count() )
        {
          /* Parent Box has no children. Send a signal to the manager that it's dead, then
             we delete it. ( Because TabManager will NOT delete it ) */
            QCustomEvent *e = new QCustomEvent( EVENT_Close_Tab );
            e->setData( myParent );
            QApplication::sendEvent( myResource->tabManager, e );
            delete myParent;
        }
    }
    else
    {
        kdError() << "TabPage::tabDragged: Can not move without a parent TabBox." << endl;
    }
}
///////////////////////////////////////
//
//      TabPage::setCaption
//
///////////////////////////////////////
void TabPage::setCaption( const QString &caption )
{
    myCaption = caption;
}
///////////////////////////////////////
//
//      TabPage::parentTabBox
//
///////////////////////////////////////
TabBox* TabPage::parentTabBox( void )
{
    QWidget *myParent = this->parentWidget();
    if( QString( myParent->className() ) == "QWidgetStack" )
    {
        myParent = myParent->parentWidget();
        if( QString( myParent->className() ) == "QTabWidget" )
        {
            myParent = myParent->parentWidget();
            if( QString( myParent->className() ) == "TabBox" )
            {
                return ((TabBox*)myParent);
            }
        }
    }
    kdWarning() << "TabPage::parentTabBox: Returning NULL parent" << endl;
    return NULL;
}
///////////////////////////////////////
//
//      TabPage::getChild
//
///////////////////////////////////////
QWidget* TabPage::getChild( void )
{
    return myChild;
}
///////////////////////////////////////
//
//      TabPage::closeChild
//
///////////////////////////////////////
bool TabPage::closeChild( void )
{
    /* Some children need to give permission to close */
    if( childFeatures.ReqPermissionForClose && !myResource->ShutdownMode )
    {
        bool status;
        connect( this, SIGNAL( requestClose( bool& ) ), myChild, SIGNAL( requestClose( bool& ) ) );
        emit requestClose( status );
        disconnect( this, SIGNAL( requestClose( bool& ) ), 0, 0 );
        return status;
    }
    return true;
}
///////////////////////////////////////
//
//  TabPage::fetchMenu
//
///////////////////////////////////////
void TabPage::fetchMenu( KPopupMenu *menu, QString &caption )
{
  myMenu = menu;
  myMenuCaption = caption;
    // myMenuBar->changeContextMenu( menu, caption );
}
///////////////////////////////////////
//
//  TabPage::recvCMD
//
///////////////////////////////////////
void TabPage::recvCMD( const Command &cmd )
{
  if( childFeatures.ContextMenu == true )
  {
    connect( this, SIGNAL( menuActionInternal( const Command& ) ), myChild, SLOT( recvCMD( const Command& ) ) );
    emit menuActionInternal( cmd );
    disconnect( this, SIGNAL( menuActionInternal( const Command& ) ), 0, 0 );
  }
  else
  {
    kdError() << "TabPage::recvCMD: Command " << cmd.getCommand() << " unrecognised." << endl;
  }
}

