/***************************************************************************
                          tabpage.h  -  description
                             -------------------
    begin                : Fri Sep 13 2002
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TABPAGE_H
#define TABPAGE_H

#include <qframe.h>
#include <qwidget.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qstring.h>
#include "tabbox.h"
#include "definitions.h"

/**
  *@author Troy Corbin Jr.

  This is the container Widget for any object to be placed as a tab in the tab system.
  No need to inherit this class... but for your object to interact with the tab system
  you need to know how to communicate with it.
  */
class resource;
class QToolButton;
class TabGrip;
class KPopupMenu;

class TabPage : public QVBox
{
    Q_OBJECT

private:
    QString      myCaption;
    QHBox        *actionBar;
    TabGrip      *grip;
    QToolButton  *closeButton;

    QWidget      *myChild;
    resource     *myResource;

    WFL          childFeatures;
    KPopupMenu   *myMenu;
    QString      myMenuCaption;

public:
    TabPage( QWidget *parent, QWidget *child, resource *rsrc );
    ~TabPage();
    void      setCaption( const QString& );
    QWidget*  getChild( void );
    bool      closeChild( void );
    void      recvCMD( const Command& );
    bool      containsQuitAction( void )
    {
      return childFeatures.ReqPermissionForClose;
    }
    KPopupMenu*  getMenu( void )
    {
      return myMenu;
    }
    QString      getMenuCaption( void )
    {
      return myMenuCaption;
    }
    const WFL&   getWidgetFeatureList( void )
    {
      return childFeatures;
    }

protected slots:
    void      tabDragged( const QPoint&, const QPoint& );
    void      recieveWidgetFeatureList( const WFL& );
    void      fetchMenu( KPopupMenu*, QString& );

signals:
    void      newParent( TabBox* );

    /** For internal use only. Used to connect to children that support Widget Feature Lists */
    void      requestWidgetFeatureList( void );

    /** This signal connects with the parent TabBox to request that we be deleted. */
    void      requestDestruction( void );

    /** For internal use only. This signal connects with the child to make sure it's ok for us to be destroyed. */
    void      requestClose( bool& );

    /** For internal use only. Never connect to this yourself. */
    void      menuActionInternal( const Command& );

    /** For use only by TabBox. Sends a copy of the Widget Feature List upstream */
    void      recievedWidgetFeatureList( const WFL& );

protected:
    TabBox*   parentTabBox( void );
};

#endif
