/***************************************************************************
                          wiz_setup.cpp  -  description
                             -------------------
    begin                : Wed Nov 21 2001
    copyright            : (C) 2003 by Troy Corbin Jr.
    email                : tcorbin@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kglobalsettings.h>
#include <kprocess.h>
#include <kstddirs.h>
#include <qstringlist.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qregexp.h>
#include <kurl.h>
#include <kio/netaccess.h>
#include "wiz_setup.moc"
#include "definitions.h"
#include "server_base.h"

wiz_setup::wiz_setup(QWidget *parent, const char *name, resource *Rsrc ) : KWizard(parent,name,true)
{
    myResource = Rsrc;
    steps = 0;
    setMaximumSize( 800, 600 );

    /*
                Call Order Of Pages:

        Page 1 - Introduction
        Page 3 - Setup Engines
        Page 4 - Setup Servers
        Page 2 - Setup Mimetypes
        Page 5 - Finished
    */
    initPage1();
    if( myResource->Config_Version < 1 )
    {
        initPage3();
        initPage4();
    }
    if( myResource->Config_Version < 2 )
    {
        initPage2();
    }
    initPage5();
    setCaption( i18n( "Knights Setup Wizard" ) );

    connect( this, SIGNAL( selected(const QString&) ), this, SLOT( slot_Work(const QString&) ) );
}
wiz_setup::~wiz_setup()
{}
///////////////////////////////////////
//
//      wiz_setup::initPage1
//
///////////////////////////////////////
void wiz_setup::initPage1( void )
{
    Page1 = new QFrame( this );
    P1B1 = new QGridLayout( Page1, 1, 4, 8 );

    SideImage1 = new QLabel( Page1 );
    SideImage1->setPixmap( locate("appdata", "wizard.png" ) );
    P1B1->addWidget( SideImage1, 1, 1 );

    WelcomeMessage = new QLabel( Page1 );
    WelcomeMessage->setAlignment( Qt::AlignLeft | Qt::AlignTop | Qt::WordBreak );
    WelcomeMessage->setText( i18n( "Thank you for installing Knights, your chess resource for KDE. To help you get started quickly, there are a few things Knights will need to setup. You should click 'Next' to see what they are." ) );
    P1B1->addWidget( WelcomeMessage, 1, 3 );

    addPage( Page1, i18n("Welcome to Knights v%1!").arg(_VERSION_) );
    setNextEnabled( Page1, true );
    setHelpEnabled( Page1, false );
}
///////////////////////////////////////
//
//      wiz_setup::initPage2
//
///////////////////////////////////////
void wiz_setup::initPage2( void )
{
    Page2 = new QFrame( this );
    P2B1 = new QGridLayout( Page2, 2, 4, 8 );

    SideImage2 = new QLabel( Page2 );
    SideImage2->setPixmap( locate("appdata", "wizard.png" ) );
    P2B1->addMultiCellWidget( SideImage2, 1, 2, 1, 1 );

    pgnExplain = new QLabel( Page2 );
    pgnExplain->setAlignment( Qt::AlignLeft | Qt::AlignTop | Qt::WordBreak );
    pgnExplain->setText( i18n( "When Knights saves a match to disk, it uses Portable Game Notation (PGN) to store the match. Several other computer chess games use PGN too. If you like, Knights can be your default PGN viewer." ) );
    P2B1->addWidget( pgnExplain, 1, 3 );

    pgnButtons = new QButtonGroup( 2, Qt::Vertical, i18n( "Let Knights handle PGN files?" ), Page2 );
    pgnYes = new QRadioButton( i18n( "Yes" ), pgnButtons );
    pgnNo = new QRadioButton( i18n( "No" ), pgnButtons );
    pgnButtons->setButton( 0 );
    P2B1->addWidget( pgnButtons, 2, 3 );

    addPage( Page2, i18n("Let Knights handle your .pgn files?") );
    setHelpEnabled( Page2, false );
}
///////////////////////////////////////
//
//      wiz_setup::initPage3
//
///////////////////////////////////////
void wiz_setup::initPage3( void )
{
    Page3 = new QFrame( this );
    P3B1 = new QGridLayout( Page3, 2, 4, 8 );

    SideImage3 = new QLabel( Page3 );
    SideImage3->setPixmap( locate("appdata", "wizard.png" ) );
    P3B1->addMultiCellWidget( SideImage3, 1, 2, 1, 1 );

    engineExplain = new QLabel( Page3 );
    engineExplain->setAlignment( Qt::AlignLeft | Qt::AlignTop | Qt::WordBreak );
    engineExplain->setText( i18n( "Knights can not play a chess match against you without help. Known as Chess Engines, these helpers can be found on the Internet and are often already installed on your system. Knights can look for Chess Engines that you may already have. If found, Knights will configure itself to use them." ) );
    P3B1->addWidget( engineExplain, 1, 3 );

    engineButtons = new QButtonGroup( 2, Qt::Vertical, i18n( "Let Knights search for chess engines?" ), Page3 );
    engineYes = new QRadioButton( i18n( "Yes" ), engineButtons );
    engineNo = new QRadioButton( i18n( "No" ), engineButtons );
    engineButtons->setButton( 0 );
    P3B1->addWidget( engineButtons, 2, 3 );

    addPage( Page3, i18n("Let Knights search for chess engines?") );
    setHelpEnabled( Page3, false );
}
///////////////////////////////////////
//
//      wiz_setup::initPage4
//
///////////////////////////////////////
void wiz_setup::initPage4( void )
{
    Page4 = new QFrame( this );
    P4B1 = new QGridLayout( Page4, 2, 4, 8 );

    SideImage4 = new QLabel( Page4 );
    SideImage4->setPixmap( locate("appdata", "wizard.png" ) );
    P4B1->addMultiCellWidget( SideImage4, 1, 2, 1, 1 );

    serverExplain = new QLabel( Page4 );
    serverExplain->setAlignment( Qt::AlignLeft | Qt::AlignTop | Qt::WordBreak );
    serverExplain->setText( i18n( "In order for you to play chess online, Knights will have to connect to a chess service. Knights already has a list of several of these services. If you like, this list can be used to automatically configure Knights." ) );
    P4B1->addWidget( serverExplain, 1, 3 );

    serverButtons = new QButtonGroup( 2, Qt::Vertical, i18n( "Let Knights configure online services?" ), Page4 );
    serverYes = new QRadioButton( i18n( "Yes" ), serverButtons );
    serverNo = new QRadioButton( i18n( "No" ), serverButtons );
    serverButtons->setButton( 0 );
    P4B1->addWidget( serverButtons, 2, 3 );

    addPage( Page4, i18n("Let Knights configure online services?") );
    setHelpEnabled( Page4, false );
}
///////////////////////////////////////
//
//      wiz_setup::initPage5
//
///////////////////////////////////////
void wiz_setup::initPage5( void )
{
    Page5 = new QFrame( this );
    P5B1 = new QGridLayout( Page5, 2, 4, 8 );

    SideImage5 = new QLabel( Page5 );
    SideImage5->setPixmap( locate("appdata", "wizard.png" ) );
    P5B1->addMultiCellWidget( SideImage5, 1, 2, 1, 1 );

    FinishExplain = new QLabel( Page5 );
    FinishExplain->setAlignment( Qt::AlignLeft | Qt::AlignTop | Qt::WordBreak );
    FinishExplain->setText( i18n( "Knights has now configured itself according to your choices. We strongly recommend that you read the documentation to make full use of all that Knights has to offer. You can access it at any time by pressing F1." ) );
    P5B1->addWidget( FinishExplain, 1, 3 );

		openHelp = new QCheckBox( i18n( "View documentation." ), Page5 );
		openHelp->setChecked( true );
		P5B1->addWidget( openHelp, 2, 3 );

    addPage( Page5, i18n("Setup Complete") );
    setFinish( Page5, true );
    setHelpEnabled( Page5, false );
}
///////////////////////////////////////
//
//      wiz_setup::slot_Work
//
///////////////////////////////////////
void wiz_setup::slot_Work( const QString& pageTitle )
{
    /* Don't proceed unless we're ready to work */
    if( pageTitle != title( Page5 ) )
		{
        return;
		}
    setBackEnabled( Page5, false );
    if( myResource->Config_Version < 1 )
    {
        if( engineButtons->selected() == engineYes )
            findEngines();
        if( serverButtons->selected() == serverYes )
            setupServers();
    }
    if( myResource->Config_Version < 2 )
    {
        if( pgnButtons->selected() == pgnYes )
            setupPGN();
    }
    myResource->ConfigWrite();
    setFinishEnabled( Page5, true );

		connect( finishButton(), SIGNAL( clicked() ), this, SLOT( slot_OpenHelp() ) );
}
///////////////////////////////////////
//
//      wiz_setup::setupPGN
//
///////////////////////////////////////
void wiz_setup::setupPGN( void )
{
    KURL origDesktop, newDesktop;
    origDesktop.setPath( locate( "appdata", "knights.desktop" ) );
    newDesktop.setPath( locateLocal( "apps", "Games/Board/knights.desktop" ) );

    kdWarning() << "Copying: " << origDesktop.path() << " to " << newDesktop.path() << endl;

    KIO::NetAccess::copy( origDesktop, newDesktop, NULL );
}
///////////////////////////////////////
//
//      wiz_setup::setupServers
//
///////////////////////////////////////
void wiz_setup::setupServers( void )
{
    serverResource *newServer;
    QStringList names, urls, ports;
    unsigned int loop;

    names << i18n("Free Internet Chess Server") << i18n("Internet Chess Club") << i18n("Global Chess Server") << i18n("Chess.Net");
    names << i18n("Chess-Square.Com") << i18n("Australian FICS") << i18n("Brazilian FICS") << i18n("Chilean FICS");
    names << i18n("Croatian FICS") << i18n("Dutch FICS") << i18n("French FICS") << i18n("German FICS") << i18n("Swedish FICS");
    urls << "freechess.org" << "chessclub.com" << "global.chessparlor.com" << "chess.net";
    urls << "chess-square.com" << "zics.org" << "jogo.cex.org.br" << "ajedrez.cec.uchile.cl";
    urls << "fly.cc.fer.hr" << "freechess.nl" << "jeu.echecs.com" << "chess.unix-ag.uni-kl.de" << "chess.mds.mdh.se";
    ports << "5000" << "5000" << "6000" << "5000" << "5000" << "5000" << "5000" << "5000";
    ports << "7890" << "5000" << "5000" << "5000" << "5000";

    for( loop = 0; loop < names.count(); loop++ )
    {
        kdDebug() << "adding " << names[loop] << endl;
        newServer = new serverResource;
        newServer->Name = names[loop];
        newServer->URL = urls[loop];
        newServer->Port = ports[loop].toInt();
        newServer->LogFile = QString::null;
        newServer->UserName = QString::null;
        newServer->Password = QString::null;
        newServer->StorePass = false;
        if(newServer->Name == "Internet Chess Club")
        {
            newServer->ServerType = server_base::ICC;
        }
        else
        {
            newServer->ServerType = server_base::FICS;
        }
        if( !loop )
            newServer->CurrentRef = true;
        else
            newServer->CurrentRef = false;
        myResource->servers.append( *newServer );
        delete newServer;
    }
}
///////////////////////////////////////
//
//      wiz_setup::findEngines
//
///////////////////////////////////////
void wiz_setup::findEngines( void )
{
    engineResource *newEngine;
    QStringList names, filenames;
    KProcess *Shell;
    bool foundOne( false );
    bool gnuChessV4( false );
    unsigned int loop;

    names << "GNUChess v4" << "GNUChess v5" << "Crafty" << "Sjeng" << "KnightCap" << "BabyChess" << "Phalanx-XXII";
    names << "Bowron Abernethy Chess Engine" << "Cilian" << "Faile" << "OliThink" << "Pepito";
    names << "Yace" << "Slibo";
    filenames << "gnuchessx" << "gnuchess" << "crafty" << "sjeng" << "KnightCap" << "babychess-xboard-engine";
    filenames << "phalanx" << "BACE" << "linuxCilian.exe" << "faile" << "olithink" << "pepito";
    filenames << "yace" << "sliboengine";

    for( loop = 0; loop < filenames.count(); loop++ )
    {
        Shell = new KProcess();
        (*Shell) << filenames[loop];
        if( Shell->start() )
        {
            if( filenames[loop] == "gnuchessx" )
            {
                /* GNUChess v4 Detected */
                gnuChessV4 = true;
            }
            if( ( filenames[loop] == "gnuchess" ) && gnuChessV4 )
            {
                /* With GNUChess v4 installed, this is probably not v5 */
            }
            else
            {
                /* Add engine to the list */
                kdWarning() << "Adding " << names[loop] << " to knightsrc" << endl;
                newEngine = new engineResource;
                newEngine->Name = names[loop];
                newEngine->Filename = filenames[loop];
                newEngine->Arguments = QString::null;
                newEngine->LogFile = QString::null;
                newEngine->Protocol = XBoard;
                newEngine->Wins = 0;
                newEngine->Losses = 0;
                newEngine->Draws = 0;

                /* Set the first engine found as the default engine */
                if( foundOne )
                {
                    newEngine->CurrentRef = 0;
                }
                else
                {
                    newEngine->CurrentRef = ENGINE_WHITE | ENGINE_BLACK | ENGINE_WHITE_BK | ENGINE_BLACK_BK;
                }

                myResource->engines.append( *newEngine );
                delete newEngine;
                foundOne = true;
            }
        }
        Shell->kill();
        delete Shell;
    }
}
///////////////////////////////////////
//
//      wiz_setup::slot_OpenHelp
//
///////////////////////////////////////
void wiz_setup::slot_OpenHelp( void )
{
	if( openHelp->isChecked() )
	{
		KApplication::kApplication()->invokeHelp();
	}
}
